!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program spinbuttons

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_adjustment_new, &
                  gtk_adjustment_get_upper, &
                  gtk_adjustment_set_value, &
                  gtk_adjustment_set_lower, &
                  gtk_adjustment_set_upper, &
                  gtk_adjustment_set_step_increment, &
                  gtk_adjustment_set_page_increment, &
                  gtk_adjustment_set_page_size, &
                  gtk_spin_button_new, &
                  gtk_vbox_new, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  TRUE, FALSE

   implicit none

   integer :: i
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
   end interface

   type(c_ptr) :: window, spin_integer, spin_real, vbox
   type(c_ptr) :: int_ptr, real_ptr

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Spin Buttons'//CNULL)
   call gtk_container_set_border_width(window, 10)
   call gtk_widget_set_size_request(window, 250, 100)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   ! create two new adjustments; the first spans between 0 and 10,
   ! starting at 5 and moves in increments of 1
   ! the second spans between 0 and 1, starting at 0.5 and
   ! moves in increments of 0.1
   !                                 value      lower       upper
   int_ptr  = gtk_adjustment_new(dble(5.0), dble(0.0), dble(10.0), &
   !                                  step       page   page size
                                 dble(1.0), dble(2.0), dble( 0.0)  )

   real_ptr = gtk_adjustment_new(dble(0.5), dble(0.0), dble( 1.0), &
                                 dble(0.1), dble(0.5), dble( 0.0)  )

   call gtk_adjustment_set_value(int_ptr,dble( 5.0))
   call gtk_adjustment_set_lower(int_ptr,dble( 0.0))
   call gtk_adjustment_set_upper(int_ptr,dble(10.0))
   call gtk_adjustment_set_step_increment(int_ptr,dble(1.0))
   call gtk_adjustment_set_page_increment(int_ptr,dble(2.0))
   call gtk_adjustment_set_page_size(int_ptr,dble(0.0))

   ! create two new spin buttons; the first will display no decimal
   ! places and the second will display two decimal places
   spin_integer = gtk_spin_button_new(int_ptr,  dble(1.0), 0)
   spin_real    = gtk_spin_button_new(real_ptr, dble(0.1), 2)

   vbox = gtk_vbox_new(FALSE,5)
   call gtk_container_set_border_width(vbox, 10)
   call gtk_box_pack_start(vbox, spin_integer, TRUE, TRUE, 0)
   call gtk_box_pack_start(vbox, spin_real,    TRUE, TRUE, 0)

   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end program

