!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program colorbuttons

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_color_button_new_with_color, &
                  gtk_color_button_set_title, &
                  gtk_label_new, &
                  gtk_widget_modify_fg, &
                  GTK_STATE_NORMAL, &
                  gtk_hbox_new, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  TRUE, FALSE

   use gdk, only: gdk_color_parse, &
                  gdk_color_to_string

   implicit none

   integer :: i
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
     subroutine color_changed(button,label) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button, label
     end subroutine
   end interface

   type(c_ptr) :: window, button, label, hbox
   integer(c_int) :: ic

   type, bind(c) :: GdkColor
     integer(kind=c_int32_t) :: pixel = 0
     integer(kind=c_int16_t) :: red   = 0
     integer(kind=c_int16_t) :: green = 0
     integer(kind=c_int16_t) :: blue  = 0
   end type GdkColor

   type(GdkColor), target :: color

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Color Button'//CNULL)
   call gtk_container_set_border_width(window, 10)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   ! set the initial color as #ffb20c and set the dialog title
   ic = gdk_color_parse('#ffb20c'//CNULL,c_loc(color))

   ! gtk_color_button_new_with_color has been deprecated since
   ! version 3.4 and should not be used in newly-written code.
   ! Use gtk_color_button_new_with_rgba() instead.

   button = gtk_color_button_new_with_color(c_loc(color))
   call gtk_color_button_set_title(button, 'Select a Color'//CNULL)

   label = gtk_label_new('Look at my color!'//CNULL)
   call gtk_widget_modify_fg(label,GTK_STATE_NORMAL,c_loc(color))

   call g_signal_connect(button, 'color_set'//CNULL, &
                         c_funloc(color_changed), label)

   hbox = gtk_hbox_new(FALSE, 5)
   call gtk_container_set_border_width(hbox, 10)
   call gtk_box_pack_start(hbox, button, TRUE, TRUE, 0)
   call gtk_box_pack_start(hbox, label,  TRUE, TRUE, 0)

   call gtk_container_add(window, hbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end program
subroutine color_changed(button, label) bind(C)
!
!  Retrieve the selected color and set it as the GtkLabel's
!  foreground color
!
   use iso_c_binding

   use gtk, only: gtk_color_button_get_color, &
                  gtk_widget_modify_fg, &
                  GTK_STATE_NORMAL

   type(c_ptr), value :: button, label

   type, bind(c) :: GdkColor
     integer(kind=c_int32_t) :: pixel = 0
     integer(kind=c_int16_t) :: red   = 0
     integer(kind=c_int16_t) :: green = 0
     integer(kind=c_int16_t) :: blue  = 0
   end type GdkColor

   type(GdkColor), target :: color

   integer :: convert_ushort_to_int, red,green,blue

   ! gtk_color_button_get_color has been deprecated since
   ! version 3.4 and should not be used in newly-written code.
   ! Use gtk_color_chooser_get_rgba() instead.

   call gtk_color_button_get_color(button,c_loc(color))
   call gtk_widget_modify_fg(label, GTK_STATE_NORMAL, c_loc(color))

   write(*,*)
   red   = convert_ushort_to_int(color%red)
   green = convert_ushort_to_int(color%green)
   blue  = convert_ushort_to_int(color%blue)
   write(*,*)
   write(*,*)'color>',red,green,blue
   write(*,*)'     >',int(red/256.0),int(green/256.0),int(blue/256.0)
   write(*,'(1x,''     >'',3(10x,z2.2))') &
                      int(red/256.0),int(green/256.0),int(blue/256.0)

end subroutine color_changed
function convert_ushort_to_int( ushort )

   use iso_c_binding

   integer :: convert_ushort_to_int
   integer, parameter   :: N = 16

   integer(C_SHORT), intent(in)   :: ushort

   integer, dimension(N) :: bits(0:N-1)

   bits = 0
   do i=0,N-1
     if( btest(ushort,i) ) bits(i) = 1
   end do
   write(*,'('' ushort bits>'',$)')
   do i=N-1,0,-1
     write(*,'(i1,$)') bits(i)
     if( mod(i,4) == 0 .and. i > 0 ) write(*,'('' '',$)')
   end do
   write(*,'(''<'')')

   ! check highest bit
   if( btest(ushort,N-1) )then
     ibig = 32768
   else
     ibig =     0
   endif
   ! clear highest bit
   iconv = ibclr(ushort,N-1)
   ! simple type conversion:

   convert_ushort_to_int = iconv + ibig

end function convert_ushort_to_int


