!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2013
!
program scrolledwindows

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  gtk_window_set_resizable, &
                  g_signal_connect, &
                  gtk_table_new, &
                  gtk_table_set_row_spacings, &
                  gtk_table_set_col_spacings, &
                  gtk_table_attach_defaults, &
                  gtk_button_new_from_stock, &
                  gtk_button_new_with_label, &
                  gtk_button_set_relief, &
                  gtk_scrolled_window_new, &
                  gtk_scrolled_window_get_hadjustment, &
                  gtk_scrolled_window_get_vadjustment, &
                  gtk_viewport_new, &
                  gtk_scrolled_window_set_policy, &
                  gtk_scrolled_window_add_with_viewport, &
                  gtk_container_add, &
                  gtk_vbox_new, &
                  gtk_box_get_homogeneous, &
                  gtk_box_set_homogeneous, &
                  gtk_box_pack_start_defaults, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtk_main_quit, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_POLICY_AUTOMATIC, &
                  GTK_POLICY_ALWAYS, &
                  GTK_RELIEF_NONE, &
                  GTK_RELIEF_HALF, &
                  GTK_RELIEF_NORMAL, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE

   implicit none

   type(c_ptr)    :: window, swin, viewport, table1, table2, vbox
   type(c_ptr)    :: horizontal, vertical
   type(c_ptr)    :: buttons1(10,10), buttons2(10,10)

   
   integer(c_int) :: i, j
  
   character(len=1), parameter :: CNULL = C_NULL_CHAR
   character(len=*), parameter :: GTK_STOCK_CLOSE = 'gtk-close'//CNULL

   character(len=7) :: string

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, &
                             'Scrolled Windows & Viewports'//CNULL)
   call gtk_container_set_border_width(window,10)
   call gtk_widget_set_size_request(window, 400, 300)
   call gtk_window_set_resizable(window,gtkFALSE)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)
  
   table1 = gtk_table_new(10, 10, gtkTRUE)
   table2 = gtk_table_new(10, 10, gtkTRUE)
  !table1 = gtk_grid_new()
  !table2 = gtk_grid_new()
   call gtk_table_set_row_spacings(table1, 1)
   call gtk_table_set_row_spacings(table2, 1)
   call gtk_table_set_col_spacings(table1, 1)
   call gtk_table_set_col_spacings(table2, 1)
  !call gtk_grid_set_row_spacing(table1, 1)
  !call gtk_grid_set_row_spacing(table2, 1)
  !call gtk_grid_set_column_spacing(table1, 1)
  !call gtk_grid_set_column_spacing(table2, 1)

   ! pack each table with 100 buttons
   do j=1,10
     do i=1,10
      !buttons1(i,j) = gtk_button_new_from_stock(GTK_STOCK_CLOSE)
      !buttons2(i,j) = gtk_button_new_from_stock(GTK_STOCK_CLOSE)

       write(string,'(i1,1x,i2,'','',i2)') 1,i,j
       buttons1(i,j) = gtk_button_new_with_label(string)
       write(string,'(i1,1x,i2,'','',i2)') 2,i,j
       buttons2(i,j) = gtk_button_new_with_label(string)

       call gtk_button_set_relief(buttons1(i,j), GTK_RELIEF_HALF)
       call gtk_button_set_relief(buttons2(i,j), GTK_RELIEF_HALF)
      
      !call gtk_table_attach_defaults(table1, buttons1(i,j),i,i+1,j,j+1)
      !     gtk_table_attach_defaults(table,widget,left,right,top,bottom)
      ! vs  gtk_grid_attach          (grid,child,  left,top,width,height) 
      !
      ! adds a widget to the grid, the position of child is determined 
      ! by left and top. The number of 'cells' that child will occupy 
      ! is determined by width and height.
      ! left  : the column number to attach the left side of child to
      ! top   : the row number to attach the top side of child to
      ! width : the number of columns that child will span
      ! height:the number of rows that child will span
      !
      ! i,j rij,col
       
      !call gtk_grid_attach(table1,buttons1(i,j), j,i,1,1)
      !call gtk_grid_attach(table2,buttons2(i,j), j,i,1,1)
       call gtk_table_attach_defaults(table1,buttons1(i,j), j-1,j,i-1,i)
       call gtk_table_attach_defaults(table2,buttons2(i,j), j-1,j,i-1,i)
    
       call g_signal_connect(buttons1(i,j), 'clicked'//CNULL, &
                                     c_funloc(gtk_main_quit), &
                                     C_NULL_PTR)
       call g_signal_connect(buttons2(i,j), 'clicked'//CNULL, &
                                     c_funloc(gtk_main_quit), &
                                     C_NULL_PTR)
    enddo
   enddo

   !
   ! create a scrolled window and a viewport, each with one table. use the
   ! adjustments in the scrolled window to synchronize both containers
   !
   swin = gtk_scrolled_window_new(C_NULL_PTR, C_NULL_PTR)
  
   horizontal = gtk_scrolled_window_get_hadjustment(swin)
   vertical   = gtk_scrolled_window_get_vadjustment(swin)

   viewport   = gtk_viewport_new(horizontal, vertical)
  
   call gtk_container_set_border_width(swin, 5)
   call gtk_container_set_border_width(viewport, 5)

   ! the policy determines when the scrollbar should appear
  !call gtk_scrolled_window_set_policy(swin,     &
  !                        GTK_POLICY_AUTOMATIC, & ! policy for horizontal bar
  !                        GTK_POLICY_AUTOMATIC)   ! policy for vertical bar
   call gtk_scrolled_window_set_policy(swin,     &
                           GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS)  

   ! add child without native scrolling capabilities:
   call gtk_scrolled_window_add_with_viewport(swin, table1) 
   call gtk_container_add(viewport, table2)        ! child has native scrolling
   !
   ! equivalent to add unscrollable child to a viewport, then add the 
   ! viewport to the scrolled window
   ! swin_viewport  = gtk_viewport_new(horizontal, vertical)
   ! call gtk_container_add(swin_viewport, table1)   
   ! call gtk_container_add(swin, swin_viewport)   
   ! call gtk_container_add(viewport, table2)
   !
   ! what's the difference between spacing (set when the box is created) and
   ! padding (set when elements are packed)? spacing is added between objects,
   ! and padding is added on either side of a widget.    
   !

   ! pack the widgets into a GtkVBox and then into the window
  !vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0)
   vbox = gtk_vbox_new(gtkTRUE, 5)
 
   call gtk_box_pack_start_defaults(vbox, viewport)
   call gtk_box_pack_start_defaults(vbox, swin)
  !call gtk_box_pack_start(vbox, viewport, gtkFALSE, gtkFALSE, 0)
  !call gtk_box_pack_start(vbox, swin,     gtkFALSE, gtkFALSE, 0)

   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end


