!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program helloworld2

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_label_new, &
                  gtk_label_set_selectable, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  GTKTRUE => TRUE

   implicit none

   !external my_destroy, delete_event
   interface
     subroutine my_destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine

     function my_delete_event(widget, event, gdata) bind(C)
       use iso_c_binding
       integer(c_int)     :: my_delete_event
       type(c_ptr), value :: widget, event, gdata
     end function my_delete_event
   end interface

   type(c_ptr) :: window, label

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Hello world'//C_NULL_CHAR)

   call gtk_container_set_border_width(window,10)
   call gtk_widget_set_size_request(window,200,100)

   ! connect the main window to the destroy and delete-event signals
   call g_signal_connect(window, 'destroy'//C_NULL_CHAR,      &
                         c_funloc(my_destroy))
   call g_signal_connect(window,'delete_event'//C_NULL_CHAR,  &
                         c_funloc(my_delete_event))

   ! create a new GtkLabel widget that is selectable
   label = gtk_label_new ('Hello World'//C_NULL_CHAR)
   call gtk_label_set_selectable(label,GTKTRUE)

   ! add the label as a child widget of the window
   call gtk_container_add (window,label)
   call gtk_widget_show_all(window)

   ! hand control over to the main loop
   call gtk_main()

end program helloworld2
function my_delete_event(widget, event, gdata) bind(C)
!
! static gboolean delete_event (GtkWidget*, GdkEvent*, gpointer)
!
! Note that events are a special type of signals, coming from the
! X Window system. Then callback functions must have an event argument.
!
   use iso_c_binding

   integer(c_int)     :: my_delete_event
   type(c_ptr), value :: widget, event, gdata

   write(*,*) 'a delete_event issued'

   my_delete_event = 0 !FALSE

end function my_delete_event
subroutine my_destroy(widget, gdata) bind(C)
!
! static void destroy (GtkWidget*, gpointer)
!
   use iso_c_binding

   use gtk, only: gtk_main_quit

   type(c_ptr), value :: widget, gdata

   write(*,*) 'subroutine destroy'
   call gtk_main_quit()

end subroutine my_destroy

