!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program boxes

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  gtk_widget_set_sensitive, &
                  gtk_widget_set_visible, &
                  gtk_widget_set_style, &
                  gtk_box_new, &
                  g_signal_connect, &
                  gtk_button_new_with_mnemonic, &
                  gtk_button_new_with_label, &
                  gtk_box_pack_start, &
                  gtk_button_set_relief, &
                  GTK_RELIEF_NONE, &
                  GTK_RELIEF_HALF, &
                  GTK_RELIEF_NORMAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  gtk_widget_destroy, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtkTRUE => TRUE, &
                  gtkFALSE => FALSE

   implicit none

   integer :: i
   character(len=1) :: CNULL = C_NULL_CHAR

   integer, parameter :: NUM_NAMES = 4
   character(len=12), dimension(NUM_NAMES) :: names

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
   end interface

   type(c_ptr)    :: window, vbox, button

   names(1) = 'Andrew'//CNULL
   names(2) = 'Joe'//CNULL
   names(3) = 'Samantha'//CNULL
   names(4) = 'Jonathan'//CNULL

   call gtk_init()

   window = gtk_window_new (GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Buttons'//CNULL)
   call gtk_container_set_border_width(window, 10)
   call gtk_widget_set_size_request(window, 150, -1)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   ! note: GtkVBox has been deprecated in gtk3
   ! function gtk_box_new(orientation, spacing)
   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5)

   ! add four buttons to the vertical box
   do i=1,NUM_NAMES
     button = gtk_button_new_with_label(names(i))

     !                                       T/F    T/F
     !    gtk_box_pack_start(box,  child,  expand,  fill,    padding)
     call gtk_box_pack_start(vbox, button, gtkTRUE, gtkTRUE, 10)

     ! connect the button to the clicked signal. the callback
     ! function recieves the window followed by the
     ! button because the arguments are swapped.
     call g_signal_connect_swapped(button, 'clicked'//CNULL, &
                                   c_funloc(destroy),      &
                                   button)
     if( i == 2 )then
       call gtk_widget_set_sensitive(button, gtkFALSE)
!     elseif( i == 3 )then
!       call gtk_widget_set_visible(button, gtkTRUE)
!     elseif( i == 4 )then
!       call gtk_widget_set_style(button, ...)
     endif
   enddo

   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end program

