!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program panes

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  gtk_paned_new, &
                  gtk_paned_add1, &
                  gtk_paned_add2, &
                  gtk_paned_pack1, &
                  gtk_paned_pack2, &
                  gtk_paned_set_position, &
                  g_signal_connect, &
                  gtk_button_new_with_label, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  gtk_widget_destroy, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtkTRUE => TRUE, &
                  gtkFALSE => FALSE

   implicit none

   integer :: i
   character(len=1) :: CNULL = C_NULL_CHAR

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
   end interface

   type(c_ptr)    :: window, paned, button1, button2

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Panes'//CNULL)
   call gtk_container_set_border_width(window, 10)
   call gtk_widget_set_size_request(window, 225, 150)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   ! note: hpaned has been deprecated in gtk3
   ! gtk_paned_new(orientation);
   paned   = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL)
   button1 = gtk_button_new_with_label('Resize'//CNULL)
   button2 = gtk_button_new_with_label('Me!!'//CNULL)

   ! connect the button to the clicked signal. the callback
   ! function recieves the window followed by the
   ! button because the arguments are swapped.
   call g_signal_connect_swapped(button1, 'clicked'//CNULL, &
                                 c_funloc(destroy),         &
                                 window)

   call g_signal_connect_swapped(button2, 'clicked'//CNULL, &
                                 c_funloc(destroy),         &
                                 window)

   ! pack both buttons as the two children of the GtkPaned widget
  !call gtk_paned_add1(paned, button1)
  !call gtk_paned_add2(paned, button2)
   call gtk_paned_pack1(paned, button1, gtkFALSE, gtkTRUE)
   call gtk_paned_pack2(paned, button2, gtkTRUE,  gtkFALSE)

   call gtk_paned_set_position(paned, 120)

   call gtk_container_add(window, paned)
   call gtk_widget_show_all(window)

   call gtk_main()

end program

