!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program handleboxes

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_notebook_new, &
                  gtk_label_new, &
                  gtk_button_new_with_label, &
                  gtk_notebook_append_page, &
                  gtk_notebook_set_tab_pos, GTK_POS_BOTTOM, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  TRUE, FALSE

   implicit none

   integer :: i
   character(len=1) :: CNULL = C_NULL_CHAR

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
     subroutine switch_page(button, notebook) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button, notebook
     end subroutine
   end interface

   type(c_ptr) :: window, notebook
   type(c_ptr) :: label1, label2, child1, child2
   integer(c_int) ::  i1, i2

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Notebook'//CNULL)
   call gtk_container_set_border_width(window, 10)
   call gtk_widget_set_size_request(window, 250, 100)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   notebook = gtk_notebook_new()
   label1   = gtk_label_new('Page One'//CNULL)
   label2   = gtk_label_new('Page Two'//CNULL)
   child1   = gtk_button_new_with_label('Go to page 2'//CNULL)
   child2   = gtk_button_new_with_label('Go to page 1'//CNULL)

   ! notice that two widgets are connected to the
   ! same callback function!
   call g_signal_connect(child1, 'clicked'//CNULL, &
                         c_funloc(switch_page), notebook)
   call g_signal_connect(child2, 'clicked'//CNULL, &
                         c_funloc(switch_page), notebook)

   ! append to pages to the notebook container
   i1 = gtk_notebook_append_page(notebook, child1, label1)
   i2 = gtk_notebook_append_page(notebook, child2, label2)

   call gtk_notebook_set_tab_pos(notebook, GTK_POS_BOTTOM)

   call gtk_container_add(window, notebook)
   call gtk_widget_show_all(window)

   call gtk_main()

end program
subroutine switch_page(button, notebook) bind(C)
!
!  Switch between the current GtkNotebook page
!
   use iso_c_binding

   use gtk, only: gtk_notebook_get_current_page, &
                  gtk_notebook_set_current_page

   type(c_ptr), value :: button, notebook
   integer(c_int) :: page

   page = gtk_notebook_get_current_page(notebook)

   if( page == 0 )then
     write(*,*) 'gtk3: set current page to 1'
     call gtk_notebook_set_current_page(notebook, 1)
   else
     write(*,*) 'gtk3: set current page to 0'
     call gtk_notebook_set_current_page(notebook, 0)
   endif


end subroutine switch_page

