!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program handleboxes

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_event_box_new, &
                  gtk_label_new, &
                  gtk_event_box_set_above_child, &
                  gtk_widget_set_events, &
                  GDK_BUTTON_PRESS_MASK, GDK_HAND1, &
                  gtk_widget_realize, &
                  gtk_container_add, &
                  gtk_widget_get_window, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  TRUE, FALSE

   use gdk, only: gdk_window_set_cursor, gdk_cursor_new, gdk_window_get_cursor

   implicit none

   integer :: i
   character(len=1) :: CNULL = C_NULL_CHAR

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
     function button_pressed(eventbox, event, label) bind(C)
       use iso_c_binding
       integer(c_int)     :: button_pressed
       type(c_ptr), value :: event
       type(c_ptr), value :: eventbox, label
     end function button_pressed
   end interface

   type(c_ptr) :: window, eventbox, label, cursor
   type(c_ptr) :: active_area

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Event Box'//CNULL)
   call gtk_container_set_border_width(window, 10)
   call gtk_widget_set_size_request(window, 200, 50)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   eventbox = gtk_event_box_new()
   label    = gtk_label_new('Double-Click Me!'//CNULL)

   ! set the order in which widgets will receive
   ! notification of events
   call gtk_event_box_set_above_child(eventbox, FALSE)

   call g_signal_connect(eventbox, 'button_press_event'//CNULL, &
                         c_funloc(button_pressed),label)

   call gtk_container_add(eventbox, label)
   call gtk_container_add(window, eventbox)

   ! allow the event box to catch button presses, realize
   ! the widget, and set the cursor that will be displayed when
   ! the mouse is over the event box
   call gtk_widget_set_events(eventbox, GDK_BUTTON_PRESS_MASK)
   call gtk_widget_realize(eventbox)

   ! gdk_window_set_cursor(eventbox->window, gdk_cursor_new(GDK_HAND1))
   active_area = gtk_widget_get_window(eventbox)
   call gdk_window_set_cursor(active_area,gdk_cursor_new(GDK_HAND1))

   call gtk_widget_show_all(window)

   call gtk_main()

end program
function button_pressed(eventbox, event, label) bind(C)
!
!  This is called every time a button-press-event occurs
!  on the GtkEventBox
!
   use iso_c_binding

   use gtk, only: GDK_2BUTTON_PRESS, FALSE, &
                  gtk_label_get_text, gtk_label_set_text

   use gdk_events

   integer(c_int)        :: button_pressed
   type(c_ptr), value    :: event
   type(c_ptr), value    :: eventbox, label
   type(c_ptr)           :: c_string

   type(GdkEventButton), pointer :: eventbutton

   character(len=24) :: text

   call c_f_pointer(event,eventbutton)
   write(*,*)'button pressed ',eventbutton%type

   ! member 'type' of struct 'GdkEventButton'
   if( eventbutton%type == GDK_2BUTTON_PRESS )then
     write(*,*)'button pressed twice'
     c_string = gtk_label_get_text(label)
     call c_f_string(c_string,text)

     if( text(1:1) == 'D')then
       call gtk_label_set_text(label, 'I Was Double-Clicked!'//C_NULL_CHAR)
     else
       call gtk_label_set_text(label, 'Double-Click Me Again!'//C_NULL_CHAR)
     endif
   endif

   button_pressed = FALSE

end function button_pressed

