!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program checkbuttons

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_check_button_new_with_label, &
                  gtk_widget_set_sensitive, &
                  gtk_button_new_from_stock, &
                  gtk_box_new, &
                  GTK_ORIENTATION_VERTICAL, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  TRUE, FALSE

   implicit none

   integer :: i
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   character(len=*), parameter :: GTK_STOCK_CLOSE = 'gtk-close'//CNULL

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
     subroutine check_toggled(check1,check2) bind(C)
       use iso_c_binding
       type(c_ptr), value :: check1, check2
     end subroutine
   end interface

   type(c_ptr) :: window, vbox, check1, check2, closeit

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Check Buttons'//CNULL)
   call gtk_container_set_border_width(window, 10)
  !call gtk_widget_set_size_request(window, 250, 100)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   check1 = gtk_check_button_new_with_label('I am the main option.'//CNULL)
   check2 = gtk_check_button_new_with_label('I rely on the other guy.'//CNULL)

   ! only enable the second check button when the first is enabled
   call gtk_widget_set_sensitive(check2, FALSE)
   call g_signal_connect(check1, 'toggled'//CNULL, &
                         c_funloc(check_toggled), check2)

   closeit = gtk_button_new_from_stock(GTK_STOCK_CLOSE)
   call g_signal_connect_swapped(closeit, 'clicked'//CNULL, &
                                 c_funloc(gtk_widget_destroy), window)

   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5)
   call gtk_container_set_border_width(vbox, 10)
   call gtk_box_pack_start(vbox, check1,  FALSE, TRUE, 0)
   call gtk_box_pack_start(vbox, check2,  FALSE, TRUE, 0)
   call gtk_box_pack_start(vbox, closeit, FALSE, TRUE, 0)

   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end program
subroutine check_toggled(check1, check2) bind(c)
!
! if the main check button is active, enable the other
! otherwise, disable the supplementary check button
!
   use iso_c_binding

   use gtk, only: gtk_toggle_button_get_active, &
                  gtk_widget_set_sensitive, &
                  TRUE, FALSE

   type(c_ptr), value :: check1, check2

   if( gtk_toggle_button_get_active(check1) == TRUE )then
     call gtk_widget_set_sensitive(check2, TRUE)
   else
     call gtk_widget_set_sensitive(check2, FALSE)
   endif

end subroutine check_toggled

