!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program scales

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_scale_new_with_range, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  gtk_scale_set_digits, &
                  gtk_scale_set_value_pos, &
                  gtk_scale_add_mark, &
                  GTK_POS_RIGHT, GTK_POS_LEFT, &
                  GTK_POS_BOTTOM, GTK_POS_TOP, &
                  gtk_box_new, &
                  GTK_ORIENTATION_VERTICAL, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  TRUE, FALSE

   implicit none

   integer :: i
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
   end interface

   type(c_ptr) :: window, scale_integer, scale_real, vbox
   character(len=32) :: str1, str2, str3, str4, str5, str6

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Scales'//CNULL)
   call gtk_container_set_border_width(window, 10)
   call gtk_widget_set_size_request(window, 250, 100)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   ! create a scale that scrolls integers and one that
   ! scrolls reals
   scale_integer = gtk_scale_new_with_range &
                   (GTK_ORIENTATION_HORIZONTAL, &
                    dble(0.0), dble(10.0), dble(1.0))
   scale_real    = gtk_scale_new_with_range &
                   (GTK_ORIENTATION_HORIZONTAL, &
                    dble(0.0), dble(1.0), dble(0.1))

   ! set the number of decimal places to display for each widget
   call gtk_scale_set_digits(scale_integer, 0)
   call gtk_scale_set_digits(scale_real, 1)

   ! set the position of the value with respect to the widget
   call gtk_scale_set_value_pos(scale_integer, GTK_POS_RIGHT)
   call gtk_scale_set_value_pos(scale_real, GTK_POS_LEFT)

   ! set marks
   str1 = 'zip'//CNULL
   str2 = 'bit'//CNULL
   str3 = 'mid'//CNULL
   str4 = 'mid'//CNULL
   str5 = 'some'//CNULL
   str6 = 'full'//CNULL

   call gtk_scale_add_mark(scale_integer,dble( 0.0),GTK_POS_TOP,str1)
   call gtk_scale_add_mark(scale_integer,dble( 2.0),GTK_POS_TOP,str2)
   call gtk_scale_add_mark(scale_integer,dble( 4.0),GTK_POS_TOP,str3)
   call gtk_scale_add_mark(scale_integer,dble( 6.0),GTK_POS_TOP,str4)
   call gtk_scale_add_mark(scale_integer,dble( 8.0),GTK_POS_TOP,str5)
   call gtk_scale_add_mark(scale_integer,dble(10.0),GTK_POS_TOP,str6)

   call gtk_scale_add_mark(scale_real,dble(0.00),GTK_POS_BOTTOM,CNULL)
   call gtk_scale_add_mark(scale_real,dble(0.25),GTK_POS_BOTTOM,CNULL)
   call gtk_scale_add_mark(scale_real,dble(0.50),GTK_POS_BOTTOM,CNULL)
   call gtk_scale_add_mark(scale_real,dble(0.75),GTK_POS_BOTTOM,CNULL)
   call gtk_scale_add_mark(scale_real,dble(1.00),GTK_POS_BOTTOM,CNULL)

   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5)
   call gtk_container_set_border_width(vbox, 10)
   call gtk_box_pack_start(vbox, scale_integer, TRUE, TRUE, 0)
   call gtk_box_pack_start(vbox, scale_real,    TRUE, TRUE, 0)

   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end program

