!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program colorbuttons

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_color_button_new_with_rgba, &
                  gtk_color_button_set_title, &
                  gtk_label_new, &
                  gtk_widget_modify_fg, &
                  GTK_STATE_NORMAL, &
                  gtk_box_new, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  TRUE, FALSE

   use gdk, only: gdk_color_parse, &
                  gdk_color_to_string

   implicit none

   integer :: i
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
     subroutine color_changed(button,label) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button, label
     end subroutine
   end interface

   type(c_ptr) :: window, button, label, hbox
   integer(c_int) :: ic

   type, bind(c) :: GdkColor
     integer(kind=c_int32_t) :: pixel = 0
     integer(kind=c_int16_t) :: red   = 0
     integer(kind=c_int16_t) :: green = 0
     integer(kind=c_int16_t) :: blue  = 0
   end type GdkColor

   type, bind(c) :: GdkRGBA
     real(kind=c_double) :: red   = 0.0
     real(kind=c_double) :: green = 0.0
     real(kind=c_double) :: blue  = 0.0
     real(kind=c_double) :: alpha = 0.0
   end type GdkRGBA

   type(GdkColor), target :: color_ints
   type(GdkRGBA), target  :: color_rgba

   integer        :: convert_ushort_to_int, ired,igreen,iblue
   real           :: red, green, blue
   real(c_double) :: col_min = 0.0, col_max = 1.0
 
   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Color Button'//CNULL)
   call gtk_container_set_border_width(window, 10)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   ! set the initial color as #003366 and set the dialog title
   !gboolean = gdk_color_parse(gchar *spec,GdkColor *color)
   ic = gdk_color_parse('#ffb20c'//CNULL,c_loc(color_ints))

   ired   = convert_ushort_to_int(color_ints%red)
   igreen = convert_ushort_to_int(color_ints%green)
   iblue  = convert_ushort_to_int(color_ints%blue)

   color_rgba%red   = dmax1(dmin1(dble(  ired/65536.0),col_max),col_min)
   color_rgba%green = dmax1(dmin1(dble(igreen/65536.0),col_max),col_min)
   color_rgba%blue  = dmax1(dmin1(dble( iblue/65536.0),col_max),col_min)

   write(*,'(1x,A,3(3x,f5.3,4x))') 'rgba >', &
                        color_rgba%red,color_rgba%green,color_rgba%blue

   ! gtk_color_button_new_with_color has been deprecated since
   ! version 3.4 and should not be used in newly-written code.
   ! Use gtk_color_button_new_with_rgba() instead.

   button = gtk_color_button_new_with_rgba(c_loc(color_rgba))
   call gtk_color_button_set_title(button, 'Select a Color'//CNULL)

   label = gtk_label_new('Look at my color!'//CNULL)
   call gtk_widget_modify_fg(label,GTK_STATE_NORMAL,c_loc(color_ints))

   call g_signal_connect(button, 'color_set'//CNULL, &
                         c_funloc(color_changed), label)

   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5)
   call gtk_container_set_border_width(hbox, 10)
   call gtk_box_pack_start(hbox, button, TRUE, TRUE, 0)
   call gtk_box_pack_start(hbox, label,  TRUE, TRUE, 0)

   call gtk_container_add(window, hbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end program
subroutine color_changed(button, label) bind(C)
!
!  Retrieve the selected color and set it as the GtkLabel's
!  foreground color
!
   use iso_c_binding

   use gtk, only: gtk_color_chooser_get_rgba, &
                  gtk_widget_modify_fg, &
                  GTK_STATE_NORMAL

   implicit none

   type(c_ptr), value :: button, label

   type, bind(c) :: GdkColor
     integer(kind=c_int32_t) :: pixel = 0
     integer(kind=c_int16_t) :: red   = 0
     integer(kind=c_int16_t) :: green = 0
     integer(kind=c_int16_t) :: blue  = 0
   end type GdkColor

   type, bind(c) :: GdkRGBA
     real(kind=c_double) :: red   = 0.0
     real(kind=c_double) :: green = 0.0
     real(kind=c_double) :: blue  = 0.0
     real(kind=c_double) :: alpha = 0.0
   end type GdkRGBA

   type(GdkColor), target :: color_ints
   type(GdkRGBA), target  :: color_rgba

   integer :: convert_ushort_to_int, ired,igreen,iblue
   real    :: red, green, blue

   ! gtk_color_button_get_color has been deprecated since
   ! version 3.4 and should not be used in newly-written code.
   ! Use gtk_color_chooser_get_rgba() instead.

   call gtk_color_chooser_get_rgba(button,c_loc(color_rgba))

   red   = sngl(color_rgba%red)
   green = sngl(color_rgba%green)
   blue  = sngl(color_rgba%blue)

   color_ints%red   = int2(  red*65535)
   color_ints%green = int2(green*65535)
   color_ints%blue  = int2( blue*65535)

   call gtk_widget_modify_fg(label, GTK_STATE_NORMAL, c_loc(color_ints))

   write(*,1) red,int(red*255.), green,int(green*255.), blue,int(blue*255.)
 1 format(1x,'color>',3(3x,f5.3,1x,i3.3))

end subroutine color_changed
function convert_ushort_to_int( ushort )

   use iso_c_binding

   integer :: convert_ushort_to_int
   integer, parameter   :: N = 16

   integer(C_SHORT), intent(in) :: ushort

   integer, dimension(N) :: bits(0:N-1)

   bits = 0
   do i=0,N-1
     if( btest(ushort,i) ) bits(i) = 1
   end do
   write(*,'('' ushort bits>'')',advance='No')
   do i=N-1,0,-1
     write(*,'(i1)',advance='No') bits(i)
     if( mod(i,4) == 0 .and. i > 0 ) write(*,'('' '')',advance='No')
   end do
   write(*,'(''<'')')

   ! check highest bit
   if( btest(ushort,N-1) )then
     ibig = 32768
   else
     ibig =     0
   endif
   ! clear highest bit
   iconv = ibclr(ushort,N-1)
   ! simple type conversion:

   convert_ushort_to_int = iconv + ibig

end function convert_ushort_to_int

