!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program filechooserbuttons

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_label_new, &
                  gtk_file_chooser_button_new, &
                  GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, &
                  GTK_FILE_CHOOSER_ACTION_OPEN, &
                  gtk_file_chooser_set_current_folder, &
                  gtk_file_filter_new, &
                  gtk_file_filter_set_name, &
                  gtk_file_filter_add_pattern, &
                  gtk_file_chooser_add_filter, &
                  gtk_box_new, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtkTRUE => TRUE, &
                  gtkFALSE => FALSE

   use g, only:   g_get_home_dir

   implicit none

   integer :: i
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
     subroutine color_changed(button,label) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button, label
     end subroutine
     subroutine folder_changed(chooser1,chooser2) bind(C)
       use iso_c_binding
       type(c_ptr), value :: chooser1, chooser2
     end subroutine
     subroutine file_changed(chooser2,label) bind(C)
       use iso_c_binding
       type(c_ptr), value :: chooser2, label
     end subroutine
   end interface

   type(c_ptr) :: window, chooser1, chooser2, label, vbox
   type(c_ptr) :: c_string, filter1, filter2

   integer(c_int) :: i1, i2
   character(len=128) :: folder

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'File Cooser Button'//CNULL)
   call gtk_container_set_border_width(window, 10)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   label = gtk_label_new(''//CNULL)

   ! create two buttons, one to select a folder and one to select a file
   chooser1 = gtk_file_chooser_button_new('Chooser a Folder'//CNULL, &
                              GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)
   chooser2 = gtk_file_chooser_button_new('Chooser a Folder'//CNULL, &
                              GTK_FILE_CHOOSER_ACTION_OPEN)

   ! monitor when the selected folder or file are changed
   call g_signal_connect(chooser1, 'selection_changed'//CNULL, &
                              c_funloc(folder_changed), chooser2)
   call g_signal_connect(chooser2, 'selection_changed'//CNULL, &
                              c_funloc(file_changed), label)

   ! set both file chooser buttons to the location of the
   ! user's home directory

   c_string = g_get_home_dir()
   call c_f_string(c_string,folder)

   i1 = gtk_file_chooser_set_current_folder(chooser1, &
                             folder(1:len_trim(folder))//CNULL)
   i1 = gtk_file_chooser_set_current_folder(chooser1, &
                             folder(1:len_trim(folder))//CNULL)

   ! provide a filter to show all files and one to shown only
   ! three types of images
   filter1 = gtk_file_filter_new()
   filter2 = gtk_file_filter_new()

   call gtk_file_filter_set_name(filter1, 'Image Files'//CNULL)
   call gtk_file_filter_set_name(filter2, 'All Files'//CNULL)
   call gtk_file_filter_add_pattern(filter1, '*.png'//CNULL)
   call gtk_file_filter_add_pattern(filter1, '*.jpg'//CNULL)
   call gtk_file_filter_add_pattern(filter1, '*.gif'//CNULL)
   call gtk_file_filter_add_pattern(filter2, '*'//CNULL)

   ! add the both filters to the file chooser button that selects files
   call gtk_file_chooser_add_filter(chooser2, filter1)
   call gtk_file_chooser_add_filter(chooser2, filter2)

   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5)
   call gtk_container_set_border_width(vbox, 10)
   call gtk_box_pack_start(vbox, chooser1, gtkTRUE, gtkTRUE, 0)
   call gtk_box_pack_start(vbox, chooser2, gtkTRUE, gtkTRUE, 0)
   call gtk_box_pack_start(vbox, label, gtkTRUE, gtkTRUE, 0)

   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end program
subroutine folder_changed(chooser1, chooser2) bind(C)
!
! when a folder is selected, use that as the new location
! of the other chooser
!
   use iso_c_binding

   use gtk, only: gtk_file_chooser_get_filename, &
                  gtk_file_chooser_set_current_folder

   implicit none

   type(c_ptr), value :: chooser1, chooser2

   type(c_ptr)        :: c_string
   character(len=128) :: folder

   integer(c_int) :: i

   c_string = gtk_file_chooser_get_filename(chooser1)
   call c_f_string(c_string,folder)

   i = gtk_file_chooser_set_current_folder(chooser2, &
                         folder(1:len_trim(folder))//C_NULL_CHAR)

   write(*,*) 'folder> ',folder(1:len_trim(folder))

end subroutine folder_changed
subroutine file_changed(chooser2, label) bind(C)
!
! when a folder is selected, use that as the new location
! of the other chooser
!
   use iso_c_binding

   use gtk, only: gtk_file_chooser_get_filename, &
                  gtk_label_set_text

   implicit none

   type(c_ptr), value :: chooser2, label

   type(c_ptr)        :: c_string
   character(len=128) :: filename
   integer            :: is

   c_string = gtk_file_chooser_get_filename(chooser2)
   call c_f_string(c_string,filename)

   is = len_trim(filename)
   call gtk_label_set_text(label, filename(1:is)//C_NULL_CHAR)

   if( is > 0 )then
     write(*,*) 'file  > ',filename(1:is)
   else
     write(*,*) 'file  > (empty)'
   endif

end subroutine file_changed


