!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program filechooserbuttons

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_label_new, &
                  gtk_widget_modify_font, &
                  gtk_font_button_new_with_font, &
                  gtk_font_button_set_title, &
                  gtk_box_new, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  TRUE, FALSE

   use pango, only: pango_font_description_from_string

   implicit none

   integer :: i
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
     subroutine font_changed(button,label) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button, label
     end subroutine
   end interface

   type(c_ptr) :: window, button, label, vbox
   type(c_ptr) :: initial_font

   integer(c_int) :: i1, i2
   character(len=128) :: folder

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Font Button'//CNULL)
   call gtk_container_set_border_width(window, 10)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   label = gtk_label_new('Look at the font!'//CNULL)

   initial_font = pango_font_description_from_string &
                                               ('Sans Bold 12'//CNULL)
   call gtk_widget_modify_font(label, initial_font)

   ! create a new font selection button with the given default font
   button = gtk_font_button_new_with_font('Sans Bold 12'//CNULL)
   call gtk_font_button_set_title(button, 'Choose a Font'//CNULL)

   ! monitor for changes to the font chosen in the font button
   call g_signal_connect(button, 'font_set'//CNULL, &
                         c_funloc(font_changed), label)

   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5)
   call gtk_container_set_border_width(vbox, 10)
   call gtk_box_pack_start(vbox, button, TRUE, TRUE, 0)
   call gtk_box_pack_start(vbox, label,  TRUE, TRUE, 0)

   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end program
subroutine font_changed(button, label) bind(C)
!
! when the font is changed, display the font both as the text
! of a label and as the label's physical font
!
   use iso_c_binding

   use gtk, only:   gtk_font_button_get_font_name, &
                    gtk_label_set_text, &
                    gtk_widget_modify_font

   use g, only:     g_snprintf

   use pango, only: pango_font_description_from_string

   implicit none

   type(c_ptr), value :: button, label
   type(c_ptr)        :: desc

   type(c_ptr)        :: c_string
   character(len=512) :: font, buffer

   integer(c_int) :: i

   c_string = gtk_font_button_get_font_name(button)
   call c_f_string(c_string,font)

   desc = pango_font_description_from_string &
                           (font(1:len_trim(font))//C_NULL_CHAR)

   write(buffer,'(''Font: '',A)') font(1:len_trim(font))
   call gtk_label_set_text(label, buffer(1:len_trim(buffer))//C_NULL_CHAR)

   call gtk_widget_modify_font(label, desc)

   write(*,*) 'font> ',font(1:len_trim(font))

end subroutine font_changed

