!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program dialogs

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_button_new_with_mnemonic, &
                  gtk_box_new, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  TRUE, FALSE

   implicit none

   integer :: i
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
     subroutine button_clicked(button,parent) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button, parent
     end subroutine
   end interface

   type(c_ptr) :: window, button

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Dialogs'//CNULL)
   call gtk_container_set_border_width(window, 10)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   button = gtk_button_new_with_mnemonic('_Click me'//C_NULL_CHAR)

   call g_signal_connect(button, 'clicked'//CNULL, &
                         c_funloc(button_clicked), window)

   call gtk_container_add(window, button)
   call gtk_widget_show_all(window)

   write(*,*) 'calling gtk_main()...'
   call gtk_main()

end program
subroutine button_clicked(button, parent) bind(C)

   use iso_c_binding

   use gtk, only: GTK_DIALOG_MODAL, &
                  GTK_RESPONSE_OK, &
                  gtk_label_new, &
                  gtk_image_new_from_stock, &
                  GTK_ICON_SIZE_DIALOG, &
                  gtk_box_new, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  gtk_container_set_border_width, &
                  gtk_dialog_get_content_area, &
                  gtk_box_get_type, &
                  gtk_dialog_get_type, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_dialog_run, &
                  gtk_dialog_get_response_for_widget, &
                  gtk_widget_destroy, &
                  TRUE, FALSE

   implicit none

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   type(c_ptr), value :: button, parent

   type(c_ptr)    :: dialog, content, label, image, hbox

   interface
     function gtk_dialog_new_with_buttons_f03(title,parent,flags, &
                                              button1, response1, &
                                              button2, response2)
       use iso_c_binding
       type(c_ptr) :: gtk_dialog_new_with_buttons_f03
       character(len=*), intent(in)  :: title
       type(c_ptr), intent(in)       :: parent
       integer(c_int), intent(in)    :: flags
       character(len=*), intent(in), optional :: button1, button2
       integer, intent(in), optional ::          response1, response2
     end function
   end interface

   character(len=*), parameter :: GTK_STOCK_OK = 'gtk-ok'//CNULL
   character(len=*), parameter :: GTK_STOCK_DIALOG_INFO = 'gtk-dialog-info'//CNULL

   integer(c_int) :: id

   write(*,*) 'button_clicked'

  !dialog = gtk_dialog_new_with_buttons('Information'//CNULL, parent, &
  !         GTK_DIALOG_MODAL, GTK_STOCK_OK, GTK_RESPONSE_OK, NULL())
   dialog = gtk_dialog_new_with_buttons_f03('Information', parent, &
                                            GTK_DIALOG_MODAL, &
                                            GTK_STOCK_OK, GTK_RESPONSE_OK )
   ! deprecated
  !call gtk_dialog_set_has_separator(dialog), FALSE)

   content = gtk_dialog_get_content_area(dialog)

   label = gtk_label_new('The button was clicked!'//CNULL)
   image = gtk_image_new_from_stock(GTK_STOCK_DIALOG_INFO, &
                                    GTK_ICON_SIZE_DIALOG)

   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0)
   call gtk_container_set_border_width(hbox, 10)
   call gtk_box_pack_start(content, hbox, TRUE, TRUE, 0)

   call gtk_box_pack_start(hbox, image, TRUE, TRUE, 0)
   call gtk_box_pack_start(hbox, label, TRUE, TRUE, 0)

   call gtk_widget_show_all(dialog)

   ! create the dialog as modal and destroy it when a button is pressed
   id = gtk_dialog_run(dialog)
   call gtk_dialog_print_response_f03(id)

   call gtk_widget_destroy(dialog)

   write(*,*) 'button_clicked done'

end subroutine button_clicked
function gtk_dialog_new_with_buttons_f03(title, parent, modal, &
                                         button1, response1,   &
                                         button2, response2)
   use iso_c_binding

   use gtk, only: gtk_dialog_new, &
                  gtk_label_new, &
                  gtk_window_set_title, &
                  gtk_window_set_modal, &
                  gtk_window_set_transient_for, &
                  gtk_window_set_destroy_with_parent, &
                  gtk_box_new, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  gtk_dialog_add_button, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  TRUE, FALSE

   type(c_ptr) :: gtk_dialog_new_with_buttons_f03
   type(c_ptr) :: dialog

   character(len=*), intent(in)           :: title
   type(c_ptr), intent(in), optional      :: parent
   integer(c_int), intent(in)             :: modal
   character(len=*), intent(in), optional :: button1, button2
   integer, intent(in), optional          :: response1, response2

   type(c_ptr) :: ib1, ib2, ib3

   dialog = gtk_dialog_new()
   call gtk_window_set_modal(dialog,modal)
   call gtk_window_set_title(dialog,title(1:len_trim(title))//C_NULL_CHAR)

   if( present(parent) )then
    !write(*,*) 'parent present'
     call gtk_window_set_transient_for(dialog, parent)
     call gtk_window_set_destroy_with_parent(dialog, TRUE)
   end if

   if( present(button1) )then
     if( present(response1) )then
       ib1 = gtk_dialog_add_button(dialog, &
                                   button1(1:len_trim(button1))//C_NULL_CHAR, &
                                   response1)
      !write(*,*) 'adding button 1',ib1
     else
       write(*,*) 'error: missing response1'
     endif
   endif

   gtk_dialog_new_with_buttons_f03 = dialog

end function gtk_dialog_new_with_buttons_f03
subroutine gtk_dialog_print_response_f03(id)

   use iso_c_binding, only: c_int

   use gtk, only: GTK_RESPONSE_NONE, &
                  GTK_RESPONSE_REJECT, &
                  GTK_RESPONSE_ACCEPT, &
                  GTK_RESPONSE_DELETE_EVENT, &
                  GTK_RESPONSE_OK, &
                  GTK_RESPONSE_CANCEL, &
                  GTK_RESPONSE_CLOSE, &
                  GTK_RESPONSE_YES, &
                  GTK_RESPONSE_NO, &
                  GTK_RESPONSE_APPLY, &
                  GTK_RESPONSE_HELP

   integer(c_int), intent(in) :: id

   select case(id)
     case(GTK_RESPONSE_NONE)
       write(*,*) 'No response'
     case(GTK_RESPONSE_REJECT)
       write(*,*) 'Rejected'
     case(GTK_RESPONSE_ACCEPT)
       write(*,*) 'Accepted'
     case(GTK_RESPONSE_DELETE_EVENT)
       write(*,*) 'Deleted'
     case(GTK_RESPONSE_OK)
       write(*,*) 'OK'
     case(GTK_RESPONSE_CANCEL)
       write(*,*) 'Cancelled'
     case(GTK_RESPONSE_CLOSE)
       write(*,*) 'Closed'
     case(GTK_RESPONSE_YES)
       write(*,*) 'Yep'
     case(GTK_RESPONSE_NO)
       write(*,*) 'Nope'
     case(GTK_RESPONSE_APPLY)
       write(*,*) 'Applied'
     case(GTK_RESPONSE_HELP)
       write(*,*) 'Help!'
     case default
       write(*,*) 'Unknown response'
   end select

end subroutine gtk_dialog_print_response_f03

