!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program dialogs3

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_dialog_set_default_response, &
                  gtk_dialog_get_content_area, &
                  GTK_DIALOG_MODAL, &
                  GTK_RESPONSE_OK, &
                  GTK_RESPONSE_CANCEL, &
                  gtk_label_new, &
                  gtk_entry_new, &
                  gtk_entry_set_text, &
                  gtk_entry_get_text, &
                  gtk_grid_new, &
                  gtk_grid_attach, &
                  gtk_grid_set_row_spacing, &
                  gtk_grid_set_column_spacing, &
                  gtk_main_quit, &
                  gtk_widget_destroy, &
                  gtk_box_new, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  gtk_box_pack_start, &
                  gtk_dialog_run, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  TRUE, FALSE

   use g, only :  g_get_user_name, &
                  g_get_real_name, &
                  g_get_home_dir, &
                  g_get_host_name

   implicit none

   integer :: i
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     function gtk_dialog_new_with_buttons_f03(title,parent,flags, &
                                              button1, response1, &
                                              button2, response2, &
                                              button3, response3)
       use iso_c_binding
       type(c_ptr) :: gtk_dialog_new_with_buttons_f03
       character(len=*), intent(in)  :: title
       type(c_ptr), intent(in)       :: parent
       integer(c_int), intent(in)    :: flags
       character(len=*), intent(in), optional :: button1, button2, button3
       integer, intent(in), optional ::          response1, response2, response3
     end function
   end interface

   type(c_ptr) :: dialog, table, user, irl, home, host
   type(c_ptr) :: content, lbl1, lbl2, lbl3, lbl4
   integer(c_int) :: response
   type(c_ptr), pointer :: parent => null()

   type(c_ptr) :: c_string
   character(len=32) :: f_string = ' '

   character(len=*), parameter :: GTK_STOCK_OK = 'gtk-ok'//CNULL
   character(len=*), parameter :: GTK_STOCK_DIALOG_INFO = 'gtk-dialog-info'//CNULL
   character(len=*), parameter :: GTK_STOCK_CLOSE  = 'gtk-close'//CNULL
   character(len=*), parameter :: GTK_STOCK_DELETE = 'gtk-delete'//CNULL
   character(len=*), parameter :: GTK_STOCK_CANCEL = 'gtk-cancel'//CNULL

   call gtk_init()

  !dialog = gtk_dialog_new_with_buttons('Information'//CNULL, parent, &
  !         GTK_DIALOG_MODAL, GTK_STOCK_OK, GTK_RESPONSE_OK, NULL())
   dialog = gtk_dialog_new_with_buttons_f03('Edit User Information',parent, &
                                  GTK_DIALOG_MODAL,  &
                                  GTK_STOCK_OK, GTK_RESPONSE_OK,   &
                                  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL )

   call gtk_dialog_set_default_response(dialog, GTK_RESPONSE_OK)

   ! create four entries that will tell the user what data to enter
   lbl1 = gtk_label_new('User Name:'//CNULL)
   lbl2 = gtk_label_new('Real Name:'//CNULL)
   lbl3 = gtk_label_new('Home Dir:'//CNULL)
   lbl4 = gtk_label_new('Host Name:'//CNULL)

   user = gtk_entry_new()
   irl  = gtk_entry_new()
   home = gtk_entry_new()
   host = gtk_entry_new()

   ! retrieve the user's information for the default values
   c_string = g_get_user_name()
   call c_f_string(c_string, f_string)
   call gtk_entry_set_text(user, f_string(1:len_trim(f_string))//CNULL)

   c_string = g_get_real_name()
   call c_f_string(c_string, f_string)
   call gtk_entry_set_text(irl, f_string(1:len_trim(f_string))//CNULL)

   c_string = g_get_home_dir()
   call c_f_string(c_string, f_string)
   call gtk_entry_set_text(home, f_string(1:len_trim(f_string))//CNULL)

   c_string = g_get_host_name()
   call c_f_string(c_string, f_string)
   call gtk_entry_set_text(host, f_string(1:len_trim(f_string))//CNULL)

   table = gtk_grid_new()

   ! left  : the column number to attach the left side of child to
   ! top   : the row number to attach the top side of child to
   ! width : the number of columns that child will span
   ! height: the number of rows that child will span
   !    gtk_grid_attach(grid,  child,  left, top, width, height) bind(c)
   call gtk_grid_attach(table, lbl1, 0, 0, 1, 1)
   call gtk_grid_attach(table, lbl2, 0, 1, 1, 1)
   call gtk_grid_attach(table, lbl3, 0, 2, 1, 1)
   call gtk_grid_attach(table, lbl4, 0, 3, 1, 1)
   call gtk_grid_attach(table, user, 1, 0, 1, 1)
   call gtk_grid_attach(table, irl , 1, 1, 1, 1)
   call gtk_grid_attach(table, home, 1, 2, 1, 1)
   call gtk_grid_attach(table, host, 1, 3, 1, 1)

   call gtk_grid_set_row_spacing(table, 5)
   call gtk_grid_set_column_spacing(table, 5)
   call gtk_container_set_border_width(table, 5)

   content = gtk_dialog_get_content_area(dialog)
   call gtk_box_pack_start(content, table, TRUE, TRUE, 0)

   call gtk_widget_show_all(dialog)

   response = gtk_dialog_run(dialog)
   if( response == GTK_RESPONSE_OK )then

     c_string = gtk_entry_get_text(user)
     call c_f_string(c_string, f_string)
     write(*,*) 'User name: ',f_string(1:len_trim(f_string))

     c_string = gtk_entry_get_text(irl)
     call c_f_string(c_string, f_string)
     write(*,*) 'Real name: ',f_string(1:len_trim(f_string))

     c_string = gtk_entry_get_text(home)
     call c_f_string(c_string, f_string)
     write(*,*) 'Home     : ',f_string(1:len_trim(f_string))

     c_string = gtk_entry_get_text(host)
     call c_f_string(c_string, f_string)
     write(*,*) 'Host     : ',f_string(1:len_trim(f_string))

   elseif( response == GTK_RESPONSE_CANCEL )then
     write(*,*) 'Cancel clicked'
   else
     write(*,*) 'error curious response', response
   endif

   call gtk_dialog_print_response_f03(response)

   call gtk_widget_destroy(dialog)

end program
function gtk_dialog_new_with_buttons_f03(title, parent, modal, &
                                         button1, response1,   &
                                         button2, response2,   &
                                         button3, response3)
   use iso_c_binding

   use gtk, only: gtk_dialog_new, &
                  GTK_DIALOG_MODAL, &
                  GTK_DIALOG_DESTROY_WITH_PARENT, &
                  gtk_label_new, &
                  gtk_window_set_title, &
                  gtk_window_set_modal, &
                  gtk_window_set_transient_for, &
                  gtk_window_set_destroy_with_parent, &
                  gtk_box_new, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  gtk_dialog_add_button, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  TRUE, FALSE

   type(c_ptr) :: gtk_dialog_new_with_buttons_f03
   type(c_ptr) :: dialog

   character(len=*), intent(in)           :: title
   type(c_ptr), intent(in), optional      :: parent
   integer(c_int), intent(in)             :: modal
   character(len=*), intent(in), optional :: button1, button2, button3
   integer, intent(in), optional          :: response1, response2, response3

   type(c_ptr) :: ib1, ib2, ib3

   dialog = gtk_dialog_new()

   ! if window is set modal, input will be grabbed when show and released when hide
   ! void gtk_window_set_modal (GtkWindow *window, gboolean modal);
   ! gboolean gtk_window_get_modal (GtkWindow *window);

   if( modal == GTK_DIALOG_MODAL )then
     !write(*,*)'modal set',modal
     call gtk_window_set_modal(dialog, TRUE)
   elseif( modal == GTK_DIALOG_DESTROY_WITH_PARENT )then
     !write(*,*)'modal false',modal
     call gtk_window_set_modal(dialog, FALSE)
   else
     write(*,*)'error: unknown modal',modal
     call gtk_window_set_modal(dialog, FALSE)
   endif

   call gtk_window_set_title(dialog, title(1:len_trim(title))//C_NULL_CHAR)

   if( present(parent)  )then
     write(*,*) 'parent present'
     call gtk_window_set_transient_for(dialog, parent)
     call gtk_window_set_destroy_with_parent(dialog, TRUE)
   endif

   if( present(button1) )then
     if( present(response1) )then
       ib1 = gtk_dialog_add_button(dialog, &
                                   button1(1:len_trim(button1))//C_NULL_CHAR, &
                                   response1)
     else
       write(*,*) 'error: missing response1'
     endif
   endif
   if( present(button2) )then
     if( present(response2) )then
       ib1 = gtk_dialog_add_button(dialog, &
                                   button2(1:len_trim(button2))//C_NULL_CHAR, &
                                   response2)
     else
       write(*,*) 'error: missing response2'
     endif
   endif
   if( present(button3) )then
     if( present(response3) )then
       ib1 = gtk_dialog_add_button(dialog, &
                                   button3(1:len_trim(button3))//C_NULL_CHAR, &
                                   response3)
     else
       write(*,*) 'error: missing response3'
     endif
   endif

   gtk_dialog_new_with_buttons_f03 = dialog

end function gtk_dialog_new_with_buttons_f03
subroutine gtk_dialog_print_response_f03(id)

   use iso_c_binding, only: c_int

   use gtk, only: GTK_RESPONSE_NONE, &
                  GTK_RESPONSE_REJECT, &
                  GTK_RESPONSE_ACCEPT, &
                  GTK_RESPONSE_DELETE_EVENT, &
                  GTK_RESPONSE_OK, &
                  GTK_RESPONSE_CANCEL, &
                  GTK_RESPONSE_CLOSE, &
                  GTK_RESPONSE_YES, &
                  GTK_RESPONSE_NO, &
                  GTK_RESPONSE_APPLY, &
                  GTK_RESPONSE_HELP

   integer(c_int), intent(in) :: id

   select case(id)
     case(GTK_RESPONSE_NONE)
       write(*,*) 'No response'
     case(GTK_RESPONSE_REJECT)
       write(*,*) 'Rejected'
     case(GTK_RESPONSE_ACCEPT)
       write(*,*) 'Accepted'
     case(GTK_RESPONSE_DELETE_EVENT)
       write(*,*) 'Deleted'
     case(GTK_RESPONSE_OK)
       write(*,*) 'OK'
     case(GTK_RESPONSE_CANCEL)
       write(*,*) 'Cancelled'
     case(GTK_RESPONSE_CLOSE)
       write(*,*) 'Closed'
     case(GTK_RESPONSE_YES)
       write(*,*) 'Yep'
     case(GTK_RESPONSE_NO)
       write(*,*) 'Nope'
     case(GTK_RESPONSE_APPLY)
       write(*,*) 'Applied'
     case(GTK_RESPONSE_HELP)
       write(*,*) 'Help!'
     case default
       write(*,*) 'Unknown response',id
   end select

end subroutine gtk_dialog_print_response_f03

