!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program aboutdialogs

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_about_dialog_new, &
                  gtk_about_dialog_set_logo, &
                  gtk_about_dialog_set_program_name, &
                  gtk_about_dialog_set_version, &
                  gtk_about_dialog_set_copyright, &
                  gtk_about_dialog_set_comments, &
                  gtk_about_dialog_set_license, &
                  gtk_about_dialog_set_website, &
                  gtk_about_dialog_set_website_label, &
                  gtk_about_dialog_set_authors, &
                  gtk_about_dialog_set_documenters, &
                  gtk_about_dialog_set_translator_credits, &
                  gtk_dialog_run, &
                  gtk_widget_destroy

   use gtk_os_dependent, only: gdk_pixbuf_new_from_file

   use gdk_pixbuf, only: gdk_pixbuf_error_quark      !#define GDK_PIXBUF_ERROR

   use g, only:   g_error_free, &
                  g_file_error_quark ! #define G_FILE_ERROR

   implicit none

   !struct GError {
   ! GQuark       domain ==> typedef guint32 GQuark = integer(c_int32_t)
   ! gint         code
   ! gchar       *message
   !}

   type, bind(c) :: GError
     integer(c_int) :: domain = 0
     integer(c_int) :: code   = 0
     type(c_ptr)    :: message
   end type GError

   interface
     subroutine f_c_string(f_string, c_string)
       use iso_c_binding, only: c_char, c_null_char
       character(len=*), intent(in) :: f_string
       character(kind=c_char), dimension(:), intent(out), allocatable :: c_string
     end subroutine
   end interface

   integer :: id
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   character(len=*), dimension(2), parameter :: authors = &
                       ['First author #1', 'Author #2      ']

   character(len=*), dimension(2), parameter :: documenters = &
                           ['Doc #1       ', 'Documenter #2']

   character(kind=c_char), dimension(:), target, allocatable :: &
                              c_string1, c_string2, c_string3
   type(c_ptr), dimension(3), target :: c_string_ptrs

   type(c_ptr) :: dialog, logo
   character(len=128) :: f_string

   type(GError), pointer :: error
   type(c_ptr), target :: error_ptr

   call gtk_init()

   dialog = gtk_about_dialog_new()

   error_ptr = c_null_ptr

   write(*,*) 'logo from file'
   logo = gdk_pixbuf_new_from_file('gtkbooklogo.png'//CNULL, &
                                                    c_loc(error_ptr))
   if( c_associated(logo) )then
     write(*,*) 'we''ve got him'
     call gtk_about_dialog_set_logo(dialog, logo)
   else
     call c_f_pointer(error_ptr, error)
     call c_f_string(error%message,f_string)
     if( error%domain == gdk_pixbuf_error_quark() )then
       write(*,*) 'GdkPixbufError: ', &
                             f_string(1:len_trim(f_string)),error%code
     elseif( error%domain == g_file_error_quark() )then
       write(*,*) 'GFileError: ', &
                             f_string(1:len_trim(f_string)),error%code
     else
       write(*,*) 'Unkown error: ',error%domain, &
                             f_string(1:len_trim(f_string)),error%code
     endif
     call g_error_free(error_ptr)
   endif

   call gtk_about_dialog_set_program_name(dialog, &
                                          'GtkAboutDialog'//CNULL)
   call gtk_about_dialog_set_version(dialog, '1.0 f03'//CNULL)
   call gtk_about_dialog_set_copyright(dialog, &
                                  '(C) 2007 Andrew Krause'//CNULL)
   call gtk_about_dialog_set_comments(dialog, &
                                'All About GtkAboutDialog'//CNULL)

   ! set the license text, which is usually loaded from a file
   ! also, set the web site address and label
   call gtk_about_dialog_set_license(dialog, &
      'Free to all!'//C_NEW_LINE//C_NEW_LINE// &
      'GTK+ 3.x see http://www.gtk.org/ and '//C_NEW_LINE// &
      'gtk-fortran see https://github.com/jerryd/gtk-fortran/wiki'//C_NEW_LINE// &
      'by Vincent Magnin, Jerry DeLisle, James Tappin, Jens Hunger et al.'//CNULL)
   call gtk_about_dialog_set_website(dialog, &
                     'http://www.apress.com/9781590597934'//CNULL)
   call gtk_about_dialog_set_website_label(dialog, &
                         'Foundations of GTK+ Development'//CNULL)

   ! set the application authors, documenters and translators
   call f_c_string(authors(1),c_string1)
   c_string_ptrs(1) = c_loc(c_string1)
   call f_c_string(authors(2),c_string2)
   c_string_ptrs(2) = c_loc(c_string2)
   call f_c_string('',c_string3)
   c_string_ptrs(3) = c_loc(c_string3)

   call gtk_about_dialog_set_authors(dialog,c_string_ptrs)

   deallocate(c_string1,stat=id)
   if( id /= 0 ) write(*,*)'error allocate 1'
   deallocate(c_string2,stat=id)
   if( id /= 0 ) write(*,*)'error allocate 2'
   deallocate(c_string3,stat=id)
   if( id /= 0 ) write(*,*)'error allocate 3'
   call f_c_string(documenters(1),c_string1)
   c_string_ptrs(1) = c_loc(c_string1)
   call f_c_string(documenters(2),c_string2)
   c_string_ptrs(2) = c_loc(c_string2)
   call f_c_string('',c_string3)
   c_string_ptrs(3) = c_loc(c_string3)

   call gtk_about_dialog_set_documenters(dialog, c_string_ptrs)
   call gtk_about_dialog_set_translator_credits(dialog, &
               'Translator #1'//C_NEW_LINE//'Translator #2'//CNULL)

   id = gtk_dialog_run(dialog)
   call gtk_widget_destroy(dialog)

end program

