!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program multiplefiles

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  g_signal_connect, &
                  gtk_main_quit, &
                  gtk_button_new_with_label, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main

   implicit none

   interface
     subroutine button_clicked(button,window) bind(c)
       use iso_c_binding
       type(c_ptr), value :: button, window
     end subroutine
   end interface

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   type(c_ptr)    :: window, button

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Open Files'//CNULL)
   call gtk_container_set_border_width(window, 10)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit))

   button = gtk_button_new_with_label('Open Files'//CNULL)

   call g_signal_connect(button, 'clicked'//CNULL, &
                         c_funloc(button_clicked), window)

   call gtk_container_add(window, button)
   call gtk_widget_show_all(window)

   call gtk_main()

end program
subroutine button_clicked(button,window) bind(c)

   use iso_c_binding

   use gtk, only: gtk_file_chooser_set_select_multiple, &
                  GTK_FILE_CHOOSER_ACTION_OPEN, &
                  gtk_dialog_run, &
                  GTK_RESPONSE_CANCEL, &
                  GTK_RESPONSE_OK, &
                  GTK_RESPONSE_ACCEPT, &
                  gtk_file_chooser_get_filenames, &
                  gtk_widget_destroy, &
                  gtkTRUE => TRUE

   use g, only:   g_slist_length, &
                  g_slist_nth_data, &
                  g_slist_free

   implicit none

   interface
!
!    GtkWidget *gtk_file_chooser_dialog_new(const gchar *title, 
!               GtkWindow *parent, GtkFileChooserAction action, 
!               const gchar *first_button_text, ...) G_GNUC_NULL_TERMINATED
!
     function gtk_file_chooser_dialog_new(title,parent,mode, &
              button1,response1,button2,response2,cnull) bind(c)
       use iso_c_binding
       type(c_ptr) :: gtk_file_chooser_dialog_new
       character(kind=c_char), dimension(*) :: title
       type(c_ptr), value                   :: parent
      !character(kind=c_char), value        :: parent
       integer(c_int), value                :: mode
       character(kind=c_char), dimension(*) :: button1, button2
       integer(c_int), value :: response1, response2
       character(kind=c_char), value        :: cnull
     end function
   end interface

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   type(c_ptr), value :: button, window
   type(c_ptr)    :: dialog
   integer(c_int) :: response

   type(c_ptr)  :: gslist_ptr
   type(c_ptr), dimension(:), allocatable :: name_ptrs
   character(len=128) :: filename 

   character(len=*), parameter :: GTK_STOCK_OK     = 'gtk-ok'//CNULL
   character(len=*), parameter :: GTK_STOCK_OPEN   = 'gtk-open'//CNULL
   character(len=*), parameter :: GTK_STOCK_CANCEL = 'gtk-cancel'//CNULL

   integer :: istat, i, n

   dialog = gtk_file_chooser_dialog_new('Open File(s) ...'//CNULL, &
                                        window, &
                                        GTK_FILE_CHOOSER_ACTION_OPEN, &
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, &
                                        GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, &
                                        C_NULL_CHAR)

   ! allow the user to choose more than one file at a time
   call gtk_file_chooser_set_select_multiple(dialog, gtkTRUE)

   response = gtk_dialog_run(dialog)
   if( response == GTK_RESPONSE_ACCEPT )then
     ! GSList * gtk_file_chooser_get_filenames (GtkFileChooser *chooser)
     ! function gtk_file_chooser_get_filenames(chooser) bind(c) 
     !   use iso_c_binding, only: c_ptr
     !   type(c_ptr) :: gtk_file_chooser_get_filenames
     !   type(c_ptr), value :: chooser
     ! end function
     gslist_ptr = gtk_file_chooser_get_filenames(dialog)

     n = g_slist_length(gslist_ptr)
     write(*,*)'number of file selected ',n

     allocate(name_ptrs(1:n),stat=istat)
     if( istat /= 0 ) write(*,*) 'error allocate',istat
     
     do i=1,n
       name_ptrs(i) = g_slist_nth_data(gslist_ptr,i-1)
     enddo    
     
     do i=1,n
       call c_f_string(name_ptrs(i),filename)
       write(*,*) 'Selected: ',trim(filename),i
     enddo

     call g_slist_free(gslist_ptr)
     deallocate(name_ptrs,stat=istat)
     if( istat /= 0 ) write(*,*) 'error deallocate',istat

   endif
  
   call gtk_widget_destroy(dialog)

end subroutine button_clicked

