!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program colorselection

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_button_new_with_label, &
                  gtk_main_quit, &
                  gtk_container_add, &
                  gtk_box_new, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  gtk_box_pack_start, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtkTRUE => TRUE

   use gdk, only: gdk_rgba_parse

   implicit none

   interface
     subroutine modal_clicked(button,window) bind(c)
       use iso_c_binding
       type(c_ptr), value :: button, window
     end subroutine
     subroutine nonmodal_clicked(button,window) bind(c)
       use iso_c_binding
       type(c_ptr), value :: button, window
     end subroutine
   end interface

   type, bind(c) :: GdkRGBA
     real(kind=c_double) :: red
     real(kind=c_double) :: green
     real(kind=c_double) :: blue
     real(kind=c_double) :: alpha
   end type GdkRGBA

   type(GdkRGBA),  target :: global_color_rgba

   common global_color_rgba

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   type(c_ptr)  :: window, hbox, modal, nonmodal
   integer(c_int) :: ic
   integer      :: i, narg, ired, igreen, iblue
   character(len=128) :: argument

   call gtk_init()

   global_color_rgba%red   =  0.0
   global_color_rgba%green =  0.0
   global_color_rgba%blue  =  0.0
   global_color_rgba%alpha = -1.0 ! use as not initialized flag

   Narg   = command_argument_count()

   do i=1,Narg
     call get_command_argument(i,argument)
     if( argument(1:1) == '-' .or. argument(1:1) == '/' )then
       cycle
     endif
     write(*,*) 'parsing >',trim(argument),'<'
     ic = gdk_rgba_parse(c_loc(global_color_rgba), &
                         argument(1:len_trim(argument))//CNULL)
     if( ic == gtkTRUE )then
       ired   = int(255.0*global_color_rgba%red)
       igreen = int(255.0*global_color_rgba%green)
       iblue  = int(255.0*global_color_rgba%blue)
       global_color_rgba%alpha = 1.0
       write(*,*) 'color found ',ired,igreen,iblue
       exit
     endif
   enddo

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Color Selection Dialogs'//CNULL)
   call gtk_container_set_border_width(window, 10)
   call gtk_widget_set_size_request(window, 200, 75)

   call g_signal_connect(window, 'destroy'//CNULL, c_funloc(gtk_main_quit))

   modal    = gtk_button_new_with_label('Modal'//CNULL)
   nonmodal = gtk_button_new_with_label('Non-Modal'//CNULL)

   call g_signal_connect(modal, 'clicked'//CNULL, &
                         c_funloc(modal_clicked), window)
   call g_signal_connect(nonmodal, 'clicked'//CNULL, &
                         c_funloc(nonmodal_clicked), window)

   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10)
   call gtk_box_pack_start(hbox, modal, gtkTRUE, gtkTRUE, 0)
   call gtk_box_pack_start(hbox, nonmodal, gtkTRUE, gtkTRUE, 0)

   call gtk_container_add(window, hbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end program
subroutine modal_clicked(button,window) bind(c)
!
! create a new color selection dialog that is modal
!
   use iso_c_binding

   use gtk, only: gtkTRUE => TRUE

   interface
     subroutine run_color_selection_dialog(button, window, domodal) bind(c)
       use iso_c_binding
       type(c_ptr), value :: button, window
       integer(c_int)    :: domodal
     end subroutine
   end interface

   type(c_ptr), value :: button, window

   call run_color_selection_dialog(button, window, gtkTRUE)

end subroutine modal_clicked
subroutine nonmodal_clicked(button,window) bind(c)
!
! create a new color selection dialog that is non-modal
!
   use iso_c_binding

   use gtk, only: gtkFALSE => FALSE

   interface
     subroutine run_color_selection_dialog(button, window, domodal) bind(c)
       use iso_c_binding
       type(c_ptr), value :: button, window
       integer(c_int)     :: domodal
     end subroutine
   end interface

   type(c_ptr), value :: button, window

   call run_color_selection_dialog(button, window, gtkFALSE)

end subroutine nonmodal_clicked
subroutine run_color_selection_dialog(button, window, domodal) bind(c)
!
! create a new color selection dialog and allow the user
! to choose a color and an opacity value
!
   use iso_c_binding

   use gtk, only: gtk_color_chooser_dialog_new, &
                  gtk_window_set_modal, &
                  gtk_color_chooser_set_rgba, &
                  gtk_color_chooser_set_use_alpha, &
                  gtk_widget_show_all, &
                  g_signal_connect, &
                  gtkTRUE => TRUE, &
                  gtkFALSE => FALSE

   interface
     subroutine dialog_response(dialog, response, gdata) bind(c)
       use iso_c_binding
       type(c_ptr), value   :: dialog
       integer(c_int), value :: response
       type(c_ptr), value   :: gdata
     end subroutine
   end interface

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   type, bind(c) :: GdkRGBA
     real(kind=c_double) :: red
     real(kind=c_double) :: green
     real(kind=c_double) :: blue
     real(kind=c_double) :: alpha
   end type GdkRGBA

   type(GdkRGBA), target :: global_color_rgba

   common global_color_rgba

   type(c_ptr), value :: button, window
   character(len=32)  :: title
   integer(c_int)     :: domodal

   type(c_ptr) :: dialog

   if( domodal == gtkTRUE )then
     title = 'Choose Color -- Modal'
   else
     title = 'Choose Color -- Non-Modal'
   endif

   dialog = gtk_color_chooser_dialog_new(title(1:len_trim(title))//CNULL,C_NULL_PTR)

   if( domodal == gtkTRUE )then
     call gtk_window_set_modal(dialog, gtkTRUE)
   else
     call gtk_window_set_modal(dialog, gtkFALSE)
   endif

   !    gtk_color_chooser_set_use_alpha(GtkColorChooser *chooser, gboolean use_alpha)
   call gtk_color_chooser_set_use_alpha(dialog, gtkTRUE)
   !all gtk_color_chooser_set_use_alpha(dialog, gtkFALSE)

   if( global_color_rgba%alpha /= -1 ) &
     call gtk_color_chooser_set_rgba(dialog,c_loc(global_color_rgba))

   call g_signal_connect(dialog, 'response'//CNULL, c_funloc(dialog_response))

   call gtk_widget_show_all(dialog)

end subroutine run_color_selection_dialog
subroutine dialog_response(dialog, response, gdata) bind(c)
!
! handle the response identifier from the assistant
! either tell the user to read the manual, retrieve the new color
! value or destroy the dialog
!
   use iso_c_binding

   use gtk, only: gtk_color_chooser_get_rgba, &
                  gtk_widget_destroy, &
                  GTK_RESPONSE_HELP, &
                  GTK_RESPONSE_OK

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   type(c_ptr), value    :: dialog
   integer(c_int), value :: response
   type(c_ptr), value    :: gdata

   type, bind(c) :: GdkRGBA
     real(kind=c_double) :: red
     real(kind=c_double) :: green
     real(kind=c_double) :: blue
     real(kind=c_double) :: alpha
   end type GdkRGBA

   type(GdkRGBA),  target :: global_color_rgba
   integer :: ired, igreen, iblue, ialpha

   common global_color_rgba

   select case(response)
     ! in GTK+ 3.x there is no HELP button
     !ase(GTK_RESPONSE_HELP)
     ! write(*,*) 'Read the GTK+ API documentation.'
     !
     case(GTK_RESPONSE_OK)

       call gtk_color_chooser_get_rgba(dialog,c_loc(global_color_rgba))

       write(*,1) global_color_rgba
     1 format(1x,'rgba:',3(f7.4),'  a:',f7.4)

       ired   = int(255.0*global_color_rgba%red)
       igreen = int(255.0*global_color_rgba%green)
       iblue  = int(255.0*global_color_rgba%blue)
       ialpha = int(255.0*global_color_rgba%alpha)

       write(*,2) ired, igreen, iblue, ialpha
     2 format(1x,'icol:',3(i7),'  a:',i7)

       call gtk_widget_destroy(dialog)

     case default
       call gtk_widget_destroy(dialog)

   end select

end subroutine



