!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program fontselection

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_font_chooser_dialog_new, &
                  gtk_font_chooser_set_font, &
                  gtk_font_chooser_set_preview_text, &
                  g_signal_connect, &
                  gtk_main_quit, &
                  gtk_widget_show_all, &
                  gtk_main

   implicit none

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine font_dialog_response(dialog,response,gdata) bind(c)
       use iso_c_binding
       type(c_ptr), value    :: dialog, gdata
       integer(c_int), value :: response
     end subroutine
   end interface

   type(c_ptr) :: dialog

   call gtk_init()

   ! use the font selection dialog as the toplevel widget
   dialog = gtk_font_chooser_dialog_new('Choose a Font'//CNULL, &
                                                      C_NULL_PTR)
   call gtk_font_chooser_set_font(dialog, &
                                  'Sans Bold Italic 12'//CNULL)
   call gtk_font_chooser_set_preview_text(dialog, &
                      'Foundations of GTK+ Development'//CNULL)

   call g_signal_connect(dialog, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit),C_NULL_PTR)
   call g_signal_connect(dialog, 'response'//CNULL, &
                         c_funloc(font_dialog_response), C_NULL_PTR)

   call gtk_widget_show_all(dialog)

   call gtk_main()

end program
subroutine font_dialog_response(dialog, response, gdata) bind(c)
!
! If the user presses "Apply", display the font, but do not
! destroy the dialog. If "OK" is pressed, display the font
! and destroy the dialog. Otherwise, just destroy the dialog.
!
   use iso_c_binding

   use gtk, only: GTK_RESPONSE_APPLY, &
                  GTK_RESPONSE_OK, &
                  gtk_font_chooser_get_font, &
                 !gtk_message_dialog_new,
                  GTK_DIALOG_MODAL, &
                  GTK_MESSAGE_INFO, &
                  GTK_BUTTONS_OK, &
                  gtk_window_set_title, &
                  gtk_dialog_run, &
                  gtk_widget_destroy

   use g, only:   g_free

   interface
     function gtk_message_dialog_new(parent,flags,mtype, &
                                       buttons,message_format) bind(c)
       use iso_c_binding
       type(c_ptr)           :: gtk_message_dialog_new
       type(c_ptr), value    :: parent
       integer(c_int), value :: flags, mtype, buttons
       character(c_char), dimension(*) :: message_format
     end
   end interface

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   type(c_ptr), value    :: dialog
   integer(c_int), value :: response
   type(c_ptr), value    :: gdata

   type(c_ptr) :: message
   type(c_ptr) :: c_string
   character(len=128) :: font

   select case(response)
     case(GTK_RESPONSE_APPLY)

       c_string  = gtk_font_chooser_get_font(dialog)
       call c_f_string(c_string,font)
       message   = gtk_message_dialog_new(C_NULL_PTR, GTK_DIALOG_MODAL, &
                     GTK_MESSAGE_INFO, GTK_BUTTONS_OK, trim(font)//CNULL)

       call gtk_window_set_title(message, 'Selected Font'//CNULL)

       ir = gtk_dialog_run(message)
       call gtk_widget_destroy(message)

       call g_free(c_string)

     case(GTK_RESPONSE_OK)

       c_string  = gtk_font_chooser_get_font(dialog)
       call c_f_string(c_string,font)
       message   = gtk_message_dialog_new(C_NULL_PTR, GTK_DIALOG_MODAL, &
                     GTK_MESSAGE_INFO, GTK_BUTTONS_OK, trim(font)//CNULL)

       call gtk_window_set_title(message, 'Selected Font'//CNULL)

       ir = gtk_dialog_run(message)
       call gtk_widget_destroy(message)

       call g_free(c_string)

     case default

       call gtk_widget_destroy(dialog)

   end select

   if( response == GTK_RESPONSE_OK ) call gtk_widget_destroy(dialog)

end subroutine


