!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program assistants

   use iso_c_binding

   use gtk, only: gtk_init, &
                  GTK_ASSISTANT_PAGE_INTRO, &
                  GTK_ASSISTANT_PAGE_CONTENT, &
                  GTK_ASSISTANT_PAGE_CONTENT, &
                  GTK_ASSISTANT_PAGE_PROGRESS, &
                  GTK_ASSISTANT_PAGE_CONFIRM, &
                  gtk_assistant_new, &
                  gtk_widget_set_size_request, &
                  gtk_window_set_title, &
                  gtk_label_new, &
                  gtk_entry_new, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_button_new_with_label, &
                  gtk_progress_bar_new, &
                  gtk_box_new, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  gtk_check_button_new_with_label, &
                  gtk_alignment_new, &
                  gtk_assistant_append_page, &
                  gtk_assistant_set_page_title, &
                  gtk_assistant_set_page_type, &
                  gtk_assistant_set_page_complete, &
                  g_signal_connect, &
                  gtk_main_quit, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtkTRUE => TRUE, &
                  gtkFALSE => FALSE

   use g, only:   g_object_set_data

   implicit none

   character(len=1), parameter :: CNULL = C_NULL_CHAR
   character(len=1), parameter :: LF = C_NEW_LINE

   interface
     subroutine entry_changed(enter,assistant) bind(c)
       use iso_c_binding
       type(c_ptr), value :: enter
       type(c_ptr), value :: assistant
     end
     subroutine button_toggled(toggle,assistant) bind(c)
       use iso_c_binding
       type(c_ptr), value :: toggle
       type(c_ptr), value :: assistant
     end
     subroutine button_clicked(button, assistant) bind(c)
       use iso_c_binding
       type(c_ptr), value :: button
       type(c_ptr), value :: assistant
     end
     subroutine assistant_cancel(assistant, gdata) bind(c)
       use iso_c_binding
       type(c_ptr), value :: assistant
       type(c_ptr), value :: gdata
     end
     subroutine assistant_close(assistant, gdata) bind(c)
       use iso_c_binding
       type(c_ptr), value :: assistant
       type(c_ptr), value :: gdata
     end
   end interface

   type  :: PageInfo
     type(c_ptr)    :: widget
     integer(c_int) :: indx
     character(len=22) :: title
     integer(c_int) :: ptype
     integer(c_int) :: complete
   end type

   type(c_ptr) :: assistant, enter, label, button, progress, hbox

   type(PageInfo), dimension(5) :: page

   integer   :: i, j
   character(len=22) :: string

   page(1) = PageInfo(C_NULL_PTR, -1, 'Introduction          ', GTK_ASSISTANT_PAGE_INTRO,    gtkTRUE)
   page(2) = PageInfo(C_NULL_PTR, -1, C_NULL_CHAR,              GTK_ASSISTANT_PAGE_CONTENT,  gtkFALSE)
   page(3) = PageInfo(C_NULL_PTR, -1, 'Click the Check Button', GTK_ASSISTANT_PAGE_CONTENT,  gtkFALSE)
   page(4) = PageInfo(C_NULL_PTR, -1, 'Click the Button      ', GTK_ASSISTANT_PAGE_PROGRESS, gtkFALSE)
   page(5) = PageInfo(C_NULL_PTR, -1, 'Confirmation          ', GTK_ASSISTANT_PAGE_CONFIRM,  gtkTRUE)

   call gtk_init()

   !
   ! create a new assistant widget with no pages
   !
   assistant = gtk_assistant_new()
   call gtk_widget_set_size_request(assistant, 450, 300)
   call gtk_window_set_title(assistant, 'GtkAssistant Example'//CNULL)

   call g_signal_connect(assistant, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)

   page(1)%widget = gtk_label_new('This is an example of a GtkAssistant.'//LF// &
                                  'By clicking the forward button, you'//LF// &
                                  'can continue to the next section!'//CNULL)
   page(2)%widget = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5)
   page(3)%widget = gtk_check_button_new_with_label('Click Me To Continue!'//CNULL)
   page(4)%widget = gtk_alignment_new(0.5, 0.5, 0.0, 0.0)
   page(5)%widget = gtk_label_new('Text has been entered in the label and the'//LF// &
                                  'combo box is clicked. If you are done, then'//LF// &
                                  'it is time to leave!'//CNULL)

   !
   ! create the necessary widgets for the second page
   !
   label = gtk_label_new('Your Name: '//CNULL)
   enter = gtk_entry_new()
   call gtk_box_pack_start(page(2)%widget, label, gtkFALSE, gtkFALSE, 5)
   call gtk_box_pack_start(page(2)%widget, enter, gtkFALSE, gtkFALSE, 5)

   !
   ! create the necessary widgets for the fourth page. the, Attach the
   ! progress bar to the GtkAlignment widget for later access.
   !
   button   = gtk_button_new_with_label('Click me!'//CNULL)
   progress = gtk_progress_bar_new()
   hbox     = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5)

   call gtk_box_pack_start(hbox, progress, gtkTRUE, gtkFALSE, 5)
   call gtk_box_pack_start(hbox, button, gtkFALSE, gtkFALSE, 5)
   call gtk_container_add(page(4)%widget, hbox)

   call g_object_set_data(page(4)%widget, 'pbar'//CNULL, progress)

   !
   ! add five pages to the GtkAssistant dialog
   !
   do i=1,5
     page(i)%indx = gtk_assistant_append_page(assistant, page(i)%widget)

     call gtk_assistant_set_page_title(assistant, &
                                  page(i)%widget, trim(page(i)%title)//CNULL)
     call gtk_assistant_set_page_type(assistant, &
                                  page(i)%widget, page(i)%ptype)

     !
     ! set the introduction and conclusion pages as complete so
     ! they can be incremented or closed
     !
     call gtk_assistant_set_page_complete(assistant, &
                                     page(i)%widget, page(i)%complete)
   end do

   !
   ! update whether pages 2 through 4 are complete based upon
   ! whether there is text in the GtkEntry, the check button is active,
   ! or the progress bar is completely filled
   !
   call g_signal_connect(enter, 'changed'//CNULL, &
                    c_funloc(entry_changed), assistant)
   call g_signal_connect(page(3)%widget, 'toggled'//CNULL, &
                    c_funloc(button_toggled), assistant)
   call g_signal_connect(button, 'clicked'//CNULL, &
                    c_funloc(button_clicked), assistant)

   call g_signal_connect(assistant, 'cancel'//CNULL, &
                    c_funloc(assistant_cancel), C_NULL_PTR)
   call g_signal_connect(assistant, 'close'//CNULL, &
                    c_funloc(assistant_close), C_NULL_PTR)

   call gtk_widget_show_all(assistant)

   call gtk_main()

end
subroutine entry_changed(enter, assistant) bind(c)
!
! if there is text in the GtkEntry, set the page as complete.
! otherwise, stop the user from progressing the next page.
!
   use iso_c_binding

   use gtk, only: gtk_entry_get_text, &
                  gtk_assistant_get_current_page, &
                  gtk_assistant_get_nth_page, &
                  gtk_assistant_set_page_complete, &
                  gtkFALSE => FALSE, &
                  gtkTRUE  => TRUE

   type(c_ptr), value :: enter
   type(c_ptr), value :: assistant

   type(c_ptr)    :: c_text, page
   integer(c_int) :: num, ok = gtkFALSE
   character(len=128) :: text

   c_text = gtk_entry_get_text(enter)
   num    = gtk_assistant_get_current_page(assistant)
   page   = gtk_assistant_get_nth_page(assistant, num)

   call c_f_string(c_text, text)
   if( len_trim(text) > 0 ) ok = gtkTRUE

   call gtk_assistant_set_page_complete(assistant, page, ok)

end subroutine entry_changed
subroutine button_toggled(toggle, assistant) bind(c)
!
! if the check button is toggled, set the page as complete.
! otherwise, stop the user from progressing the next page.
!
   use iso_c_binding

   use gtk, only: gtk_toggle_button_get_active, &
                  gtk_assistant_set_page_complete

   type(c_ptr), value :: toggle
   type(c_ptr), value :: assistant

   integer(c_int) :: active

   active = gtk_toggle_button_get_active(toggle)
   call gtk_assistant_set_page_complete(assistant, toggle, active)

end subroutine button_toggled
subroutine button_clicked(button, assistant) bind(c)
!
! fill up the progress bar, 10% every second when the
! button is clicked. then, set the page as complete when
! the progress bar is filled.
!
   use iso_c_binding

   use gtk, only: gtk_widget_set_sensitive, &
                  gtk_assistant_get_nth_page , &
                  gtk_progress_bar_set_fraction, &
                  gtk_progress_bar_set_text, &
                  gtk_events_pending, &
                  gtk_main_iteration, &
                  gtk_assistant_set_page_complete, &
                  gtkTRUE => TRUE, &
                  gtkFALSE => FALSE

   use g, only:   g_object_get_data, &
                  g_usleep

   character(len=1), parameter :: CNULL = C_NULL_CHAR
   character(len=1), parameter :: LF = C_NEW_LINE

   type(c_ptr), value :: button
   type(c_ptr), value :: assistant

   type(c_ptr)  :: progress
   type(c_ptr)  :: page
   real         :: percent = 0.0
   character(len=14) :: message
   integer(c_int)    :: i

   call gtk_widget_set_sensitive(button, gtkFALSE)

   page = gtk_assistant_get_nth_page(assistant, 3)
   progress = g_object_get_data(page,'pbar'//CNULL)

   do while( percent <= 100.0 )

     write(message,'(i3,''% Complete'')') int(percent)
     call gtk_progress_bar_set_fraction(progress, dble(percent/100.0))
     call gtk_progress_bar_set_text(progress, trim(message)//CNULL)

     do while( gtk_events_pending() == gtkTRUE )
       i = gtk_main_iteration()
     end do

     call g_usleep(int8(100000))
     percent = percent + 5.0

   end do

   call gtk_assistant_set_page_complete(assistant, page, gtkTRUE)

end subroutine button_clicked
subroutine assistant_cancel(assistant, gdata) bind(c)
!
! if the dialog is cancelled, delete it from memory and
! then clean up after the Assistant structure
!
   use iso_c_binding

   use gtk, only: gtk_widget_destroy

   type(c_ptr), value :: assistant
   type(c_ptr), value :: gdata

   call gtk_widget_destroy(assistant)

end subroutine assistant_cancel
subroutine assistant_close(assistant, gdata) bind(c)
!
! this function is where you would apply the changes
! and destroy the assistant
!
   use iso_c_binding

   use gtk, only: gtk_widget_destroy

   type(c_ptr), value :: assistant
   type(c_ptr), value :: gdata

   write(*,*) 'You would apply your changes now!'
   call gtk_widget_destroy(assistant)

end subroutine assistant_close
