!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program timers

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_main_quit, &
                  gtk_button_new_with_label, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main

   use g, only:   g_timer_new

   implicit none

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine button_clicked(button,timez) bind(c)
       use iso_c_binding
       type(c_ptr), value :: button, timez
     end subroutine
   end interface

   type(c_ptr) :: window, button
   type(c_ptr), target :: timez

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Timers'//CNULL)
   call gtk_container_set_border_width(window, 10)
   call gtk_widget_set_size_request(window, 150, 75)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)

   ! initialize the timer
   timez = g_timer_new()

   button = gtk_button_new_with_label('Start Timer'//CNULL)

   call g_signal_connect(button, 'clicked'//CNULL, &
                         c_funloc(button_clicked), c_loc(timez))

   call gtk_container_add(window, button)
   call gtk_widget_show_all(window)

   call gtk_main()

end
subroutine button_clicked(button,timez) bind(c)
!
! count the amount of elapsed time between two button clicks
!
   use iso_c_binding

   use gtk, only: gtk_button_set_label

   use g, only:   g_timer_elapsed

   implicit none

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   type(c_ptr), value :: button
   type(c_ptr) :: timez

   real(c_double), save :: start_time = 0.0
   real(c_double), save :: end_time = 0.0
   logical, save :: running = .false.

   if( .not. running )then
     start_time = g_timer_elapsed(timez, C_NULL_PTR)
     call gtk_button_set_label(button, 'Stop Timer'//CNULL)
   else
     end_time = g_timer_elapsed(timez, C_NULL_PTR)
     call gtk_button_set_label(button, 'Start Timer'//CNULL)
     write(*,'(1x,''Elapsed Time: '',f8.2)') end_time-start_time
   endif

   running = .not. running

end




