!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2013
!
program scrolledwindows

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  gtk_window_set_resizable, &
                  g_signal_connect, &
                  gtk_grid_new, &
                  gtk_grid_set_row_spacing, &
                  gtk_grid_set_column_spacing, &
                  gtk_grid_attach, &
                  gtk_paned_new, &
                  gtk_paned_pack1, gtk_paned_pack2, &
                  gtk_button_new_from_stock, &
                  gtk_button_new_with_label, &
                  gtk_button_set_relief, &
                  gtk_scrolled_window_new, &
                  gtk_scrolled_window_get_hadjustment, &
                  gtk_scrolled_window_get_vadjustment, &
                  gtk_viewport_new, &
                  gtk_scrolled_window_set_policy, &
                  gtk_scrolled_window_add_with_viewport, &
                  gtk_container_add, &
                  gtk_widget_set_size_request, &
                  gtk_box_new, &
                  gtk_box_get_homogeneous, &
                  gtk_box_set_homogeneous, &
                  gtk_box_set_spacing, &
                  gtk_box_pack_start, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtk_main_quit, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_POLICY_AUTOMATIC, &
                  GTK_POLICY_ALWAYS, &
                  GTK_RELIEF_NONE, &
                  GTK_RELIEF_HALF, &
                  GTK_RELIEF_NORMAL, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE

   implicit none

   type(c_ptr)    :: window, swin, viewport, table1, table2, vbox
   type(c_ptr)    :: horizontal, vertical
   type(c_ptr)    :: buttons1(10,10), buttons2(10,10)
 
   interface 
     function gtk_table_new(rows, columns, homogeneous) bind(c) 
       use iso_c_binding, only: c_ptr, c_int
       type(c_ptr) :: gtk_table_new
       integer(c_int), value :: rows
       integer(c_int), value :: columns
       integer(c_int), value :: homogeneous
     end function
     subroutine gtk_table_attach_defaults(table, widget, & 
                left_attach, right_attach, top_attach, bottom_attach) bind(c) 
       use iso_c_binding, only: c_ptr, c_int
       type(c_ptr), value :: table
       type(c_ptr), value :: widget
       integer(c_int), value :: left_attach
       integer(c_int), value :: right_attach
       integer(c_int), value :: top_attach
       integer(c_int), value :: bottom_attach
     end subroutine
     subroutine gtk_table_set_row_spacings(table, spacing) bind(c) 
       use iso_c_binding, only: c_ptr, c_int
       type(c_ptr), value :: table
       integer(c_int), value :: spacing
     end subroutine
     subroutine gtk_table_set_col_spacings(table, spacing) bind(c) 
       use iso_c_binding, only: c_ptr, c_int
       type(c_ptr), value :: table
       integer(c_int), value :: spacing
     end subroutine
     function gtk_vbox_new(homogeneous, spacing) bind(c) 
       use iso_c_binding, only: c_ptr, c_int
       type(c_ptr) :: gtk_vbox_new
       integer(c_int), value :: homogeneous
       integer(c_int), value :: spacing
     end function
   end interface
   
   integer(c_int) :: i, j
  
   character(len=1), parameter :: CNULL = C_NULL_CHAR
   character(len=*), parameter :: GTK_STOCK_CLOSE = 'gtk-close'//CNULL

   character(len=7) :: string
   integer :: iGTK = 6

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, &
                             'Scrolled Windows & Viewports'//CNULL)
   call gtk_container_set_border_width(window,10)
   call gtk_widget_set_size_request(window, 400, 300)
   call gtk_window_set_resizable(window,gtkFALSE)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)
  
   if( iGTK == 2 )then
     write(*,*) 'GTK+2 table used'
     table1 = gtk_table_new(10, 10, gtkTRUE)
     table2 = gtk_table_new(10, 10, gtkTRUE)
     call gtk_table_set_row_spacings(table1, 5)
     call gtk_table_set_row_spacings(table2, 5)
     call gtk_table_set_col_spacings(table1, 5)
     call gtk_table_set_col_spacings(table2, 5)
   else
     table1 = gtk_grid_new()
     table2 = gtk_grid_new()
     call gtk_grid_set_row_spacing(table1, 5)
     call gtk_grid_set_row_spacing(table2, 5)
     call gtk_grid_set_column_spacing(table1, 5)
     call gtk_grid_set_column_spacing(table2, 5)
   endif
   
   ! pack each table with 100 buttons
   do j=1,10
     do i=1,10
      !buttons1(i,j) = gtk_button_new_from_stock(GTK_STOCK_CLOSE)
      !buttons2(i,j) = gtk_button_new_from_stock(GTK_STOCK_CLOSE)

       write(string,'(i1,1x,i2,'','',i2)') 1,i,j
       buttons1(i,j) = gtk_button_new_with_label(string)
       write(string,'(i1,1x,i2,'','',i2)') 2,i,j
       buttons2(i,j) = gtk_button_new_with_label(string)

       call gtk_button_set_relief(buttons1(i,j), GTK_RELIEF_HALF)
       call gtk_button_set_relief(buttons2(i,j), GTK_RELIEF_HALF)
      
      !call gtk_table_attach_defaults(table1, buttons1(i,j),i,i+1,j,j+1)
      !     gtk_grid_attach(grid,  child,  left, top, width, height) bind(c)
      !
      ! adds a widget to the grid, the position of child is determined 
      ! by left and top. The number of 'cells' that child will occupy 
      ! is determined by width and height.
      ! left  : the column number to attach the left side of child to
      ! top   : the row number to attach the top side of child to
      ! width : the number of columns that child will span
      ! height:the number of rows that child will span
      !
      ! i,j rij,col
       
       if( iGTK == 2 )then
         call gtk_table_attach_defaults(table1,buttons1(i,j), j-1,j,i-1,i)
         call gtk_table_attach_defaults(table2,buttons2(i,j), j-1,j,i-1,i)
       else
         call gtk_grid_attach(table1,buttons1(i,j), j-1,i-1,1,1)
         call gtk_grid_attach(table2,buttons2(i,j), j-1,i-1,1,1)
       endif

       call g_signal_connect(buttons1(i,j), 'clicked'//CNULL, &
                                     c_funloc(gtk_main_quit), &
                                     C_NULL_PTR)
       call g_signal_connect(buttons2(i,j), 'clicked'//CNULL, &
                                     c_funloc(gtk_main_quit), &
                                     C_NULL_PTR)
    enddo
   enddo

   !
   ! create a scrolled window and a viewport, each with one table. use the
   ! adjustments in the scrolled window to synchronize both containers
   !
   swin = gtk_scrolled_window_new(C_NULL_PTR, C_NULL_PTR)
  
   horizontal = gtk_scrolled_window_get_hadjustment(swin)
   vertical   = gtk_scrolled_window_get_vadjustment(swin)

   if( iGTK == 6 )then
     viewport   = gtk_scrolled_window_new(horizontal, vertical)
   else
     viewport   = gtk_viewport_new(horizontal, vertical)
   endif
   
   call gtk_container_set_border_width(swin, 5)
   call gtk_container_set_border_width(viewport, 5)

   ! the policy determines when the scrollbar should appear
  !call gtk_scrolled_window_set_policy(swin,     &
  !                        GTK_POLICY_AUTOMATIC, & ! policy for horizontal bar
  !                        GTK_POLICY_AUTOMATIC)   ! policy for vertical bar
   call gtk_scrolled_window_set_policy(swin,     &
                           GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS)  

   ! add child without native scrolling capabilities:
   call gtk_scrolled_window_add_with_viewport(swin, table1) 
   if( iGTK == 6 )then
     call gtk_scrolled_window_add_with_viewport(viewport, table2) 
   else
     call gtk_container_add(viewport, table2)        ! child has native scrolling
   endif
   !
   ! equivalent to add unscrollable child to a viewport, then add the 
   ! viewport to the scrolled window
   ! swin_viewport  = gtk_viewport_new(horizontal, vertical)
   ! call gtk_container_add(swin_viewport, table1)   
   ! call gtk_container_add(swin, swin_viewport)   
   ! call gtk_container_add(viewport, table2)
   !
   ! what's the difference between spacing (set when the box is created) and
   ! padding (set when elements are packed)? spacing is added between objects,
   ! and padding is added on either side of a widget.    
   !

   ! pack the widgets into a GtkVBox and then into the window
   if( iGTK == 1 )then  
     write(*,*) 'GTK+3'
     vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0)
     call gtk_box_pack_start(vbox, viewport, gtkTRUE, gtkTRUE, 0)
     call gtk_box_pack_start(vbox, swin,     gtkTRUE, gtkTRUE, 0)
   elseif( iGTK == 2 )then  
     write(*,*) 'GTK+2 vbox used'
     vbox = gtk_vbox_new(gtkTRUE,5)
     call gtk_box_pack_start(vbox, viewport, gtkFALSE, gtkFALSE, 0)
     call gtk_box_pack_start(vbox, swin,     gtkFALSE, gtkFALSE, 0)
   elseif( iGTK == 3 )then  
     write(*,*) 'GTK+3 box used'
     vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0)
     call gtk_box_set_homogeneous(vbox,gtkTRUE)
     call gtk_box_set_spacing(vbox,5)
     call gtk_box_pack_start(vbox, viewport, gtkFALSE, gtkFALSE, 0)
     call gtk_box_pack_start(vbox, swin,     gtkFALSE, gtkFALSE, 0)
   elseif( iGTK == 4 )then  
     write(*,*) 'GTK+3 grid used'
     vbox = gtk_grid_new()                 ! width height
     call gtk_widget_set_size_request(swin,-1,300)
     call gtk_grid_set_row_spacing(vbox, 5)
     call gtk_grid_attach(vbox,viewport, 0,0,1,1)
     call gtk_grid_attach(vbox,swin,     0,1,1,1)
   elseif( iGTK == 5 )then  
     write(*,*) 'GTK+3 paned used'
     vbox = gtk_paned_new(GTK_ORIENTATION_VERTICAL)
     call gtk_paned_pack1(vbox,viewport,gtkTRUE,gtkTRUE)
     call gtk_paned_pack2(vbox,swin    ,gtkTRUE,gtkFALSE)
   elseif( iGTK == 6 )then  
     write(*,*) 'GTK+3 two scrolled windows used'
     vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0)
     call gtk_box_pack_start(vbox, viewport, gtkTRUE, gtkTRUE, 0)
     call gtk_box_pack_start(vbox, swin,     gtkTRUE, gtkTRUE, 0)
   endif
 
   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end


