!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2013
!
program textview2

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_text_view_new, &
                  gtk_text_view_get_buffer, &
                  gtk_text_buffer_set_text, &
                  gtk_widget_modify_font, &
                  gtk_text_view_set_wrap_mode, &
                  gtk_text_view_set_justification, &
                  gtk_text_view_set_editable, &
                  gtk_text_view_set_cursor_visible, &
                  gtk_text_view_set_pixels_above_lines, &
                  gtk_text_view_set_pixels_below_lines, &
                  gtk_text_view_set_pixels_inside_wrap, &
                  gtk_text_view_set_left_margin, &
                  gtk_text_view_set_right_margin, &
                  gtk_text_view_set_justification, &
                  gtk_text_view_set_wrap_mode, &
                  gtk_scrolled_window_new, &
                  gtk_scrolled_window_get_hadjustment, &
                  gtk_scrolled_window_get_vadjustment, &
                  gtk_scrolled_window_set_policy, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtk_main_quit, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_POLICY_AUTOMATIC, &
                  GTK_POLICY_ALWAYS, &
                  GTK_RELIEF_NONE, &
                  GTK_RELIEF_HALF, &
                  GTK_RELIEF_NORMAL, &
                  GTK_JUSTIFY_RIGHT, &
                  GTK_WRAP_WORD, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE

   use pango, only: pango_font_description_from_string

   implicit none

   type(c_ptr) :: window, scrolled_window, textview
   type(c_ptr) :: buffer
   type(c_ptr) :: font
    
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Text Views Properties'//CNULL)
   call gtk_container_set_border_width(window,10)
   call gtk_widget_set_size_request(window, 250, 150)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)

   font = pango_font_description_from_string('Monospace Bold 10')

   textview = gtk_text_view_new()
   call gtk_widget_modify_font(textview, font)

   call gtk_text_view_set_wrap_mode(textview, GTK_WRAP_WORD)
   call gtk_text_view_set_justification(textview, GTK_JUSTIFY_RIGHT)

   call gtk_text_view_set_editable(textview, gtkTRUE)
   call gtk_text_view_set_cursor_visible(textview, gtkTRUE)
  
   call gtk_text_view_set_pixels_above_lines(textview, 5)
   call gtk_text_view_set_pixels_below_lines(textview, 5)
   call gtk_text_view_set_pixels_inside_wrap(textview, 5)
  
   call gtk_text_view_set_left_margin(textview, 10)
   call gtk_text_view_set_right_margin(textview, 10)
  
   buffer = gtk_text_view_get_buffer(textview)
  
   call gtk_text_buffer_set_text(buffer, &
                    'This is some long text line!'//char(10)// &
                    'Change me!'//char(10)// &
                    'Please'//CNULL, -1)
  
   scrolled_window = gtk_scrolled_window_new(C_NULL_PTR,C_NULL_PTR)
   call gtk_scrolled_window_set_policy(scrolled_window, &
                                GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS)
  
   call gtk_container_add(scrolled_window, textview)
   call gtk_container_add(window, scrolled_window)
   call gtk_widget_show_all(window)

   call gtk_main()

end


