!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2013
!
program iterators

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_text_view_new, &
                  gtk_text_view_get_buffer, &
                  gtk_text_buffer_set_text, &
                  gtk_entry_get_text, &
                  gtk_entry_new, &
                  gtk_button_new_with_label, &
                  gtk_scrolled_window_new, &
                  gtk_box_new, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtk_main_quit, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_POLICY_AUTOMATIC, &
                  GTK_POLICY_ALWAYS, &
                  GTK_RELIEF_NONE, &
                  GTK_RELIEF_HALF, &
                  GTK_RELIEF_NORMAL, &
                  GTK_JUSTIFY_RIGHT, &
                  GTK_WRAP_WORD, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE

   implicit none

   type, bind(c) :: widgets
     type(c_ptr) :: enter
     type(c_ptr) :: textview
   end type   

   type(c_ptr)   :: window, scrolled_window, hbox, vbox, insert, retrieve
   type(widgets), target :: w
    
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine insert_text(button,widget) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button
       type, bind(c) :: widgets
         type(c_ptr) :: enter
         type(c_ptr) :: textview
       end type
       type(widgets) :: widget  
     end subroutine
     subroutine retrieve_text(button,widget) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button
       type, bind(c) :: widgets
         type(c_ptr) :: enter
         type(c_ptr) :: textview
       end type
       type(widgets) :: widget  
     end subroutine
   end interface

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Text Iterators'//CNULL)
   call gtk_container_set_border_width(window,10)
   call gtk_widget_set_size_request(window, -1, 200)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)

   w%textview = gtk_text_view_new()
   w%enter    = gtk_entry_new()

   insert   = gtk_button_new_with_label('Insert Text'//CNULL)
   retrieve = gtk_button_new_with_label('Get Text'//CNULL)

   call g_signal_connect(insert, 'clicked'//CNULL, &
                         c_funloc(insert_text), c_loc(w))
   call g_signal_connect(retrieve, 'clicked'//CNULL, &
                         c_funloc(retrieve_text), c_loc(w))

   scrolled_window = gtk_scrolled_window_new(C_NULL_PTR, C_NULL_PTR)
   call gtk_container_add(scrolled_window, w%textview)

   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5)
   call gtk_box_pack_start(hbox, w%enter,  gtkFALSE, gtkFALSE, 0)
   call gtk_box_pack_start(hbox, insert,   gtkFALSE, gtkFALSE, 0)
   call gtk_box_pack_start(hbox, retrieve, gtkFALSE, gtkFALSE, 0)
 
   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5)
   call gtk_box_pack_start(vbox, scrolled_window, gtkTRUE,  gtkTRUE, 0)
   call gtk_box_pack_start(vbox, hbox,            gtkFALSE, gtkTRUE, 0)
   
   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end
subroutine insert_text(button, w) bind(C)
!
! insert the text from the GtkEntry into the GtkTextView
!
   use iso_c_binding

   use gtk, only: gtk_text_view_get_buffer, &
                  gtk_entry_get_text, &
                  gtk_text_buffer_get_insert, &
                  gtk_text_buffer_get_iter_at_mark, &
                  gtk_text_buffer_insert

   type, bind(c) :: widgets
     type(c_ptr) :: enter
     type(c_ptr) :: textview
   end type   

   type(c_ptr), intent(in), value :: button
   type(widgets) :: w

   ! define a GtkTextIter (this has to be pre-allocated in the calls)
   type, bind(c) :: gtktextiter 
     type(c_ptr) :: d1, d2
     integer(kind=c_int) :: d3, d4, d5, d6, d7, d8
     type(c_ptr) :: d9, d10
     integer(kind=c_int) :: d11, d12, d13
     type(c_ptr) :: d14
   end type gtktextiter

   type(c_ptr) :: buffer, text, mark
   type(gtktextiter), target :: iter

   character(len=128), target :: textstring = ' '
   integer(c_int) :: lent

   buffer = gtk_text_view_get_buffer(w%textview)
   text   = gtk_entry_get_text(w%enter)

   call c_f_string(text, textstring)
   lent = len_trim(textstring)
   textstring(lent+1:lent+1) = C_NULL_CHAR
   write(*,*) 'inserting >',textstring(1:lent),'<'

   mark = gtk_text_buffer_get_insert(buffer)

   ! gtk_text_buffer_get_iter_at_mark 
   ! (GtkTextBuffer *buffer, GtkTextIter *iter, GtkTextMark *mark)
   call gtk_text_buffer_get_iter_at_mark(buffer,c_loc(iter), mark)

   ! gtk_text_buffer_insert
   ! (GtkTextBuffer *buffer, GtkTextIter *iter, const gchar *text, gint len)
   call gtk_text_buffer_insert(buffer,c_loc(iter), &
                               textstring(1:lent+1), -1)

end subroutine insert_text
subroutine retrieve_text(button, w) bind(C)
!
! retrieve the selected text from the GtkTextView and display it
!
   use iso_c_binding

   use gtk, only: gtk_text_view_get_buffer, &
                  gtk_entry_get_text, &
                  gtk_text_buffer_get_insert, &
                  gtk_text_buffer_get_selection_bounds, &
                  gtk_text_buffer_get_text, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE

   type, bind(c) :: widgets
     type(c_ptr) :: enter
     type(c_ptr) :: textview
   end type   

   type(c_ptr), intent(in), value :: button
   type(widgets) :: w

   ! define a GtkTextIter type
   type, bind(c) :: gtktextiter 
     type(c_ptr) :: d1, d2
     integer(kind=c_int) :: d3, d4, d5, d6, d7, d8
     type(c_ptr) :: d9, d10
     integer(kind=c_int) :: d11, d12, d13
     type(c_ptr) :: d14
   end type gtktextiter

   type(c_ptr) :: buffer, mark, text
   type(gtktextiter), target :: start, end

   character(len=128), target :: textstring = ' '
   integer(c_int) :: lens, i

   buffer = gtk_text_view_get_buffer(w%textview)
   text   = gtk_entry_get_text(w%enter)

   call c_f_string(text, textstring)
   lens = len_trim(textstring)

   ! returns the mark that represents the cursor (insertion point)
   mark = gtk_text_buffer_get_insert(buffer)
   
   ! returns TRUE if some text is selected
   i = gtk_text_buffer_get_selection_bounds(buffer,c_loc(start),c_loc(end))

   if( i > 0 )then
     write(*,*)'some text selected'
   else
     write(*,*)'nothing selected'
   endif
    
   ! obtain strings from the buffer
   ! gchar *gtk_text_buffer_get_text
   ! (GtkTextBuffer *buffer, const GtkTextIter *start, 
   !  const GtkTextIter *end, gboolean include_hidden_chars)

   text = gtk_text_buffer_get_text(buffer,c_loc(start),c_loc(end), gtkTRUE)

   call c_f_string(text, textstring)
   lens = len_trim(textstring)
   write(*,*)'got=>',textstring(1:lens),'<=',i,lens
   
end subroutine retrieve_text

