!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2013
!
program cutcopypaste 

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_button_new_from_stock, &
                  gtk_text_view_new, &
                  gtk_text_view_get_buffer, &
                  gtk_text_buffer_set_text, &
                  gtk_entry_get_text, &
                  gtk_entry_new, &
                  gtk_button_new_with_label, &
                  gtk_scrolled_window_new, &
                  gtk_box_new, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtk_main_quit, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_POLICY_AUTOMATIC, &
                  GTK_POLICY_ALWAYS, &
                  GTK_RELIEF_NONE, &
                  GTK_RELIEF_HALF, &
                  GTK_RELIEF_NORMAL, &
                  GTK_JUSTIFY_RIGHT, &
                  GTK_WRAP_WORD, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE

   implicit none

   type(c_ptr)   :: window, scrolled_window, textview
   type(c_ptr)   :: cut, copy, paste
   type(c_ptr)   :: hbox, vbox
    
   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine cut_clicked(button,textview) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button, textview
     end subroutine
     subroutine copy_clicked(button,textview) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button, textview
     end subroutine
     subroutine paste_clicked(button,textview) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button, textview
     end subroutine
   end interface

   character(len=*), parameter :: GTK_STOCK_CUT   = 'gtk-cut'//CNULL
   character(len=*), parameter :: GTK_STOCK_COPY  = 'gtk-copy'//CNULL
   character(len=*), parameter :: GTK_STOCK_PASTE = 'gtk-paste'//CNULL

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Cut, Copy & Paste'//CNULL)
   call gtk_container_set_border_width(window,10)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)

   textview = gtk_text_view_new()
   cut   = gtk_button_new_from_stock(GTK_STOCK_CUT)
   copy  = gtk_button_new_from_stock(GTK_STOCK_COPY)
   paste = gtk_button_new_from_stock(GTK_STOCK_PASTE)

   call g_signal_connect(cut, 'clicked'//CNULL, &
                         c_funloc(cut_clicked), textview)
   call g_signal_connect(copy, 'clicked'//CNULL, &
                         c_funloc(copy_clicked), textview)
   call g_signal_connect(paste, 'clicked'//CNULL, &
                         c_funloc(paste_clicked), textview)

   scrolled_window = gtk_scrolled_window_new(C_NULL_PTR, C_NULL_PTR)

   call gtk_widget_set_size_request(scrolled_window, 300, 200)
   call gtk_container_add(scrolled_window, textview)

   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5)
   call gtk_box_pack_start(hbox, cut,   gtkTRUE, gtkTRUE, 0)
   call gtk_box_pack_start(hbox, copy,  gtkTRUE, gtkTRUE, 0)
   call gtk_box_pack_start(hbox, paste, gtkTRUE, gtkTRUE, 0)
 
   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5)
   call gtk_box_pack_start(vbox, scrolled_window, gtkTRUE,  gtkTRUE, 0)
   call gtk_box_pack_start(vbox, hbox,            gtkFALSE, gtkTRUE, 0)
   
   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end
subroutine cut_clicked(button, textview) bind(C)
!
! copy the selected text to the clipboard and remove it from the buffer
!
   use iso_c_binding

   use gtk, only: gtk_clipboard_get, &
                  gtk_text_view_get_buffer, &
                  gtk_text_buffer_cut_clipboard, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE
   
   interface
     function f03_gdk_selection_clipboard() bind(C,name="f03_GDK_SELECTION_CLIPBOARD")
       use iso_c_binding
       type(c_ptr) :: f03_gdk_selection_clipboard
     end function
   end interface
   
   type(c_ptr), value :: button, textview
   type(c_ptr) :: clipboard, buffer

   clipboard = gtk_clipboard_get(f03_GDK_SELECTION_CLIPBOARD())   
   buffer    = gtk_text_view_get_buffer(textview)
  
   call gtk_text_buffer_cut_clipboard(buffer, clipboard, gtkTRUE)

end subroutine cut_clicked
subroutine copy_clicked(button, textview) bind(C)
!
! copy the selected text to the clipboard
!
   use iso_c_binding

   use gtk, only: gtk_clipboard_get, &
                  gtk_text_view_get_buffer, &
                  gtk_text_buffer_copy_clipboard, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE

   interface
     function f03_gdk_selection_clipboard() bind(C,name="f03_GDK_SELECTION_CLIPBOARD")
       use iso_c_binding
       type(c_ptr) :: f03_gdk_selection_clipboard
     end function
   end interface

   type(c_ptr), value :: button, textview
   type(c_ptr) :: clipboard, buffer

   clipboard = gtk_clipboard_get(f03_GDK_SELECTION_CLIPBOARD())
   buffer = gtk_text_view_get_buffer(textview)
  
   call gtk_text_buffer_copy_clipboard(buffer, clipboard)

end subroutine copy_clicked
subroutine paste_clicked(button, textview) bind(C)
!
! copy the selected text to the clipboard
!
   use iso_c_binding

   use gtk, only: gtk_clipboard_get, &
                  gtk_text_view_get_buffer, &
                  gtk_text_buffer_paste_clipboard, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE

   interface
     function f03_gdk_selection_clipboard() bind(C,name="f03_GDK_SELECTION_CLIPBOARD")
       use iso_c_binding
       type(c_ptr) :: f03_gdk_selection_clipboard
     end function
   end interface

   type(c_ptr), value :: button, textview
   type(c_ptr) :: clipboard, buffer

   clipboard = gtk_clipboard_get(f03_GDK_SELECTION_CLIPBOARD())
   buffer = gtk_text_view_get_buffer(textview)
  
   call gtk_text_buffer_paste_clipboard(buffer, clipboard, C_NULL_PTR, gtkTRUE)

end subroutine paste_clicked


