!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2013
!
program find_string

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_text_view_new, &
                  gtk_text_view_set_buffer, &
                  gtk_text_view_get_buffer, &
                  gtk_text_buffer_set_text, &
                  gtk_button_new_from_stock, &
                  gtk_entry_set_text, &
                  gtk_entry_get_text, &
                  gtk_entry_new, &
                  gtk_button_new_with_label, &
                  gtk_scrolled_window_new, &
                  gtk_box_new, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtk_main_quit, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_POLICY_AUTOMATIC, &
                  GTK_POLICY_ALWAYS, &
                  GTK_RELIEF_NONE, &
                  GTK_RELIEF_HALF, &
                  GTK_RELIEF_NORMAL, &
                  GTK_JUSTIFY_RIGHT, &
                  GTK_WRAP_WORD, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE

   implicit none

   type, bind(c) :: widgets
     type(c_ptr) :: enter
     type(c_ptr) :: textview
   end type   

   type(c_ptr)   :: window, buffer, find, scrolled_window, hbox, vbox
   type(widgets), target :: w
    
   character(len=1), parameter :: CNULL = C_NULL_CHAR
   character(len=*), parameter :: GTK_STOCK_FIND = 'gtk-find'//CNULL
   
   interface
     subroutine search(button,widget) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button
       type, bind(c) :: widgets
         type(c_ptr) :: enter
         type(c_ptr) :: textview
       end type
       type(widgets) :: widget  
     end subroutine
   end interface

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Search for ...'//CNULL)
   call gtk_container_set_border_width(window,10)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)

   w%textview = gtk_text_view_new()
   w%enter    = gtk_entry_new()

   buffer = gtk_text_view_get_buffer(w%textview)
   call gtk_text_buffer_set_text(buffer, &
                    'Lorem ipsum dolor sit amet,'    //C_CARRIAGE_RETURN// &
                    'consectetur adipisicing elit,'  //C_CARRIAGE_RETURN// &
                    'sed do eiusmod tempor...'       //C_CARRIAGE_RETURN// &
                    'Falsches Üben von Xylophonmusik'//C_CARRIAGE_RETURN// &
                    'quält jeden größeren Zwerg...'  //C_CARRIAGE_RETURN// &
                    'The quick brown fox jumps over' //C_CARRIAGE_RETURN// &
                    'the lazy dog...'//CNULL, -1)

   call gtk_entry_set_text(w%enter, 'Search for ...'//CNULL)
   find = gtk_button_new_from_stock(GTK_STOCK_FIND);

   call g_signal_connect(find, 'clicked'//CNULL, &
                         c_funloc(search), c_loc(w))

   scrolled_window = gtk_scrolled_window_new(C_NULL_PTR, C_NULL_PTR)
   call gtk_widget_set_size_request(scrolled_window, 250, 200)
   call gtk_container_add(scrolled_window, w%textview)

   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5)
   call gtk_box_pack_start(hbox, w%enter, gtkTRUE,  gtkTRUE, 0)
   call gtk_box_pack_start(hbox, find,    gtkFALSE, gtkTRUE, 0)
 
   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5)
   call gtk_box_pack_start(vbox, scrolled_window, gtkTRUE,  gtkTRUE, 0)
   call gtk_box_pack_start(vbox, hbox,            gtkFALSE, gtkTRUE, 0)
   
   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end
subroutine search(button, w) bind(C)
!
! search for the entered string within the GtkTextView
! then tell the user how many times it was found
!
   use iso_c_binding

   use gtk, only: gtk_entry_get_text, &
                  gtk_text_view_get_buffer, &
                  gtk_text_buffer_get_start_iter, &
                  gtk_text_iter_forward_search, &
                  gtk_text_iter_forward_char, &
                  GTK_DIALOG_MODAL, &
                  GTK_MESSAGE_INFO, &
                  GTK_BUTTONS_OK, &
                  gtk_dialog_run, &
                  gtk_widget_destroy

   use g, only:   g_free

   interface
     !
     ! function commented out in gtk-auto.f90 and redefined
     !
     ! GtkWidget* gtk_message_dialog_new(GtkWindow *parent,
     !            GtkDialogFlags flags, GtkMessageType type,
     !            GtkButtonsType buttons, const gchar *message_format, ...)
     function gtk_message_dialog_new(parent,flags,mtype,button,mformat) bind(c)
       use iso_c_binding, only: c_ptr, c_int, c_char
       type(c_ptr) :: gtk_message_dialog_new
       type(c_ptr), value :: parent
       integer(c_int), value :: flags
       integer(c_int), value :: button
       integer(c_int), value :: mtype
       character(kind=c_char), dimension(*) :: mformat
     end function
     !
     ! void gtk_message_dialog_format_secondary_text(GtkMessageDialog *message_dialog,
     !                                               const gchar *message_format, ...) 
     !
     subroutine gtk_message_dialog_format_secondary_text(dialog,mformat) bind(c)
       use iso_c_binding, only: c_ptr, c_char
       type(c_ptr), value :: dialog
       character(kind=c_char), dimension(*) :: mformat
     end subroutine
   end interface

   type, bind(c) :: widgets
     type(c_ptr) :: enter
     type(c_ptr) :: textview
   end type   

   type(c_ptr), intent(in), value :: button
   type(widgets) :: w

   ! define a GtkTextIter type
   type, bind(c) :: gtktextiter 
     type(c_ptr) :: d1, d2
     integer(kind=c_int) :: d3, d4, d5, d6, d7, d8
     type(c_ptr) :: d9, d10
     integer(kind=c_int) :: d11, d12, d13
     type(c_ptr) :: d14
   end type gtktextiter

   type(c_ptr) :: find, buffer, dialog
   type(gtktextiter), target :: start, begin, end
   integer(c_int) :: success, moved, response
   integer        :: i 
 
   character(len=1), parameter :: CNULL = C_NULL_CHAR
   character(len=128) :: textstring = ' '
   character(len=128) :: findstring = ' '

   find   = gtk_entry_get_text(w%enter)
   buffer = gtk_text_view_get_buffer(w%textview)

   call c_f_string(find,findstring)
   write(*,*)'search for>',trim(findstring),'<'
  
   call gtk_text_buffer_get_start_iter(buffer,c_loc(start))
   
   ! searches forward for findstring, any match is returned by setting 
   ! begin to the first character of the match and end to the first character 
   ! after the match
   ! flags: flags affecting how the search is done
   success = gtk_text_iter_forward_search(c_loc(start), trim(findstring)//CNULL, 0, & 
                                          c_loc(begin), c_loc(end), C_NULL_PTR)
   i = 0
   do while( success == 1 )
     i     = i + 1
     start = begin

     ! moves iter forward by one character offset
     moved   = gtk_text_iter_forward_char(c_loc(start))
     
     success = gtk_text_iter_forward_search(c_loc(start), trim(findstring)//CNULL, 0, & 
                                            c_loc(begin), c_loc(end), C_NULL_PTR)
   end do

   write(textstring,'(''The string '',A,'' was found '',i3,'' times!'')') &
         trim(findstring), i
   write(*,*)'string>',trim(textstring),'<'

   dialog = gtk_message_dialog_new(C_NULL_PTR, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, &
                                   GTK_BUTTONS_OK, trim(textstring)//CNULL)
    
   call gtk_message_dialog_format_secondary_text(dialog, &
                                   'With some secondary text'//CNULL)

   response = gtk_dialog_run(dialog)
   call gtk_widget_destroy(dialog)
   
end subroutine search

