!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2013
!
program texttags

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_text_view_new, &
                  gtk_text_view_set_buffer, &
                  gtk_text_view_get_buffer, &
                  gtk_text_buffer_set_text, &
                  gtk_button_new_from_stock, &
                  gtk_combo_box_text_new, &
                  gtk_combo_box_text_append, &
                  gtk_entry_set_text, &
                  gtk_entry_get_text, &
                  gtk_entry_new, &
                  gtk_button_new_with_label, &
                  gtk_scrolled_window_new, &
                  gtk_scrolled_window_set_policy, &
                  gtk_box_new, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtk_main_quit, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_POLICY_AUTOMATIC, &
                  GTK_POLICY_ALWAYS, &
                  GTK_RELIEF_NONE, &
                  GTK_RELIEF_HALF, &
                  GTK_RELIEF_NORMAL, &
                  GTK_JUSTIFY_RIGHT, &
                  GTK_WRAP_WORD, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE, &
                  PANGO_WEIGHT_BOLD, &
                  PANGO_STYLE_ITALIC, &
                  PANGO_UNDERLINE_SINGLE

   use g, only:   g_object_set_data

   implicit none

   interface
     !
     ! GtkTextTag * gtk_text_buffer_create_tag(GtkTextBuffer *buffer,
     ! const gchar *tag_name, const gchar *first_property_name, ...);
     !
     subroutine f03gtk_text_buffer_create_tag_i(buffer, tag_name, prop, val) bind(C)
       use iso_c_binding, only: c_ptr, c_char, c_int
       type(c_ptr), value :: buffer
       character(kind=c_char), dimension(*) :: tag_name
       character(kind=c_char), dimension(*) :: prop
       integer(c_int), value :: val
     end subroutine
     subroutine f03gtk_text_buffer_create_tag_d(buffer, tag_name, prop, val) bind(C)
       use iso_c_binding, only: c_ptr, c_char, c_double
       type(c_ptr), value :: buffer
       character(kind=c_char), dimension(*) :: tag_name
       character(kind=c_char), dimension(*) :: prop
       real(c_double), value :: val
     end subroutine
     !
     subroutine f_c_string(f_string, c_string)
       use iso_c_binding, only: c_char, c_null_char
       character(len=*), intent(in) :: f_string
       character(kind=c_char), dimension(:), intent(out), allocatable :: c_string
     end subroutine
     !
     subroutine sform(widget, textview) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, textview 
     end subroutine
     subroutine size_changed(combo, textview) bind(C)
       use iso_c_binding
       type(c_ptr), value :: combo, textview 
     end subroutine
     subroutine clear_clicked(button, textview) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button, textview
     end subroutine
   end interface

   type(c_ptr) :: window, buffer, find, scrolled_window, hbox, vbox
   type(c_ptr), target :: textview
   type(c_ptr) :: bold, italic, underline, strike, clear, fsize  

   character(len=1), parameter :: CNULL = C_NULL_CHAR
   character(len=1), parameter :: CR    = C_CARRIAGE_RETURN
   character(len=*), parameter :: GTK_STOCK_FIND          = 'gtk-find'//CNULL
   character(len=*), parameter :: GTK_STOCK_BOLD          = 'gtk-bold'//CNULL
   character(len=*), parameter :: GTK_STOCK_ITALIC        = 'gtk-italic'//CNULL
   character(len=*), parameter :: GTK_STOCK_UNDERLINE     = 'gtk-underline'//CNULL
   character(len=*), parameter :: GTK_STOCK_STRIKETHROUGH = 'gtk-strikethrough'//CNULL
   character(len=*), parameter :: GTK_STOCK_CLEAR         = 'gtk-clear'//CNULL

   real(c_double), parameter :: PANGO_SCALE_XX_SMALL = 0.5787037037037
   real(c_double), parameter :: PANGO_SCALE_X_SMALL  = 0.6444444444444
   real(c_double), parameter :: PANGO_SCALE_SMALL    = 0.8333333333333
   real(c_double), parameter :: PANGO_SCALE_MEDIUM   = 1.0
   real(c_double), parameter :: PANGO_SCALE_LARGE    = 1.2
   real(c_double), parameter :: PANGO_SCALE_X_LARGE  = 1.4399999999999
   real(c_double), parameter :: PANGO_SCALE_XX_LARGE = 1.728

   type :: text_to_double
     character(len=18) :: str
     real(c_double)    :: scl
   end type

   integer, parameter :: Nscales = 9
   type(text_to_double), dimension(Nscales) :: text_scales = (/&
     text_to_double('Quarter Sized'//CNULL, 0.25) , &
     text_to_double('Double Extra Small'//CNULL, PANGO_SCALE_XX_SMALL), &
     text_to_double('Extra Small'//CNULL, PANGO_SCALE_X_SMALL), &
     text_to_double('Small'//CNULL, PANGO_SCALE_SMALL), &
     text_to_double('Medium'//CNULL, PANGO_SCALE_MEDIUM), &
     text_to_double('Large'//CNULL, PANGO_SCALE_LARGE), &
     text_to_double('Extra Large'//CNULL, PANGO_SCALE_X_LARGE), &
     text_to_double('Double Extra Large'//CNULL, PANGO_SCALE_XX_LARGE), &
     text_to_double('Double Sized'//CNULL, 2.0) /)

   integer :: i

   character(kind=c_char), dimension(:), target, allocatable :: &
              c_str1, c_str2, c_str3, c_str4 

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Text Tags'//CNULL)
   call gtk_container_set_border_width(window,10)
   call gtk_widget_set_size_request(window, 500, -1)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)

   textview = gtk_text_view_new()
   buffer   = gtk_text_view_get_buffer(textview)

   call gtk_text_buffer_set_text(buffer, &
                    'Lorem ipsum dolor sit amet,'    //CR// &
                    'consectetur adipisicing elit,'  //CR// &
                    'sed do eiusmod tempor...'       //CR// &
                    'Falsches Üben von Xylophonmusik'//CR// &
                    'quält jeden größeren Zwerg...'  //CR// &
                    'The quick brown fox jumps over' //CR// &
                    'the lazy dog...'//CNULL, -1)

   call f03gtk_text_buffer_create_tag_i(buffer, 'bold'//CNULL, & 
                                        'weight'//CNULL, PANGO_WEIGHT_BOLD)
   call f03gtk_text_buffer_create_tag_i(buffer, 'italic'//CNULL, & 
                                        'style'//CNULL, PANGO_STYLE_ITALIC)
   call f03gtk_text_buffer_create_tag_i(buffer, 'strike'//CNULL, & 
                                        'strikethrough'//CNULL, gtkTRUE)
   call f03gtk_text_buffer_create_tag_i(buffer, 'underline'//CNULL, & 
                                        'underline'//CNULL, PANGO_UNDERLINE_SINGLE)

   bold      = gtk_button_new_from_stock(GTK_STOCK_BOLD)
   italic    = gtk_button_new_from_stock(GTK_STOCK_ITALIC)
   underline = gtk_button_new_from_stock(GTK_STOCK_UNDERLINE)
   strike    = gtk_button_new_from_stock(GTK_STOCK_STRIKETHROUGH)
   clear     = gtk_button_new_from_stock(GTK_STOCK_CLEAR)

   fsize     = gtk_combo_box_text_new()

   ! add choices to the GtkComboBox widget
   do i=1,Nscales
     ! gtk_combo_box_text_append (GtkComboBoxText *combo_box,
     !                            const gchar *id, const gchar *text);
     call gtk_combo_box_text_append(fsize, CNULL, text_scales(i)%str//CNULL)
     
     call f03gtk_text_buffer_create_tag_d(buffer, text_scales(i)%str//CNULL, &
                                          'scale'//CNULL, text_scales(i)%scl)
  
   enddo

   ! add the name of the text tag as a data parameter of the object
   ! void g_object_set_data (GObject *object, const gchar *key, gpointer data);
   
   call f_c_string('bold',c_str1)
   call g_object_set_data(bold, 'tag'//CNULL, c_loc(c_str1))

   call f_c_string('italic',c_str1)
   call g_object_set_data(italic, 'tag'//CNULL, c_loc(c_str1))

   call f_c_string('underline',c_str1)
   call g_object_set_data(underline, 'tag'//CNULL, c_loc(c_str1))

   call f_c_string('strike',c_str4)
   call g_object_set_data(strike, 'tag'//CNULL, c_loc(c_str4))

   ! connect each of the buttons and the combo box to the necessary signals
   call g_signal_connect(bold,      'clicked'//CNULL, c_funloc(sform), textview)
   call g_signal_connect(italic,    'clicked'//CNULL, c_funloc(sform), textview)
   call g_signal_connect(underline, 'clicked'//CNULL, c_funloc(sform), textview)
   call g_signal_connect(strike,    'clicked'//CNULL, c_funloc(sform), textview)
   call g_signal_connect(fsize,     'changed'//CNULL, c_funloc(size_changed), textview)
   call g_signal_connect(clear,     'clicked'//CNULL, c_funloc(clear_clicked), textview)

   ! pack the widgets into a GtkVBox, GtkHBox, and then into the window.  
   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5)

   call gtk_box_pack_start(vbox, bold,      gtkFALSE, gtkFALSE, 0)
   call gtk_box_pack_start(vbox, italic,    gtkFALSE, gtkFALSE, 0)
   call gtk_box_pack_start(vbox, underline, gtkFALSE, gtkFALSE, 0)
   call gtk_box_pack_start(vbox, strike,    gtkFALSE, gtkFALSE, 0)
   call gtk_box_pack_start(vbox, fsize,     gtkFALSE, gtkFALSE, 0)
   call gtk_box_pack_start(vbox, clear,     gtkFALSE, gtkFALSE, 0)

   scrolled_window = gtk_scrolled_window_new(C_NULL_PTR, C_NULL_PTR)
   call gtk_container_add(scrolled_window, textview)
   call gtk_scrolled_window_set_policy(scrolled_window, &
                                       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS)

   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5)
   call gtk_box_pack_start(hbox, scrolled_window, gtkTRUE,  gtkTRUE, 0)
   call gtk_box_pack_start(hbox, vbox,            gtkFALSE, gtkTRUE, 0)
   
   call gtk_container_add(window, hbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end
recursive subroutine sform(widget, textview) bind(C)
!
! retrieve the tag from the 'tag' object data and apply it to the selection
!
   use iso_c_binding

   use gtk, only: gtk_text_view_get_buffer, &
                  gtk_text_buffer_get_selection_bounds, &
                  gtk_text_buffer_apply_tag_by_name
 
   use g, only:   g_object_get_data

   ! define a GtkTextIter type
   type, bind(c) :: gtktextiter 
     type(c_ptr) :: d1, d2
     integer(kind=c_int) :: d3, d4, d5, d6, d7, d8
     type(c_ptr) :: d9, d10
     integer(kind=c_int) :: d11, d12, d13
     type(c_ptr) :: d14
   end type gtktextiter

   type(c_ptr), value :: widget, textview 
   type(c_ptr) :: buffer, c_string
   type(gtktextiter), target :: start, begin, end
   integer(c_int) :: i
 
   character(len=1), parameter :: CNULL = C_NULL_CHAR
   character(len=128) :: tagname = ' '

   c_string =  g_object_get_data(widget, 'tag'//CNULL)
   call c_f_string(c_string, tagname)
   write(*,*)'tag>',trim(tagname),'<'

   buffer = gtk_text_view_get_buffer(textview)
   i = gtk_text_buffer_get_selection_bounds(buffer,c_loc(start),c_loc(end))
   call gtk_text_buffer_apply_tag_by_name(buffer, trim(tagname)//CNULL, &
                                                  c_loc(start), c_loc(end))
end subroutine sform
recursive subroutine size_changed(combo, textview) bind(C)
!
! apply the selected text size property as the tag
!
   use iso_c_binding

   use gtk, only: gtk_combo_box_get_active, &
                  gtk_combo_box_text_get_active_text, &
                  gtk_combo_box_set_active
 
   use g, only:   g_object_set_data

   type(c_ptr), value :: combo, textview 
   type(c_ptr) :: text

   interface
     subroutine sform(widget, textview) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, textview 
     end subroutine
   end interface
 
   character(len=1), parameter :: CNULL = C_NULL_CHAR
   character(len=128) :: string = ' '

   if( gtk_combo_box_get_active(combo) == -1 ) return

   ! return the currently active string in combo_box
   text = gtk_combo_box_text_get_active_text(combo)
   call c_f_string(text, string)
   write(*,*)'active text>',trim(string),'<'

   ! void g_object_set_data (GObject *object, const gchar *key, gpointer data)
   call g_object_set_data(combo, 'tag'//CNULL, text)
   call sform(combo, textview)

   call gtk_combo_box_set_active(combo, -1)

end subroutine size_changed
subroutine clear_clicked(button, textview) bind(C)
!
! remove all of the tags from the selected text
!
   use iso_c_binding

   use gtk, only: gtk_text_view_get_buffer, &
                  gtk_text_buffer_get_selection_bounds, &
                  gtk_text_buffer_remove_all_tags
 
   type(c_ptr), value :: button, textview 

   ! define a GtkTextIter type
   type, bind(c) :: gtktextiter 
     type(c_ptr) :: d1, d2
     integer(kind=c_int) :: d3, d4, d5, d6, d7, d8
     type(c_ptr) :: d9, d10
     integer(kind=c_int) :: d11, d12, d13
     type(c_ptr) :: d14
   end type gtktextiter

   type(gtktextiter), target :: start, end
   type(c_ptr)    :: buffer
   integer(c_int) :: i
 
   buffer = gtk_text_view_get_buffer(textview)
   i = gtk_text_buffer_get_selection_bounds(buffer, c_loc(start), c_loc(end))
   call gtk_text_buffer_remove_all_tags(buffer, c_loc(start), c_loc(end))
 
end subroutine clear_clicked


