!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2013
!
program childwidgets

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_text_view_new, &
                  gtk_text_view_get_buffer, &
                  gtk_text_buffer_set_text, &
                  gtk_text_buffer_get_iter_at_offset, &
                  gtk_text_buffer_create_child_anchor, &
                  gtk_button_new_with_label, &
                  gtk_text_view_add_child_at_anchor, &
                  gtk_scrolled_window_new, &
                  gtk_scrolled_window_set_policy, &
                  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtk_main_quit, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE

   use gtk_os_dependent, only: gdk_pixbuf_new_from_file

   implicit none

   ! define a GtkTextIter type
   type, bind(c) :: gtktextiter 
     type(c_ptr) :: d1, d2
     integer(kind=c_int) :: d3, d4, d5, d6, d7, d8
     type(c_ptr) :: d9, d10
     integer(kind=c_int) :: d11, d12, d13
     type(c_ptr) :: d14
   end type gtktextiter

   character(len=1), parameter :: CNULL = C_NULL_CHAR
   character(len=1), parameter :: CR    = C_CARRIAGE_RETURN

   type(c_ptr) :: window, textview, buffer, anchor, button, scrolled_window
   type(gtktextiter), target :: iter

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Child Widgets'//CNULL)
   call gtk_container_set_border_width(window,10)
   call gtk_widget_set_size_request(window, 250, 100)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)

   textview = gtk_text_view_new()
   buffer   = gtk_text_view_get_buffer(textview)

   !                                     0    12345678
   call gtk_text_buffer_set_text(buffer, CR//' Click  to exit!'//CNULL, -1)

   ! create a new child widget anchor at the specified iterator
   call gtk_text_buffer_get_iter_at_offset (buffer, c_loc(iter), 8)
   anchor = gtk_text_buffer_create_child_anchor (buffer, c_loc(iter))

   ! insert a GtkButton widget at the child anchor
   button = gtk_button_new_with_label('the button'//CNULL)
   call gtk_text_view_add_child_at_anchor(textview, button, anchor)

   call g_signal_connect_swapped(button,'clicked'//CNULL, &
                                 c_funloc(gtk_widget_destroy), window)

   scrolled_window = gtk_scrolled_window_new(C_NULL_PTR, C_NULL_PTR)
   call gtk_container_add(scrolled_window, textview)
   call gtk_scrolled_window_set_policy(scrolled_window, &
                                       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS)

   call gtk_container_add(window, scrolled_window)
   call gtk_widget_show_all(window)

   call gtk_main()

end

