!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2013
!
program liststore

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_tree_view_new, & 
                  gtk_tree_view_set_model, &                 
                  gtk_scrolled_window_new, &
                  gtk_scrolled_window_set_policy, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_POLICY_AUTOMATIC, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtk_main_quit, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE 

   use g, only:   g_object_unref

   implicit none

   type, bind(c) :: gtktreeiter
     integer(kind=c_int) :: stamp = 0  ! unique stamp to catch invalid iterators
     type(c_ptr) :: ptr0 = C_NULL_PTR  ! model-specific data
     type(c_ptr) :: ptr1 = C_NULL_PTR
     type(c_ptr) :: ptr2 = C_NULL_PTR
   end type gtktreeiter

   interface
     ! GtkTreeStore *gtk_tree_store_new(gint n_columns, ...)
     function gtk_tree_store_new(n_columns, int1, int2, int3) bind(C)
       use iso_c_binding, only : c_ptr, c_int, c_long
       type(c_ptr) :: gtk_tree_store_new
       integer(c_int), value  :: n_columns
       integer(c_long), value :: int1, int2, int3
     end function
     ! GtkListStore *gtk_list_store_new(gint n_columns, ...)
     function gtk_list_store_new(n_columns, int1, int2, int3) bind(C)
       use iso_c_binding, only : c_ptr, c_int, c_long
       type(c_ptr) :: gtk_list_store_new
       integer(c_int), value  :: n_columns
       integer(c_long), value :: int1, int2, int3
     end function
     ! GtkListStore *gtk_list_store_append(GtkListStore *list_store, GtkTreeIter *iter)
     subroutine gtk_list_store_append(store, iter) bind(C)
       use iso_c_binding, only : c_ptr, c_int, c_long, c_null_ptr
       type(c_ptr), value :: store, iter
     end subroutine
     ! void gtk_list_store_set(GtkListStore *list_store, GtkTreeIter *iter, ...)
     subroutine gtk_list_store_set(store, iter, &
                                   int1, int2, int3, int4, int5, string, idum) bind(C)
       use iso_c_binding, only : c_ptr, c_int, c_char, c_null_ptr
       type(c_ptr), value :: store, iter
       integer(c_int), value :: int1, int2, int3, int4, int5, idum
       character(kind=c_char), dimension(*) :: string
     end subroutine
     function G_TYPE_BOOLEAN() bind(C,name="f03_G_TYPE_BOOLEAN")
       use iso_c_binding, only : c_long
       integer(c_long) :: G_TYPE_BOOLEAN
     end function
     function G_TYPE_INT() bind(C,name="f03_G_TYPE_INT")
        use iso_c_binding, only : c_long
       integer(c_long) :: G_TYPE_INT
     end function
     function G_TYPE_STRING() bind(C,name="f03_G_TYPE_STRING")
       use iso_c_binding, only : c_long
       integer(c_long) :: G_TYPE_STRING
     end function

   end interface

   character(len=1), parameter :: CNULL = C_NULL_CHAR
   character(len=1), parameter :: CR    = C_CARRIAGE_RETURN

   enum, bind(c) 
     enumerator :: BUY_IT = 0 
     enumerator :: QUANTITY
     enumerator :: PRODUKT
     enumerator :: COLUMNS
   end enum
     
   type, bind(C) :: GroceryItem 
     integer(c_int) :: buy
     integer(c_int) :: quantity
     character(len=13,kind=c_char) :: produkt 
   end type

   integer, parameter :: Ngroceries = 7
   type(groceryitem), dimension(Ngroceries) :: list = (/ &
      GroceryItem(gtkTRUE,  1, 'Paper Towels'//CNULL), &
      GroceryItem(gtkTRUE,  2, 'Bread'//CNULL), &
      GroceryItem(gtkFALSE, 1, 'Butter'//CNULL), &
      GroceryItem(gtkTRUE,  1, 'Milk'//CNULL), &
      GroceryItem(gtkFALSE, 3, 'Chips'//CNULL), &
      GroceryItem(gtkTRUE,  4, 'Soda'//CNULL), &
      GroceryItem(gtkFALSE, 0, CNULL) /)

   type(c_ptr) :: window, treeview, store, scrolled_window
   type(gtktreeiter), target :: iter
   integer     :: i

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Grocery List'//CNULL)
   call gtk_container_set_border_width(window,10)
   call gtk_widget_set_size_request(window, 250, 150)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)

   treeview = gtk_tree_view_new()
   call setup_tree_view(treeview)
  
   ! create a new tree model with three columns, as string, gint and guint
   store = gtk_list_store_new(COLUMNS, G_TYPE_BOOLEAN(), G_TYPE_INT(), & 
                                       G_TYPE_STRING() )
  
   ! add all of the products to the GtkListStore
   i = 1
   do
     if (list(i)%produkt == CNULL) exit

     call gtk_list_store_append(store, c_loc(iter))
     call gtk_list_store_set(store, c_loc(iter), BUY_IT, list(i)%buy, &
                       QUANTITY, list(i)%quantity, PRODUKT, list(i)%produkt, -1)
     i = i + 1
   end do
  
   ! add the tree model to the tree view and unreference it so that the 
   ! model will be destroyed along with the tree view
   call gtk_tree_view_set_model(treeview, store)
   call g_object_unref(store)
  
   scrolled_window = gtk_scrolled_window_new(C_NULL_PTR, C_NULL_PTR)
   call gtk_scrolled_window_set_policy(scrolled_window, &
                                     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC)

   call gtk_container_add(scrolled_window, treeview)
   call gtk_container_add(window, scrolled_window)
   call gtk_widget_show_all(window)

   call gtk_main()

end
subroutine setup_tree_view(treeview)
!
! add three columns to the GtkTreeView; all three of the columns 
! will be displayed as text, although one is a gboolean value and
! another is an integer
!
   use iso_c_binding

   use gtk, only: gtk_cell_renderer_text_new, &
                  gtk_tree_view_append_column

   interface
     !
     ! GtkTreeViewColumn *gtk_tree_view_column_new_with_attributes 
     !   (const gchar *title, GtkCellRenderer *cell, ...) G_GNUC_NULL_TERMINATED
     !
     function gtk_tree_view_column_new_with_attributes &
                            (string1, ptr1, string2, int2, eod) bind(C)
       use iso_c_binding, only : c_ptr, c_int, c_char
       type(c_ptr) :: gtk_tree_view_column_new_with_attributes
       character(kind=c_char), dimension(*) :: string1
       type(c_ptr), value :: ptr1
       character(kind=c_char), dimension(*) :: string2
       integer(c_int), value :: int2
       type(c_ptr), value :: eod
     end function
   end interface

   character(len=1), parameter :: CNULL = C_NULL_CHAR
   
   enum, bind(c) 
     enumerator :: BUY_IT = 0 
     enumerator :: QUANTITY
     enumerator :: PRODUKT
     enumerator :: COLUMNS
   end enum

   type(c_ptr) :: treeview
   type(c_ptr) :: renderer, column
   integer(c_int) :: i
   
   ! create a new GtkCellRendererText, add it to the tree view column 
   ! and append the column to the tree view
   renderer = gtk_cell_renderer_text_new()
   column   = gtk_tree_view_column_new_with_attributes &
                  ('Get it'//CNULL, renderer, 'text'//CNULL, BUY_IT, c_null_ptr)
   i = gtk_tree_view_append_column(treeview, column)
  
   renderer = gtk_cell_renderer_text_new()
   column   = gtk_tree_view_column_new_with_attributes &
                  ('Count'//CNULL, renderer, 'text'//CNULL, QUANTITY, c_null_ptr)
   i = gtk_tree_view_append_column(treeview, column)
  
   renderer = gtk_cell_renderer_text_new ()
   column   = gtk_tree_view_column_new_with_attributes &
                  ('Stuff'//CNULL, renderer, 'text'//CNULL, PRODUKT, c_null_ptr)
   i = gtk_tree_view_append_column(treeview, column)

end subroutine



