!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2013
!
module c08_06_selections_module

   use iso_c_binding

   use gtk, only: gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE 

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   type, bind(c) :: gtktreeiter
     integer(kind=c_int) :: stamp = 0  ! unique stamp to catch invalid iterators
     type(c_ptr) :: ptr0 = C_NULL_PTR  ! model-specific data
     type(c_ptr) :: ptr1 = C_NULL_PTR
     type(c_ptr) :: ptr2 = C_NULL_PTR
   end type gtktreeiter

   enum, bind(c) 
     enumerator :: BUY_IT = 0 
     enumerator :: QUANTITY
     enumerator :: PRODUKT
     enumerator :: COLUMNS
   end enum

   enum, bind(c) 
     enumerator :: PRODUKT_CATEGORY
     enumerator :: PRODUKT_CHILD
   end enum
     
   type, bind(C) :: GroceryItem 
     integer(c_int) :: produkt_type
     integer(c_int) :: buy
     integer(c_int) :: quantity
     character(len=13,kind=c_char) :: produkt 
   end type

   integer, parameter :: Ngroceries = 10
   type(groceryitem), dimension(Ngroceries) :: list = (/ &
      GroceryItem(PRODUKT_CATEGORY, gtkTRUE,  0, 'Cleaning Supplies'//CNULL), &
      GroceryItem(PRODUKT_CHILD,    gtkTRUE,  1, 'Paper Towels'//CNULL), &
      GroceryItem(PRODUKT_CHILD,    gtkTRUE,  3, 'Toilet Paper'//CNULL), &
      GroceryItem(PRODUKT_CATEGORY, gtkTRUE,  0, 'Food'//CNULL), &
      GroceryItem(PRODUKT_CHILD,    gtkTRUE,  2, 'Bread'//CNULL), &
      GroceryItem(PRODUKT_CHILD,    gtkFALSE, 1, 'Butter'//CNULL), &
      GroceryItem(PRODUKT_CHILD,    gtkTRUE,  1, 'Milk'//CNULL), &
      GroceryItem(PRODUKT_CHILD,    gtkFALSE, 3, 'Chips'//CNULL), &
      GroceryItem(PRODUKT_CHILD,    gtkTRUE,  4, 'Soda'//CNULL), &
      GroceryItem(PRODUKT_CATEGORY, gtkFALSE, 0, CNULL) /)

end module
program selections

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_tree_view_new, & 
                  gtk_tree_view_set_model, &                 
                  gtk_tree_view_expand_all, &
                  gtk_tree_store_append, &
                  gtk_scrolled_window_new, &
                  gtk_tree_view_get_selection, &
                  GTK_SELECTION_MULTIPLE, &
                  gtk_button_new_from_stock, &
                  gtk_scrolled_window_set_policy, &
                  gtk_box_new, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_POLICY_AUTOMATIC, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtk_main_quit

   use g, only:   g_object_unref

   use c08_06_selections_module

   implicit none

   interface
     ! local subroutines
     subroutine add_produkt(add, treeview) bind(C)
       use iso_c_binding, only : c_ptr 
       type(c_ptr), value :: add, treeview
     end subroutine
     subroutine remove_produkts(remove, treeview) bind(C)
       use iso_c_binding, only : c_ptr 
       type(c_ptr), value :: remove, treeview
     end subroutine
     ! GtkTreeStore *gtk_tree_store_new(gint n_columns, ...)
     function gtk_tree_store_new(n_columns, int1, int2, int3) bind(C)
       use iso_c_binding, only : c_ptr, c_int, c_long
       type(c_ptr) :: gtk_tree_store_new
       integer(c_int), value  :: n_columns
       integer(c_long), value :: int1, int2, int3
     end function

     ! void gtk_tree_store_set(GtkTreeStore *tree_store, GtkTreeIter *iter, ...)
     subroutine gtk_tree_store_set(store, iter, &
                                   int1, int2, int3, int4, int5, string, idum) bind(C)
       use iso_c_binding, only : c_ptr, c_int, c_char, c_null_ptr
       type(c_ptr), value :: store, iter
       integer(c_int), value :: int1, int2, int3, int4, int5, idum
       character(kind=c_char), dimension(*) :: string
     end subroutine
     function G_TYPE_BOOLEAN() bind(C,name="f03_G_TYPE_BOOLEAN")
       use iso_c_binding, only : c_long
       integer(c_long) :: G_TYPE_BOOLEAN
     end function
     function G_TYPE_INT() bind(C,name="f03_G_TYPE_INT")
        use iso_c_binding, only : c_long
       integer(c_long) :: G_TYPE_INT
     end function
     function G_TYPE_STRING() bind(C,name="f03_G_TYPE_STRING")
       use iso_c_binding, only : c_long
       integer(c_long) :: G_TYPE_STRING
     end function

   end interface

   type(c_ptr) :: window, treeview, store, scrolled_window
   type(c_ptr) :: selection, add, remove, hbox, vbox
   type(gtktreeiter), target :: iter, child
   integer     :: i = 0, j

   character(len=*), parameter :: GTK_STOCK_ADD     = 'gtk-add'//CNULL
   character(len=*), parameter :: GTK_STOCK_REMOVE  = 'gtk-remove'//CNULL

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Grocery List 3'//CNULL)
   call gtk_container_set_border_width(window,10)
   call gtk_widget_set_size_request(window, 275, 300)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)

   treeview = gtk_tree_view_new()
   call setup_tree_view(treeview)
  
   ! create a new tree model with three columns, as string, gint and guint
   store = gtk_tree_store_new(COLUMNS, G_TYPE_BOOLEAN(), G_TYPE_INT(), & 
                                       G_TYPE_STRING() )
  
   ! add all of the products to the GtkListStore
   i = 1
   do
     if( list(i)%produkt == CNULL ) exit           ! style preferred by Metcalf

     ! if the product type is a category, count the quantity of all of the 
     ! products in the category that are going to be bought
     if( list(i)%produkt_type == PRODUKT_CATEGORY )then
       j = i + 1
      
       ! calculate how many products will be bought in the category 
       do while( list(j)%produkt      /= CNULL  .and.     &
                 list(j)%produkt_type /= PRODUKT_CATEGORY )
         if( list(j)%buy == gtkTRUE )then
           list(i)%quantity = list(i)%quantity + list(j)%quantity
         endif
         j = j + 1
       end do
      
       ! add the category as a new root element
       call gtk_tree_store_append(store, c_loc(iter), C_NULL_PTR)
       call gtk_tree_store_set(store, c_loc(iter),         &
                               BUY_IT,   list(i)%buy,      &
                               QUANTITY, list(i)%quantity, &
                               PRODUKT,  list(i)%produkt, -1)   
     else
       ! otherwise, add the product as a child of the category   
       call gtk_tree_store_append(store, c_loc(child), c_loc(iter))
       call gtk_tree_store_set(store, c_loc(child),        &
                               BUY_IT,   list(i)%buy,      &
                               QUANTITY, list(i)%quantity, &
                               PRODUKT,  list(i)%produkt, -1)
     endif
    
     i = i + 1

   end do
  
   call gtk_tree_view_set_model(treeview, store)
   call gtk_tree_view_expand_all(treeview)
   call g_object_unref(store)
  
   ! allow multiple rows to be selected at the same time
   selection = gtk_tree_view_get_selection(treeview)
   call gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE)

   scrolled_window = gtk_scrolled_window_new(C_NULL_PTR, C_NULL_PTR)
   call gtk_scrolled_window_set_policy(scrolled_window, &
                         GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC)

   add    = gtk_button_new_from_stock(GTK_STOCK_ADD)
   remove = gtk_button_new_from_stock(GTK_STOCK_REMOVE)
  
   call g_signal_connect(add, 'clicked'//CNULL, &
                         c_funloc(add_produkt), treeview)
   call g_signal_connect(remove, 'clicked'//CNULL, &
                         c_funloc(remove_produkts), treeview)
   
   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5)
   call gtk_box_pack_start(hbox, add,    gtkFALSE, gtkTRUE, 0)
   call gtk_box_pack_start(hbox, remove, gtkFALSE, gtkTRUE, 0)
  
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5)
   call gtk_box_pack_start(vbox, scrolled_window, gtkTRUE, gtkTRUE, 0)
   call gtk_box_pack_start(vbox, hbox, gtkFALSE, gtkTRUE, 0)

   call gtk_container_add(scrolled_window, treeview)
   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end
subroutine add_produkt(add, treeview) bind(C)

   use iso_c_binding

   use gtk, only: gtk_combo_box_text_new, &
                  gtk_entry_new, &
                  gtk_spin_button_new_with_range, &
                  gtk_check_button_new_with_mnemonic, &
                  gtk_spin_button_set_digits, &
                  gtk_combo_box_text_append, &
                  gtk_grid_new, &
                  gtk_grid_set_row_spacing, &
                  gtk_grid_set_column_spacing, &
                  gtk_container_set_border_width, &
                  gtk_grid_attach, &
                  gtk_label_new, &
                  gtk_widget_show_all, &
                  gtk_dialog_run, &
                  gtk_spin_button_get_value, &
                  gtk_entry_get_text, &
                  gtk_combo_box_text_get_active_text, &
                  gtk_toggle_button_get_active, &
                  gtk_widget_destroy, &
                  gtk_tree_view_get_model, &
                  gtk_tree_model_get_iter_from_string, &
                  gtk_tree_model_iter_next, &
                  gtk_tree_model_get_path, &
                  gtk_tree_store_append, &
                  gtk_tree_model_get_iter, &
                  gtk_tree_path_free, &
                  gtk_widget_destroy, &
                  GTK_DIALOG_MODAL, &
                  GTK_RESPONSE_OK, &
                  GTK_RESPONSE_CANCEL
      
   use g, only:   g_ascii_strcasecmp, &
                  g_free                  

   use c08_06_selections_module

   interface
     function gtk_dialog_new_with_buttons_f03(title,parent,flags, &
                                              button1, response1, &
                                              button2, response2, &
                                              button3, response3)
       use iso_c_binding
       type(c_ptr) :: gtk_dialog_new_with_buttons_f03
       character(len=*), intent(in)  :: title
       type(c_ptr), intent(in)       :: parent
       integer(c_int), intent(in)    :: flags
       character(len=*), intent(in), optional :: button1, button2, button3
       integer, intent(in), optional ::          response1, response2, response3
     end function

     ! variadic functions
     ! gtk_tree_model_get (GtkTreeModel *tree_model, GtkTreeIter *iter, ...);
     subroutine gtk_tree_model_get(model, iter, int1, int2, int3) bind(C)
       import 
       type(c_ptr), value     :: model
       type(gtktreeiter)      :: iter
       integer(c_long), value :: int1, int2, int3
     end subroutine
     ! gtk_tree_store_set (GtkTreeStore *tree_store, GtkTreeIter *iter, ...); 
     subroutine gtk_tree_store_set(store, iter, int1, int2, int3) bind(C)
       import
       type(c_ptr), value     :: store
       type(gtktreeiter)      :: iter
       integer(c_long), value :: int1, int2, int3
     end subroutine

   end interface

   type(c_ptr) :: add, treeview

   character(len=*), parameter :: GTK_STOCK_ADD    = 'gtk-add'//CNULL
   character(len=*), parameter :: GTK_STOCK_CANCEL = 'gtk-cancel'//CNULL

   type(c_ptr) :: dialog, table, combobox, item, spin, check

   type(GtkTreeIter), target :: iter, child

   type(c_ptr) :: path
   type(c_ptr) :: model
   type(c_ptr), target :: c_string
   character(len=12), target :: produkt_string, category, naam
   integer(c_int) :: kwantity, i = 0, j
   integer(c_int) :: buy
  
   ! create a dialog that will be used to create a new product
   dialog = gtk_dialog_new_with_buttons_f03('Add a Product'//CNULL, C_NULL_PTR, &
                                            GTK_DIALOG_MODAL, &
                                            GTK_STOCK_ADD, GTK_RESPONSE_OK, &
                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL)

   ! create widgets that will be packed into the dialog
   combobox = gtk_combo_box_text_new()
   item     = gtk_entry_new()
   spin     = gtk_spin_button_new_with_range(dble(0.0), dble(100.0), dble(1.0))
   check    = gtk_check_button_new_with_mnemonic('_Buy the Product'//CNULL)
   
   call gtk_spin_button_set_digits(spin, 0)
  
   ! add all of the categories to the combo box
   do while( list(i)%produkt /= CNULL )
     if( list(i)%produkt_type == PRODUKT_CATEGORY )then
       call gtk_combo_box_text_append(combobox, CNULL, list(i)%produkt)
     endif
     i = i + 1
   end do
  
   table = gtk_grid_new()
   call gtk_grid_set_row_spacing(table, 5)
   call gtk_grid_set_column_spacing(table, 5)
   call gtk_container_set_border_width(table, 5)
  
   ! pack the table that will hold the dialog widgets
   call gtk_grid_attach(table, gtk_label_new('Category:'//CNULL), 0, 1, 0, 1)
   call gtk_grid_attach(table, combobox, 1, 2, 0, 1)
   call gtk_grid_attach(table, gtk_label_new('Product:'//CNULL),  0, 1, 1, 2)
   call gtk_grid_attach(table, item,     1, 2, 1, 2)
   call gtk_grid_attach(table, gtk_label_new('Quantity:'//CNULL), 0, 1, 2, 3)
   call gtk_grid_attach(table, spin,     1, 2, 2, 3)
   call gtk_grid_attach(table, check,    1, 2, 3, 4)
  
  !call gtk_box_pack_start_defaults(GTK_BOX (GTK_DIALOG (dialog)->vbox), table)
   call gtk_container_add(dialog,table)
   call gtk_widget_show_all(dialog)

   ! if the user presses OK, verify the entries and add the product
   if( gtk_dialog_run(dialog) == GTK_RESPONSE_OK )then
  
     kwantity = transfer(gtk_spin_button_get_value(spin),kwantity)
     c_string  = gtk_entry_get_text(item)
     call c_f_string(c_string,produkt_string)

     c_string = gtk_combo_box_text_get_active_text(combobox)
     call c_f_string(c_string,category)

     buy      = gtk_toggle_button_get_active(check)
    
     if( g_ascii_strcasecmp(produkt_string, ''//CNULL) == 0 .or. category == CNULL )then
    
       write(*,*) 'All of the fields were not correctly filled out!'
       call gtk_widget_destroy (dialog)
      
       if( category /= CNULL) call g_free(c_string)
       return

     endif
    
     model = gtk_tree_view_get_model(treeview)

     ! gboolean gtk_tree_model_get_iter_from_string(tree_model, iter, path_string)
     j = gtk_tree_model_get_iter_from_string(model,c_loc(iter), '0'//CNULL)
    
     ! retrieve an iterator pointing to the selected category
     do
    
       call gtk_tree_model_get(model, iter, int(PRODUKT,c_long), &
                                            c_loc(c_string), -1)
       call c_f_string(c_string,naam)
     
       if( g_ascii_strcasecmp(naam, category) == 0 )then
         call g_free(c_string)
         exit
       endif
      
       call g_free(c_string)

       if( gtk_tree_model_iter_next(model, c_loc(iter)) == 0 ) exit
     end do
     
     ! convert the category iterator to a path so that it will not become 
     ! invalid and add the new product as a child of the category
     path = gtk_tree_model_get_path (model, c_loc(iter))
     call gtk_tree_store_append(model, c_loc(child), c_loc(iter))
     call gtk_tree_store_set(model, c_loc(child), BUY_IT, buy, & 
                             QUANTITY, kwantity, PRODUKT, produkt, -1)
    
     ! add the quantity to the running total if it is to be purchased
     if( buy == gtkTRUE )then
    
       j = gtk_tree_model_get_iter(model, c_loc(iter), path)
       call gtk_tree_model_get(model, c_loc(iter), QUANTITY, c_loc(i), -1)
       i = i + kwantity
       call gtk_tree_store_set(model, c_loc(iter), QUANTITY, i, -1)
    
     endif
    
     call gtk_tree_path_free(path)
     call g_free(category)

   endif  

   call gtk_widget_destroy(dialog)

end subroutine add_produkt
subroutine remove_row(ref, model)

   use iso_c_binding

   use gtk, only: gtk_tree_row_reference_get_path, &
                  gtk_tree_model_get_iter, &
                  gtk_tree_model_iter_parent, &
                  gtk_tree_store_remove

   use c08_06_selections_module

   interface
     ! variadic functions
     ! gtk_tree_model_get (GtkTreeModel *tree_model, GtkTreeIter *iter, ...);
     subroutine gtk_tree_model_get(model, iter, int1, int2, int3) bind(C)
       import 
       type(c_ptr)       :: model
       type(gtktreeiter) :: iter
       integer(c_long), value :: int1, int2, int3
     end subroutine
     ! gtk_tree_store_set (GtkTreeStore *tree_store, GtkTreeIter *iter, ...); 
     subroutine gtk_tree_store_set(store, iter, int1, int2, int3) bind(C)
       import
       type(c_ptr)       :: store
       type(gtktreeiter) :: iter
       integer(c_long), value :: int1, int2, int3
     end subroutine
   end interface

   type(c_ptr) :: ref, model
 
   type(GtkTreeIter), target :: parent, iter

   type(c_ptr)    :: path
   integer(c_int) :: buy
   integer(c_int) :: pnum
  
   path = gtk_tree_row_reference_get_path(ref)
   call gtk_tree_model_get_iter(model, c_loc(iter), path)

   ! only remove the row if it is not a root row
   if( gtk_tree_model_iter_parent(model, c_loc(parent), c_loc(iter)) )then
  
     call gtk_tree_model_get(model, c_loc(iter), BUY_IT, c_loc(buy), &
                             QUANTITY, c_loc(quantity), -1)
     call gtk_tree_model_get(model, c_loc(parent), QUANTITY, c_loc(pnum), -1)
    
     if( buy == gtkTRUE )then
       pnum = pnum - quantity
       call gtk_tree_store_set(model, c_loc(parent), QUANTITY, pnum, -1)
     endif
    
     call gtk_tree_model_get_iter(model, c_loc(iter), path)
     call gtk_tree_store_remove(model, c_loc(iter))    

   endif
   
end subroutine remove_row
subroutine remove_produkts(remove, treeview) bind(C)

   use iso_c_binding

   use gtk, only: gtk_tree_view_get_selection, &
                  gtk_tree_view_get_model, &
                  gtk_tree_selection_get_selected_rows, &
                  gtk_tree_row_reference_new, &
                  gtk_tree_row_reference_copy, &
                  gtk_tree_row_reference_free

   use g, only:   g_list_prepend, &
                  g_list_foreach, &
                  g_list_free

   use c08_06_selections_module

   type(c_ptr) :: remove, treeview
   type(c_ptr) :: selection, ref, model
  !GList *rows, *ptr, *references = NULL
  
   selection = gtk_tree_view_get_selection(treeview)
   model     = gtk_tree_view_get_model(treeview)
   rows      = gtk_tree_selection_get_selected_rows(selection, c_loc(model))
  
   ! create tree row references to all of the selected rows
   ptr = rows
   do 
     if( ptr /= CNULL ) exit
  
!    ref = gtk_tree_row_reference_new(model, (GtkTreePath*) ptr->data )
!    references = g_list_prepend(references, gtk_tree_row_reference_copy(ref) )
!    call gtk_tree_row_reference_free(ref)
!    ptr = ptr->next

   enddo
   
   ! remove each of the selected rows pointed to by the row reference
   call g_list_foreach (references, c_funloc(remove_row), model)

   ! free the tree paths, tree row references and lists
   call g_list_foreach(references, c_funloc(gtk_tree_row_reference_free), CNULL)
   call g_list_foreach(rows, c_funloc(gtk_tree_path_free), CNULL)
   call g_list_free(references)
   call g_list_free(rows)

end subroutine remove_produkts
subroutine setup_tree_view(treeview)
!
! add three columns to the GtkTreeView; all three of the columns 
! will be displayed as text, although one is a gboolean value and
! another is an integer
!
   use iso_c_binding

   use gtk, only: gtk_cell_renderer_text_new, &
                  gtk_tree_view_append_column

   use c08_06_selections_module

   interface
     !
     ! GtkTreeViewColumn *gtk_tree_view_column_new_with_attributes 
     !   (const gchar *title, GtkCellRenderer *cell, ...) G_GNUC_NULL_TERMINATED
     !
     function gtk_tree_view_column_new_with_attributes &
                            (string1, ptr1, string2, int2, eod) bind(C)
       use iso_c_binding, only : c_ptr, c_int, c_char
       type(c_ptr) :: gtk_tree_view_column_new_with_attributes
       character(kind=c_char), dimension(*) :: string1
       type(c_ptr), value :: ptr1
       character(kind=c_char), dimension(*) :: string2
       integer(c_int), value :: int2
       type(c_ptr), value :: eod
     end function
   end interface

   type(c_ptr) :: treeview
   type(c_ptr) :: renderer, column
   integer(c_int) :: i
   
   ! create a new GtkCellRendererText, add it to the tree view column 
   ! and append the column to the tree view
   renderer = gtk_cell_renderer_text_new()
   column   = gtk_tree_view_column_new_with_attributes &
                  ('Get it'//CNULL, renderer, 'text'//CNULL, BUY_IT, c_null_ptr)
   i = gtk_tree_view_append_column(treeview, column)
  
   renderer = gtk_cell_renderer_text_new()
   column   = gtk_tree_view_column_new_with_attributes &
                  ('Count'//CNULL, renderer, 'text'//CNULL, QUANTITY, c_null_ptr)
   i = gtk_tree_view_append_column(treeview, column)
  
   renderer = gtk_cell_renderer_text_new ()
   column   = gtk_tree_view_column_new_with_attributes &
                  ('Stuff'//CNULL, renderer, 'text'//CNULL, PRODUKT, c_null_ptr)
   i = gtk_tree_view_append_column(treeview, column)

end subroutine setup_tree_view
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function gtk_dialog_new_with_buttons_f03(title, parent, modal, &
                                         button1, response1,   &
                                         button2, response2,   &
                                         button3, response3)
   use iso_c_binding

   use gtk, only: gtk_dialog_new, &
                  GTK_DIALOG_MODAL, &
                  GTK_DIALOG_DESTROY_WITH_PARENT, &
                  gtk_label_new, &
                  gtk_window_set_title, &
                  gtk_window_set_modal, &
                  gtk_window_set_transient_for, &
                  gtk_window_set_destroy_with_parent, &
                  gtk_box_new, &
                  GTK_ORIENTATION_VERTICAL, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  gtk_dialog_add_button, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  TRUE, FALSE

   type(c_ptr) :: gtk_dialog_new_with_buttons_f03
   type(c_ptr) :: dialog

   character(len=*), intent(in)           :: title
   type(c_ptr), intent(in), optional      :: parent
   integer(c_int), intent(in)             :: modal
   character(len=*), intent(in), optional :: button1, button2, button3
   integer, intent(in), optional          :: response1, response2, response3

   type(c_ptr) :: ib1, ib2, ib3

   dialog = gtk_dialog_new()

   ! if window is set modal, input will be grabbed when show and released when hide
   ! void gtk_window_set_modal (GtkWindow *window, gboolean modal);
   ! gboolean gtk_window_get_modal (GtkWindow *window);

   if( modal == GTK_DIALOG_MODAL )then
     !write(*,*)'modal set',modal
     call gtk_window_set_modal(dialog, TRUE)
   elseif( modal == GTK_DIALOG_DESTROY_WITH_PARENT )then
     !write(*,*)'modal false',modal
     call gtk_window_set_modal(dialog, FALSE)
   else
     write(*,*)'error: unknown modal',modal
     call gtk_window_set_modal(dialog, FALSE)
   endif

   call gtk_window_set_title(dialog, title(1:len_trim(title))//C_NULL_CHAR)

   if( present(parent)  )then
     write(*,*) 'parent present'
     call gtk_window_set_transient_for(dialog, parent)
     call gtk_window_set_destroy_with_parent(dialog, TRUE)
   endif

   if( present(button1) )then
     if( present(response1) )then
       ib1 = gtk_dialog_add_button(dialog, &
                                   button1(1:len_trim(button1))//C_NULL_CHAR, &
                                   response1)
     else
       write(*,*) 'error: missing response1'
     endif
   endif
   if( present(button2) )then
     if( present(response2) )then
       ib1 = gtk_dialog_add_button(dialog, &
                                   button2(1:len_trim(button2))//C_NULL_CHAR, &
                                   response2)
     else
       write(*,*) 'error: missing response2'
     endif
   endif
   if( present(button3) )then
     if( present(response3) )then
       ib1 = gtk_dialog_add_button(dialog, &
                                   button3(1:len_trim(button3))//C_NULL_CHAR, &
                                   response3)
     else
       write(*,*) 'error: missing response3'
     endif
   endif

   gtk_dialog_new_with_buttons_f03 = dialog

end function gtk_dialog_new_with_buttons_f03


