!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program exercise

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  gtk_window_set_resizable, &
                  g_signal_connect, &
                  gtk_label_new, &
                  gtk_label_set_selectable, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  GTKTRUE => TRUE, &
                  GTKFALSE => FALSE

   implicit none

   !external my_destroy, my_key_event
   interface
     subroutine my_destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine

     function my_key_event(window,event,label) bind(C)
       use iso_c_binding
       integer(c_int) :: my_key_event
       type(c_ptr), value :: window, event, label
     end function my_key_event
   end interface

   type(c_ptr) :: window, label

   ! very oldschool but easy and simple
   character(len=9) :: STRING1
   character(len=9) :: STRING2
   common/block/ STRING1, STRING2

   STRING1 = 'String 1'//C_NULL_CHAR
   STRING2 = 'String 2'//C_NULL_CHAR

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,STRING1)

   call gtk_container_set_border_width(window,10)
   call gtk_widget_set_size_request(window,300,100)
   call gtk_window_set_resizable(window,GTKFALSE)

   ! Connect the main window to the destroy signals
   call g_signal_connect(window, 'destroy'//C_NULL_CHAR, &
                         c_funloc(my_destroy))

   ! Create a new GtkLabel widget that is selectable
   label = gtk_label_new(STRING2)
   call gtk_label_set_selectable(label,GTKTRUE)

   call g_signal_connect(window,'key-press-event'//C_NULL_CHAR, &
                         c_funloc(my_key_event),label)

   ! Add the label as a child widget of the window
   call gtk_container_add (window,label)
   call gtk_widget_show_all(window)

   ! Hand control over to the main loop
   call gtk_main()

end program exercise
!                     GtkWidget GdkEventKey GtkLabel
function my_key_event(window,   event,      label) bind(C)
!
! This callback function is called every time key-press-event
! is emitted from the main GtkWindow.
!
   use iso_c_binding

   use gtk, only: gtk_window_set_title, &
                  gtk_label_set_text,   &
                  gtk_label_get_text

   integer(c_int) :: my_key_event
   type(c_ptr), value :: window, event, label

   type(c_ptr) :: pointer_to_c_string
   character(len=512), pointer :: pointer_to_fortran_str

   logical, save :: toggle = .true.

   ! very oldschool but easy and simple
   character(len=9) :: STRING1
   character(len=9) :: STRING2
   common/block/ STRING1, STRING2

   ! G_CONST_RETURN gchar* gtk_label_get_text (GtkLabel *label);
   ! function gtk_label_get_text(label) bind(c)
   !   use iso_c_binding, only: c_ptr
   !   type(c_ptr) :: gtk_label_get_text
   !   type(c_ptr), value :: label
   ! end function

   pointer_to_c_string = gtk_label_get_text(label)
   call c_f_pointer(cptr=pointer_to_c_string, fptr=pointer_to_fortran_str)
   length = min(len(pointer_to_fortran_str), &
                index(pointer_to_fortran_str,char(0)))

   write(*,*) 'Label found is ',pointer_to_fortran_str(1:length)

   if( toggle )then
     call gtk_window_set_title(window,STRING2)
     call gtk_label_set_text(label,STRING1)
   else
     call gtk_window_set_title(window,STRING1)
     call gtk_label_set_text(label,STRING2)
   endif

   toggle = .not. toggle

   write(*,*) 'my_key_event'

   my_key_event = 0 !FALSE

end function my_key_event
subroutine my_destroy(widget, gdata) bind(C)
!
! static void destroy (GtkWidget*, gpointer)
!
   use iso_c_binding

   use gtk, only: gtk_main_quit

   type(c_ptr), value :: widget, gdata

   write(*,*) 'subroutine destroy'
   call gtk_main_quit()

end subroutine my_destroy

