!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program excersise

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_notebook_new, &
                  gtk_button_new_with_mnemonic, &
                  gtk_notebook_set_show_tabs, &
                  gtk_label_new, &
                  gtk_button_new_with_label, &
                  gtk_expander_new, &
                  gtk_expander_set_expanded, &
                  gtk_notebook_append_page, &
                  gtk_notebook_next_page, &
                  gtk_box_new, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  gtk_box_set_spacing, &
                  gtk_box_pack_end, &
                  gtk_paned_new, &
                  gtk_paned_pack1, &
                  gtk_paned_pack2, &
                  gtk_frame_new, &
                  gtk_frame_set_shadow_type, &
                  GTK_SHADOW_IN, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_widget_destroy, &
                  gtk_main, &
                  TRUE, FALSE

   implicit none

   character(len= 1) :: CNULL = C_NULL_CHAR
   character(len=14) :: str

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
     function button_pressed(eventbox, event, label) bind(C)
       use iso_c_binding
       integer(c_int)     :: button_pressed
       type(c_ptr), value :: event
       type(c_ptr), value :: eventbox, label
     end function button_pressed
     subroutine previous_tab(button, notebook) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button, notebook
     end subroutine
     subroutine next_tab(button, notebook) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button, notebook
     end subroutine
   end interface

   type(c_ptr) :: window, notebook, paned, hbox, previous, closeit
   type(c_ptr) :: label, button, expander, frame1, frame2

   integer(c_int) :: i1
   integer :: i

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Exercise 3-2'//CNULL)
   call gtk_container_set_border_width(window, 10)
   call gtk_widget_set_size_request(window, 250, 200)

   ! subroutine destroy in tools
   call g_signal_connect(window,'destroy'//CNULL, &
                         c_funloc(destroy))

   notebook = gtk_notebook_new()
   call gtk_notebook_set_show_tabs(notebook, FALSE)

   previous = gtk_button_new_with_mnemonic('_Previous Tab'//CNULL)
   call g_signal_connect(previous,'clicked'//CNULL, &
                         c_funloc(previous_tab), notebook)

   closeit = gtk_button_new_with_mnemonic('_Close'//CNULL)
   call g_signal_connect_swapped(closeit,'clicked'//CNULL, &
                         c_funloc(gtk_widget_destroy), window)

   ! create four GtkNotebook pages that will move to the next tab
   ! when the child GtkButton is pressed
   do i=1,4

     label  = gtk_label_new('Tab')
     button = gtk_button_new_with_mnemonic('_Next Tab'//CNULL)

     write(str,'(''This is tab '',i1,A1)') i-1,CNULL
     expander = gtk_expander_new(str)

     call gtk_expander_set_expanded(expander, TRUE)
     call gtk_container_add(expander, button)

     ! append page to the notebook container
     i1 = gtk_notebook_append_page(notebook, expander, label)
     call gtk_container_set_border_width(notebook, 10)
    !call gtk_container_set_border_width(button, 10)

     call g_signal_connect(button, 'clicked'//CNULL, &
                           c_funloc(next_tab), notebook)

   enddo

   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5)
   call gtk_container_set_border_width(hbox, 10)

   !                                     def: T   def: T   def: 0
   !                                     T/F      T/F
   !    gtk_box_pack_end(box,  child,    expand,  fill,    padding)
   call gtk_box_pack_end(hbox, closeit,  FALSE,   FALSE,   0)
   call gtk_box_pack_end(hbox, previous, FALSE,   FALSE,   0)

   paned = gtk_paned_new(GTK_ORIENTATION_VERTICAL)

   frame1 = gtk_frame_new(CNULL)
   frame2 = gtk_frame_new(CNULL)

   call gtk_frame_set_shadow_type(frame1, GTK_SHADOW_IN)
   call gtk_frame_set_shadow_type(frame2, GTK_SHADOW_IN)

   call gtk_container_add(frame1, notebook)
   call gtk_container_add(frame2, hbox)

   call gtk_paned_pack1(paned, frame1, TRUE, FALSE)
   call gtk_paned_pack2(paned, frame2, TRUE, TRUE)

   call gtk_container_add(window, paned)
   call gtk_widget_show_all(window)

   call gtk_main()

end program
subroutine previous_tab(button, notebook) bind(C)
!
! move to the previous tab in the GtkNotebook
!
   use iso_c_binding

   use gtk, only: gtk_notebook_get_current_page, &
                  gtk_notebook_set_current_page, &
                  gtk_notebook_prev_page

   type(c_ptr), value :: button, notebook
   integer(c_int)     :: page

   page = gtk_notebook_get_current_page(notebook)
   write(*,*) 'previous: got page ',page

   if( page == 0 )then
     call gtk_notebook_set_current_page(notebook, 3)
   else
     call gtk_notebook_prev_page(notebook)
   endif

end subroutine previous_tab
subroutine next_tab(button, notebook) bind(C)
!
! move to the next tab in the GtkNotebook
!
   use iso_c_binding

   use gtk, only: gtk_notebook_get_current_page, &
                  gtk_notebook_set_current_page, &
                  gtk_notebook_next_page

   type(c_ptr), value :: button, notebook
   integer(c_int)     :: page

   page = gtk_notebook_get_current_page(notebook)
   write(*,*) 'next: got page ',page

   if( page == 3 )then
     call gtk_notebook_set_current_page(notebook, 0)
   else
     call gtk_notebook_next_page(notebook)
   endif

end subroutine next_tab


