!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program main

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_label_new, &
                  gtk_button_new_from_stock, &
                  gtk_entry_new, &
                  gtk_widget_set_sensitive, &
                  gtk_file_chooser_button_new, &
                  GTK_FILE_CHOOSER_ACTION_OPEN, &
                  gtk_file_chooser_set_current_folder, &
                  gtk_box_new, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  TRUE, FALSE

   use g, only:   g_get_home_dir, g_get_current_dir

   implicit none

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   character(len=*), parameter :: GTK_STOCK_APPLY = 'gtk-apply'//CNULL

   type, bind(c) :: Widgets
     type(c_ptr) :: ffil
     type(c_ptr) :: fnam
     type(c_ptr) :: fren
   end type

   type(Widgets), target :: w

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
     subroutine file_changed(filename,w) bind(C)
       use iso_c_binding
       type(c_ptr), value :: filename
       type, bind(c) :: Widgets
         type(c_ptr) :: ffil
         type(c_ptr) :: fnam
         type(c_ptr) :: fren
       end type
       type(Widgets) :: w
     end subroutine
     subroutine rename_clicked(button,w) bind(C)
       use iso_c_binding
       type(c_ptr), value :: button
       type, bind(c) :: Widgets
         type(c_ptr) :: ffil
         type(c_ptr) :: fnam
         type(c_ptr) :: fren
       end type
       type(Widgets) :: w
     end subroutine
   end interface

   type(c_ptr) :: window, hbox, vbox
   type(c_ptr) :: c_string

   integer(c_int) :: i
   character(len=128) :: folder

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Exercise 4-1'//CNULL)
   call gtk_container_set_border_width(window, 10)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   w%fren = gtk_button_new_from_stock(GTK_STOCK_APPLY)
   w%fnam = gtk_entry_new()

   call gtk_widget_set_sensitive(w%fren, FALSE)
   call gtk_widget_set_sensitive(w%fnam, FALSE)

  !c_string = g_get_home_dir()
   c_string = g_get_current_dir()
   call c_f_string(c_string,folder)

   w%ffil = gtk_file_chooser_button_new('Choose File'//CNULL, &
                                  GTK_FILE_CHOOSER_ACTION_OPEN)
   i = gtk_file_chooser_set_current_folder(w%ffil, &
                                  folder(1:len_trim(folder))//CNULL)

   call g_signal_connect(w%ffil, 'selection_changed'//CNULL, &
                                  c_funloc(file_changed),c_loc(w))
   call g_signal_connect(w%fren, 'clicked'//CNULL, &
                                  c_funloc(rename_clicked),c_loc(w))

   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5)
   call gtk_box_pack_start(hbox, w%fnam, TRUE,  TRUE, 0)
   call gtk_box_pack_start(hbox, w%fren, FALSE, TRUE, 0)

   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5)
   call gtk_container_set_border_width(vbox, 10)
   call gtk_box_pack_start(vbox, w%ffil, TRUE, TRUE, 0)
   call gtk_box_pack_start(vbox, hbox, TRUE, TRUE, 0)

   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end program
subroutine file_changed(cfile, w) bind(C)

   use iso_c_binding

   use gtk, only: gtk_file_chooser_get_filename, &
                  gtk_widget_set_sensitive, &
                  TRUE, FALSE

   implicit none

   interface
     function g_access(filename,mode) bind(C)
       use iso_c_binding
       integer(c_int) :: g_access
       character(kind=c_char):: filename(*)
       integer(c_int), value    :: mode
     end function
   end interface

   type, bind(c) :: Widgets
     type(c_ptr) :: ffil
     type(c_ptr) :: fnam
     type(c_ptr) :: fren
   end type

   type(Widgets) :: w

   type(c_ptr), value :: cfile

   type(c_ptr)        :: c_string
   character(len=128) :: filename
   integer            :: is

   integer(c_int)     :: mode
   integer(c_int), parameter :: R_OK = 4 ! Test for read permission
   integer(c_int), parameter :: W_OK = 2 ! Test for write permission
   integer(c_int), parameter :: X_OK = 1 ! Test for execute permission
   integer(c_int), parameter :: F_OK = 0 ! Test for existence

   c_string = gtk_file_chooser_get_filename(cfile)
   call c_f_string(c_string,filename)

   is = len_trim(filename)
   mode = g_access(filename(1:is)//C_NULL_CHAR,W_OK)

   if( mode == 0 )then
     write(*,*) filename(1:is),' writeable'
     call gtk_widget_set_sensitive(w%fren, TRUE)
     call gtk_widget_set_sensitive(w%fnam, TRUE)
   endif

end subroutine file_changed
subroutine rename_clicked(button, w) bind(C)

   use iso_c_binding

   use gtk, only: gtk_file_chooser_get_filename, &
                  gtk_file_chooser_get_current_folder, &
                  gtk_entry_get_text, &
                  gtk_widget_set_sensitive, &
                  TRUE, FALSE

   implicit none

   interface
     function g_rename(old,new) bind(C)
       use iso_c_binding
       integer(c_int) :: g_rename
       character(kind=c_char):: old(*), new(*)
     end function
   end interface

   type, bind(c) :: Widgets
     type(c_ptr) :: ffil
     type(c_ptr) :: fnam
     type(c_ptr) :: fren
   end type

   type(Widgets) :: w

   type(c_ptr), value :: button

   type(c_ptr)        :: c_string
   character(len=128) :: old, new, location, text
   integer            :: i

   c_string = gtk_file_chooser_get_filename(w%ffil)
   call c_f_string(c_string,old)

   c_string = gtk_file_chooser_get_current_folder(w%ffil)
   call c_f_string(c_string,location)

   c_string = gtk_entry_get_text(w%fnam)
   call c_f_string(c_string,text)

   new = location(1:len_trim(location))//'/'// &
                                   text(1:len_trim(text))

   i = g_rename(old(1:len_trim(old))//C_NULL_CHAR, &
                new(1:len_trim(new))//C_NULL_CHAR)
   if( i /= 0 ) write(*,*) 'g_rename stat = ',i

   call gtk_widget_set_sensitive(w%fren, FALSE)
   call gtk_widget_set_sensitive(w%fnam, FALSE)

   write(*,*) 'old  > ',old(1:len_trim(old))
   write(*,*) 'new  > ',new(1:len_trim(new))
   write(*,*) 'loc  > ',location(1:len_trim(location))
   write(*,*) 'text > ',text(1:len_trim(text))

end subroutine rename_clicked

