!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program main

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  g_signal_connect, &
                  gtk_widget_destroy, &
                  gtk_adjustment_new, &
                  gtk_spin_button_new, &
                  gtk_scale_new, &
                  gtk_check_button_new_with_label, &
                  gtk_toggle_button_set_active, &
                  gtk_scale_set_digits, &
                  gtk_scale_set_value_pos, &
                  GTK_POS_RIGHT, GTK_POS_LEFT, &
                  GTK_POS_BOTTOM, GTK_POS_TOP, &
                  gtk_box_new, &
                  GTK_ORIENTATION_HORIZONTAL, &
                  GTK_ORIENTATION_VERTICAL, &
                  gtk_box_pack_start, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  TRUE, FALSE

   use g, only:   g_get_home_dir, g_get_current_dir

   implicit none

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   character(len=*), parameter :: GTK_STOCK_APPLY = 'gtk-apply'//CNULL

   type, bind(c) :: Widgets
     type(c_ptr) :: spin
     type(c_ptr) :: skale
     type(c_ptr) :: check
   end type

   type(Widgets), target :: w

   interface
     subroutine destroy(widget,gdata) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget, gdata
     end subroutine
     subroutine value_changed(widget,w) bind(C)
       use iso_c_binding
       type(c_ptr), value :: widget
       type, bind(c) :: Widgets
         type(c_ptr) :: spin
         type(c_ptr) :: skale
         type(c_ptr) :: check
       end type
       type(Widgets) :: w
     end subroutine
   end interface

   type(c_ptr) :: window, vbox, adj1, adj2

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window,'Exercise 4-2'//CNULL)
   call gtk_container_set_border_width(window, 10)

   ! subroutine destroy in tools
   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(destroy))

   adj1 = gtk_adjustment_new(dble( 0.5),dble( 0.0),dble( 1.0), &
                             dble(0.01),dble(0.02),dble( 0.0)  )
   adj2 = gtk_adjustment_new(dble( 0.5),dble( 0.0),dble(1.02), &
                             dble(0.01),dble(0.02),dble( 0.0)  )

   w%spin  = gtk_spin_button_new(adj1, dble(0.01), 2)
   w%skale = gtk_scale_new(GTK_ORIENTATION_HORIZONTAL,adj2)
   w%check = gtk_check_button_new_with_label &
                             ('Synchronize Spin and Scale'//CNULL)

   call gtk_toggle_button_set_active(w%check, TRUE)
   call gtk_scale_set_digits(w%skale, 2)
   call gtk_scale_set_value_pos(w%skale, GTK_POS_RIGHT)

   call g_signal_connect(w%spin,  'value_changed'//CNULL, &
                                  c_funloc(value_changed),c_loc(w))
   call g_signal_connect(w%skale, 'value_changed'//CNULL, &
                                  c_funloc(value_changed),c_loc(w))

   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5)
   call gtk_container_set_border_width(vbox, 10)
   call gtk_box_pack_start(vbox, w%spin,  FALSE, TRUE, 0)
   call gtk_box_pack_start(vbox, w%skale, FALSE, TRUE, 0)
   call gtk_box_pack_start(vbox, w%check, FALSE, TRUE, 0)

   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end program
subroutine value_changed(widget, w) bind(C)

   use iso_c_binding

   use gtk, only: gtk_spin_button_get_value, &
                  gtk_toggle_button_get_active, &
                  gtk_range_get_value, &
                  gtk_range_set_value, &
                  gtk_spin_button_set_value, &
                  TRUE, FALSE

   implicit none

   integer(c_int) :: gtk_is_spin_button

   type, bind(c) :: Widgets
     type(c_ptr) :: spin
     type(c_ptr) :: skale
     type(c_ptr) :: check
   end type

   type(Widgets) :: w
   type(c_ptr)   :: widget
   integer, pointer :: w1, w2 => NULL()

   real(c_double) :: val1, val2
   integer(c_int) :: m1 = 0, m2 = 0, mode = 0

   val1 = gtk_spin_button_get_value(w%spin)
   val2 = gtk_range_get_value(w%skale)

write(*,*)'vals>',val1,val2
   ! synchronize the widget's value based upon the type of "widget"
   m1 = gtk_toggle_button_get_active(w%check)
   if( val1 /= val2 ) m2 = 1

write(*,*) 'mm:',m1,m2,mode
write(*,*)'tezt:',gtk_is_spin_button(widget)

   if( gtk_toggle_button_get_active(w%check) == TRUE .and. &
       val1 /= val2 )then
     if( .true. )then
       write(*,*) 'set range value to',val1
       call gtk_range_set_value(w%skale, val1)
     else
       write(*,*) 'set spin button value to',val2
       call gtk_spin_button_set_value(w%spin, val2)
     endif
   endif

end subroutine value_changed

