!
! Original program by Andrew Krause
! Foundations of GTK+ Development
! Fortran2003 and gtk-fortran version by Henk Krus, 2012
!
program exercise5

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_border_width, &
                  g_signal_connect, &
                  gtk_main_quit, &
                  gtk_button_new_with_label, &
                  gtk_box_new, &
                  gtk_box_pack_start, &
                  GTK_ORIENTATION_VERTICAL, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_main, &
                  gtkTRUE  => TRUE, &
                  gtkFALSE => FALSE

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine save_clicked(parent) bind(c)
       use iso_c_binding
       type(c_ptr), value :: parent
     end
     subroutine create_folder_clicked(parent) bind(c)
       use iso_c_binding
       type(c_ptr), value :: parent
     end
     subroutine open_clicked(parent) bind(c)
       use iso_c_binding
       type(c_ptr), value :: parent
     end
     subroutine select_folder_clicked(parent) bind(c)
       use iso_c_binding
       type(c_ptr), value :: parent
     end
  end interface

   type(c_ptr) :: window, vbox, saveit, createit, openit, selectit

   call gtk_init()

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, 'Exercise 5-1'//CNULL)
   call gtk_container_set_border_width(window, 10)

   call g_signal_connect(window, 'destroy'//CNULL, &
                         c_funloc(gtk_main_quit), C_NULL_PTR)

   saveit   = gtk_button_new_with_label('Save a File'//CNULL)
   createit = gtk_button_new_with_label('Create a New Folder'//CNULL)
   openit   = gtk_button_new_with_label('Open One or More Files'//CNULL)
   selectit = gtk_button_new_with_label('Select a Folder'//CNULL)
  
   call g_signal_connect_swapped(saveit, 'clicked'//CNULL, &
                                 c_funloc(save_clicked), window)
   call g_signal_connect_swapped(createit, 'clicked'//CNULL, &
                                 c_funloc(create_folder_clicked), window)
   call g_signal_connect_swapped(openit, 'clicked'//CNULL, &
                                 c_funloc(open_clicked), window)
   call g_signal_connect_swapped(selectit, 'clicked'//CNULL, &
                                 c_funloc(select_folder_clicked), window)

   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5)
   call gtk_box_pack_start(vbox, saveit,   gtkTRUE, gtkTRUE, 0)
   call gtk_box_pack_start(vbox, createit, gtkTRUE, gtkTRUE, 0)
   call gtk_box_pack_start(vbox, openit,   gtkTRUE, gtkTRUE, 0)
   call gtk_box_pack_start(vbox, selectit, gtkTRUE, gtkTRUE, 0)

   call gtk_container_add(window, vbox)
   call gtk_widget_show_all(window)

   call gtk_main()

end program
subroutine save_clicked(parent) bind(c)

   use iso_c_binding

   use gtk, only: GTK_DIALOG_MODAL, &
                  GTK_RESPONSE_CANCEL, &
                  GTK_RESPONSE_APPLY, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  GTK_FILE_CHOOSER_ACTION_SAVE, &
                  gtk_file_chooser_widget_new, &
                  gtk_dialog_get_content_area, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_dialog_run, &
                  gtk_file_chooser_get_filename, &
                  gtk_widget_destroy
                  
   use g, only:   g_free

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     function gtk_dialog_new_with_buttons(title,parent,flags, &
                         button1,response1,button2,response2,EOD) bind(c)
       use iso_c_binding
       type(c_ptr) :: gtk_dialog_new_with_buttons
       character(kind=c_char), dimension(*) :: title
       type(c_ptr), value                   :: parent
       integer(c_int), value                :: flags
       character(kind=c_char), dimension(*) :: button1, button2
       integer(c_int), value                :: response1, response2
       character(kind=c_char), value :: eod
     end function
   end interface

   type(c_ptr), value :: parent
   type(c_ptr)        :: dialog, chooser, content_area
   integer(c_int)     :: response

   type(c_ptr)        :: c_string
   character(len=128) :: filename

   character(len=*), parameter :: GTK_STOCK_SAVE   = 'gtk-save'//CNULL
   character(len=*), parameter :: GTK_STOCK_CANCEL = 'gtk-cancel'//CNULL

   dialog = gtk_dialog_new_with_buttons('Save File As ...'//CNULL, &
                                        C_NULL_PTR, &
                                        GTK_DIALOG_MODAL, &
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, &
                                        GTK_STOCK_SAVE, GTK_RESPONSE_APPLY, &
                                        C_NULL_CHAR)

   call gtk_container_set_border_width(dialog, 10)
   call gtk_widget_set_size_request(dialog, 600, 400)

   ! create a file chooser dialog that is used to save a file
   chooser = gtk_file_chooser_widget_new(GTK_FILE_CHOOSER_ACTION_SAVE)

   !gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), chooser)
   content_area = gtk_dialog_get_content_area(dialog)
   call gtk_container_add(content_area,chooser)


   call gtk_widget_show_all(dialog)

   response = gtk_dialog_run(dialog)

   if( response == GTK_RESPONSE_APPLY )then
  
     c_string = gtk_file_chooser_get_filename(chooser)
     call c_f_string(c_string_ptr,filename)
     write(*,*) 'Saving file as: ', filename
 
     call g_free(c_string)
  
   endif
   
   call gtk_widget_destroy(dialog)

end
subroutine create_folder_clicked(parent) bind(c)

   use iso_c_binding

   use gtk, only: GTK_DIALOG_MODAL, &
                  GTK_RESPONSE_CANCEL, &
                  GTK_RESPONSE_OK, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER, &
                  gtk_file_chooser_widget_new, &
                  gtk_dialog_get_content_area, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_dialog_run, &
                  gtk_file_chooser_get_filename, &
                  gtk_widget_destroy
                  
   use g, only:   g_free

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     subroutine g_mkdir(filename,mode) bind(c)
       use iso_c_binding
       character(kind=c_char), dimension(*) :: filename
       integer(c_int), value :: mode
     end
     function gtk_dialog_new_with_buttons(title,parent,flags, &
                         button1,response1,button2,response2,EOD) bind(c)
       use iso_c_binding
       type(c_ptr) :: gtk_dialog_new_with_buttons
       character(kind=c_char), dimension(*) :: title
       type(c_ptr), value                   :: parent
       integer(c_int), value                :: flags
       character(kind=c_char), dimension(*) :: button1, button2
       integer(c_int), value                :: response1, response2
       character(kind=c_char), value :: eod
     end function
   end interface

   type(c_ptr), value :: parent
   type(c_ptr)        :: dialog, chooser, content_area
   integer(c_int)     :: response

   type(c_ptr)        :: c_string
   character(len=128) :: filename

   character(len=*), parameter :: GTK_STOCK_OK     = 'gtk-ok'//CNULL
   character(len=*), parameter :: GTK_STOCK_CANCEL = 'gtk-cancel'//CNULL

   dialog = gtk_dialog_new_with_buttons('Create a Folder ...'//CNULL, &
                                        C_NULL_PTR, &
                                        GTK_DIALOG_MODAL, &
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, &
                                        GTK_STOCK_OK, GTK_RESPONSE_OK, &
                                        C_NULL_CHAR)

   call gtk_container_set_border_width(dialog, 10)
   call gtk_widget_set_size_request(dialog, 600, 400)

   ! create a file chooser dialog that is used to save a file
   chooser = gtk_file_chooser_widget_new(GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER)

   content_area = gtk_dialog_get_content_area(dialog)
   call gtk_container_add(content_area,chooser)

   call gtk_widget_show_all(dialog)

   response = gtk_dialog_run(dialog)

   if( response == GTK_RESPONSE_OK )then
  
     c_string = gtk_file_chooser_get_filename(chooser)
     call c_f_string(c_string_ptr,filename)
     
     call g_mkdir(filename, 0777)
     write(*,*) 'Creating directory: %s\n', filename
 
     call g_free(c_string)
  
   endif
   
   call gtk_widget_destroy(dialog)

end
subroutine open_clicked(parent) bind(c)

   use iso_c_binding

   use gtk, only: GTK_DIALOG_MODAL, &
                  GTK_RESPONSE_CANCEL, &
                  GTK_RESPONSE_ACCEPT, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  GTK_FILE_CHOOSER_ACTION_OPEN, &
                  gtk_file_chooser_widget_new, &
                  gtk_file_chooser_set_select_multiple, &
                  gtk_dialog_get_content_area, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_dialog_run, &
                  gtk_file_chooser_get_filenames, &
                  gtk_widget_destroy, &
                  gtkTRUE => TRUE
                  
   use g, only:   g_slist_nth_data, &
                  g_slist_length, &
                  g_slist_foreach, &
                  g_free

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     function gtk_file_chooser_dialog_new (title,parent,flags, &
                         button1,response1,button2,response2,EOD) bind(c)
       use iso_c_binding
       type(c_ptr) :: gtk_file_chooser_dialog_new
       character(kind=c_char), dimension(*) :: title
       type(c_ptr), value                   :: parent
       integer(c_int), value                :: flags
       character(kind=c_char), dimension(*) :: button1, button2
       integer(c_int), value                :: response1, response2
       character(kind=c_char), value :: eod
     end function
   end interface

   type(c_ptr), value :: parent
   type(c_ptr)        :: dialog, chooser, content_area
   integer(c_int)     :: response

   type(c_ptr)        :: gslist_ptr
   type(c_ptr), dimension(:), allocatable :: name_ptrs
   character(len=128) :: filename
   integer(c_int)     :: n
   integer            :: i

   character(len=*), parameter :: GTK_STOCK_OPEN   = 'gtk-open'//CNULL
   character(len=*), parameter :: GTK_STOCK_CANCEL = 'gtk-cancel'//CNULL

   dialog = gtk_file_chooser_dialog_new ('Open File(s) ...'//CNULL, &
                                        C_NULL_PTR, &
                                        GTK_DIALOG_MODAL, &
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, &
                                        GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, &
                                        C_NULL_CHAR)

   call gtk_container_set_border_width(dialog, 10)
   call gtk_widget_set_size_request(dialog, 600, 400)

   ! create a file chooser dialog that is used to multile files
   chooser = gtk_file_chooser_widget_new(GTK_FILE_CHOOSER_ACTION_OPEN)
   call gtk_file_chooser_set_select_multiple(chooser, gtkTRUE)

   !gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), chooser)
   !content_area = gtk_dialog_get_content_area(dialog)
   call gtk_container_add(dialog,chooser)

   call gtk_widget_show_all(dialog)

   response = gtk_dialog_run(dialog)

   if( response == GTK_RESPONSE_ACCEPT )then
  
     gslist_ptr = gtk_file_chooser_get_filenames(chooser)
     n = g_slist_length(gslist_ptr)
     write(*,*)'number of file selected ',n

     allocate(name_ptrs(1:n),stat=istat)
     if( istat /= 0 ) write(*,*) 'error allocate',istat

     do i=1,n
       name_ptrs(i) = g_slist_nth_data(gslist_ptr,i-1)
     enddo    
     
     do i=1,n
       call c_f_string(name_ptrs(i),filename)
       write(*,*) 'Selected: ',trim(filename),i
     enddo
 
     call g_slist_foreach(gslist_ptr, c_funloc(g_free), C_NULL_PTR)
     call g_free(gslist_ptr)
  
   endif
   
   call gtk_widget_destroy(dialog)

end
subroutine select_folder_clicked(parent) bind(c)

   use iso_c_binding

   use gtk, only: GTK_DIALOG_MODAL, &
                  GTK_RESPONSE_CANCEL, &
                  GTK_RESPONSE_OK, &
                  gtk_container_set_border_width, &
                  gtk_widget_set_size_request, &
                  GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, &
                  gtk_file_chooser_widget_new, &
                  gtk_dialog_get_content_area, &
                  gtk_container_add, &
                  gtk_widget_show_all, &
                  gtk_dialog_run, &
                  gtk_file_chooser_get_filename, &
                  gtk_widget_destroy
                  
   use g, only:   g_free

   character(len=1), parameter :: CNULL = C_NULL_CHAR

   interface
     function gtk_dialog_new_with_buttons(title,parent,flags, &
                         button1,response1,button2,response2,EOD) bind(c)
       use iso_c_binding
       type(c_ptr) :: gtk_dialog_new_with_buttons
       character(kind=c_char), dimension(*) :: title
       type(c_ptr), value                   :: parent
       integer(c_int), value                :: flags
       character(kind=c_char), dimension(*) :: button1, button2
       integer(c_int), value                :: response1, response2
       character(kind=c_char), value :: eod
     end function
   end interface

   type(c_ptr), value :: parent
   type(c_ptr)        :: dialog, chooser, content_area
   integer(c_int)     :: response

   type(c_ptr)        :: c_string
   character(len=128) :: filename

   character(len=*), parameter :: GTK_STOCK_OK     = 'gtk-ok'//CNULL
   character(len=*), parameter :: GTK_STOCK_CANCEL = 'gtk-cancel'//CNULL

   dialog = gtk_dialog_new_with_buttons('Select Folder ...'//CNULL, &
                                        C_NULL_PTR, &
                                        GTK_DIALOG_MODAL, &
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, &
                                        GTK_STOCK_OK, GTK_RESPONSE_OK, &
                                        C_NULL_CHAR)

   call gtk_container_set_border_width(dialog, 10)
   call gtk_widget_set_size_request(dialog, 600, 400)

   ! create a file chooser dialog that is used to save a file
   chooser = gtk_file_chooser_widget_new(GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)

   !gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), chooser)
   !content_area = gtk_dialog_get_content_area(dialog)
   call gtk_container_add(dialog,chooser)

   call gtk_widget_show_all(dialog)

   response = gtk_dialog_run(dialog)

   if( response == GTK_RESPONSE_APPLY )then
  
     c_string = gtk_file_chooser_get_filename(chooser)
     call c_f_string(c_string_ptr,filename)
     write(*,*) 'Selected folder: ', filename
 
     call g_free(c_string)
  
   endif
   
   call gtk_widget_destroy(dialog)

end
