!
! 3-D gear wheels.  This program is in the public domain.
!
! Brian Paul
!
! Conversion to GLUT by Mark J. Kilgard
! Conversion to GtkGLExt by Naofumi Yasufuku
! Conversion to Fortran 2003/2008 by Henk Krus
!
! Draw a gear wheel.  You'll probably want to call this function when
! building a display list since we do a lot of trig here.
!
! Input:  inner_radius - radius of hole at center
! outer_radius - radius at center of teeth
! width - width of gear
! teeth - number of teeth
! tooth_depth - depth of tooth
!
module moduul

   use iso_c_binding

   use opengl_gl

   real(GLfloat)   :: view_rotx = 20.0, view_roty = 30.0, view_rotz = 0.0
   integer(GLint)  :: gear1, gear2, gear3
   real(GLfloat)   :: angle = 0.0
 
   integer(GLint)     :: frames  = 0
   integer(GLboolean) :: is_sync = 1

   integer(GLuint)    :: idle_id  = 0

end module moduul
subroutine gear(inner_radius,outer_radius,width,teeth,tooth_depth)
   
   use moduul
   
   real, intent(in)    :: inner_radius,outer_radius,width,tooth_depth
   integer, intent(in) :: teeth
   
   real(GLfloat), parameter :: Pi = 3.141592654
   integer(GLint) :: i
   real(GLfloat)  :: r0, r1, r2, da, u, v, length

   r0 = inner_radius
   r1 = outer_radius - tooth_depth / 2.0
   r2 = outer_radius + tooth_depth / 2.0

   da = 2.0 * Pi / teeth / 4.0

   call glShadeModel(GL_FLAT)

   call glNormal3f(0.0, 0.0, 1.0)

   ! draw front face 
   call glBegin(GL_QUAD_STRIP)
   do i=0,teeth
     angle = i * 2.0 * Pi / teeth
     call glVertex3f(r0*cos(angle), r0*sin(angle), width*0.5)
     call glVertex3f(r1*cos(angle), r1*sin(angle), width*0.5)
     if( i < teeth )then
       call glVertex3f(r0*cos(angle),      r0*sin(angle),      width*0.5)
       call glVertex3f(r1*cos(angle+3*da), r1*sin(angle+3*da), width*0.5)
     endif
   end do
   call glEnd()

   ! draw front sides of teeth
   call glBegin(GL_QUADS)
   da = 2.0 * Pi / teeth / 4.0
   do i=0,teeth-1
     angle = i * 2.0 * Pi / teeth
     call glVertex3f(r1*cos(angle),      r1*sin(angle),      width*0.5)
     call glVertex3f(r2*cos(angle+da),   r2*sin(angle+da),   width*0.5)
     call glVertex3f(r2*cos(angle+2*da), r2*sin(angle+2*da), width*0.5)
     call glVertex3f(r1*cos(angle+3*da), r1*sin(angle+3*da), width*0.5)
   end do
   call glEnd()

   call glNormal3f(0.0, 0.0, -1.0)

   ! draw back face
   call glBegin(GL_QUAD_STRIP)
   do i=0,teeth
     angle = i * 2.0 * Pi / teeth
     call glVertex3f(r1*cos(angle), r1*sin(angle), -width*0.5)
     call glVertex3f(r0*cos(angle), r0*sin(angle), -width*0.5)
     if( i < teeth )then
       call glVertex3f(r1*cos(angle+3*da), r1*sin(angle+3*da), -width*0.5)
       call glVertex3f(r0*cos(angle),      r0*sin(angle),      -width*0.5)
     endif
   end do
   call glEnd()

   ! draw back sides of teeth
   call glBegin(GL_QUADS)
   da = 2.0 * G_PI / teeth / 4.0
   do i=0,teeth-1
     angle = i * 2.0 * Pi / teeth
     call glVertex3f(r1*cos(angle+3*da), r1*sin(angle+3*da), -width*0.5)
     call glVertex3f(r2*cos(angle+2*da), r2*sin(angle+2*da), -width*0.5)
     call glVertex3f(r2*cos(angle+da),   r2*sin(angle+da),   -width*0.5)
     call glVertex3f(r1*cos(angle),      r1*sin(angle),      -width*0.5)
   end do
   call glEnd()

   ! draw outward faces of teeth
   call glBegin(GL_QUAD_STRIP)
   do i=0,teeth-1
     angle = i * 2.0 * Pi / teeth

     call glVertex3f(r1*cos(angle), r1*sin(angle),  width*0.5)
     call glVertex3f(r1*cos(angle), r1*sin(angle), -width*0.5)

     u = r2*cos(angle+da) - r1*cos(angle)
     v = r2*sin(angle+da) - r1*sin(angle)
     length = sqrt(u*u + v*v)
     u = u/length
     v = v/length
     
     call glNormal3f(v, -u, 0.0)
     call glVertex3f(r2*cos(angle+da),   r2*sin(angle+da),    width*0.5)
     call glVertex3f(r2*cos(angle+da),   r2*sin(angle+da),   -width*0.5)
     call glNormal3f(cos(angle), sin(angle), 0.0)
     call glVertex3f(r2*cos(angle+2*da), r2*sin(angle+2*da),  width*0.5)
     call glVertex3f(r2*cos(angle+2*da), r2*sin(angle+2*da), -width*0.5)

     u = r1*cos(angle+3*da) - r2*cos(angle+2*da)
     v = r1*sin(angle+3*da) - r2*sin(angle+2*da)

     call glNormal3f(v, -u, 0.0)
     call glVertex3f(r1*cos(angle+3*da), r1*sin(angle+3*da),  width * 0.5)
     call glVertex3f(r1*cos(angle+3*da), r1*sin(angle+3*da), -width * 0.5)
     call glNormal3f(cos(angle), sin(angle), 0.0)
   enddo

   call glVertex3f(r1*cos(0.0), r1*sin(0.0),  width*0.5)
   call glVertex3f(r1*cos(0.0), r1*sin(0.0), -width*0.5)

   call glEnd()

   call glShadeModel(GL_SMOOTH)

   ! draw inside radius cylinder
   call glBegin(GL_QUAD_STRIP)
   do i=0,teeth
     angle = i * 2.0 * Pi / teeth
     call glNormal3f(-cos(angle), -sin(angle), 0.0)
     call glVertex3f(r0*cos(angle), r0*sin(angle), -width*0.5)
     call glVertex3f(r0*cos(angle), r0*sin(angle),  width*0.5)
   enddo
   call glEnd()

end subroutine gear
integer function draw(widget,event,gdata)

   use moduul

   use gtkgldrawable

   type(c_ptr), value, intent(in) :: widget, event, gdata
   
   type(c_ptr) :: glcontext  
   type(c_ptr) :: gldrawable 

   !
   ! *** OpenGL BEGIN ***
   !
   if( gdk_gl_drawable_gl_begin(gldrawable, glcontext) == 0 )then
     draw = FALSE
     return
   endif

   call glClear(ior(GL_COLOR_BUFFER_BIT,GL_DEPTH_BUFFER_BIT))

   call glPushMatrix()
     call glRotatef(view_rotx, 1.0, 0.0, 0.0)
     call glRotatef(view_roty, 0.0, 1.0, 0.0)
     call glRotatef(view_rotz, 0.0, 0.0, 1.0)

     call glPushMatrix()
       call glTranslatef(-3.0, -2.0, 0.0)
       call glRotatef(angle, 0.0, 0.0, 1.0)
       call glCallList(gear1)
     call glPopMatrix()

     call glPushMatrix()
       call glTranslatef(3.1, -2.0, 0.0)
       call glRotatef(-2.0 * angle - 9.0, 0.0, 0.0, 1.0)
       call glCallList(gear2)
     call glPopMatrix()

     call glPushMatrix()
       call glTranslatef(-3.1, 4.2, 0.0)
       call glRotatef(-2.0 * angle - 25.0, 0.0, 0.0, 1.0)
       call glCallList(gear3)
     call glPopMatrix()

   call glPopMatrix()

   if( gdk_gl_drawable_is_double_buffered(gldrawable) > 0 )then
     call gdk_gl_drawable_swap_buffers(gldrawable)
   else
     call glFlush()
   endif

   call gdk_gl_drawable_gl_end (gldrawable)
   !
   ! *** OpenGL END ***
   !
   frames = frames + 1

  
   ! gdouble seconds = g_timer_elapsed (timer, NULL)
   ! if (seconds >= 5.0) {
   !   gdouble fps = frames / seconds
   !   g_print ("%d frames in %6.3f seconds = %6.3f FPS\n", frames, seconds, fps)
   !    call g_timer_reset (timer)
   !   frames = 0
   !
  

    draw = TRUE

end function draw
integer function resize(widget, event, gdata)
!
!  new window size or exposure */
! 
   use gtkgldrawable, only: gdk_gl_drawable_gl_begin, &
                            gdk_gl_drawable_gl_end 
   use gtkglwidget, only:   gtk_widget_get_gl_context, &
                            gtk_widget_get_gl_window 
   use gtk, only:           gtk_widget_get_allocated_height, &
                            gtk_widget_get_allocated_width 

   use moduul

   type(c_ptr), value, intent(in) :: widget, event, gdata

   type(c_ptr)    :: glcontext, gldrawable, allocation 
   integer(c_int) :: height, width


   !type(c_ptr) :: glcontext
   !type(c_ptr) :: gldrawable
   
   ! GtkAllocation allocation
   glcontext = gtk_widget_get_gl_context(widget)
   gldrawable = gtk_widget_get_gl_window(widget)

   height = gtk_widget_get_allocated_height(widget)
   width  = gtk_widget_get_allocated_width(widget)

   !
   ! *** OpenGL BEGIN ***
   !
   if( gdk_gl_drawable_gl_begin(gldrawable, glcontext) > 0 )then
     resize = FALSE
   endif

   call glViewport(0, 0, width, height)
   call glMatrixMode(GL_PROJECTION)
   call glLoadIdentity()
   call glFrustum(dble(-1.0), dble(1.0), dble(-h), dble(h), dble(5.0), dble(60.0))
   call glMatrixMode(GL_MODELVIEW)
   call glLoadIdentity()
   call glTranslatef(0.0, 0.0, -40.0)

   call gdk_gl_drawable_gl_end(gldrawable)
   !
   ! *** OpenGL END ***
   !
   draw = TRUE

end function resize
subroutine init(widget, gdata)

   use gtkglwidget, only:   gtk_widget_get_gl_context, &
                            gtk_widget_get_gl_window 

   use moduul

   type(c_ptr), value, intent(in) :: widget, gdata
   
   type(c_ptr) :: glcontext 
   type(c_ptr) :: gldrawable 

   real(GLfloat), dimension(4) :: pos   = (/ 5.0, 5.0, 10.0, 0.0 /)
   real(GLfloat), dimension(4) :: red   = (/ 0.8, 0.1, 0.0, 1.0 /)
   real(GLfloat), dimension(4) :: green = (/ 0.0, 0.8, 0.2, 1.0 /)
   real(GLfloat), dimension(4) :: blue  = (/ 0.2, 0.2, 1.0, 1.0 /)

   character(len=512), pointer :: str
   type(C_PTR), target :: cstr

   !
   ! *** OpenGL BEGIN ***
   !
   if( gdk_gl_drawable_gl_begin(gldrawable, glcontext) == 0 )then
     !init = 0
     return
   endif

   call glLightfv(GL_LIGHT0, GL_POSITION, pos)
   call glEnable(GL_CULL_FACE)
   call glEnable(GL_LIGHTING)
   call glEnable(GL_LIGHT0)
   call glEnable(GL_DEPTH_TEST)

   !
   ! make the gears
   !
   gear1 = glGenLists(1)
   call glNewList(gear1, GL_COMPILE)
     call glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, red)
     call gear(1.0, 4.0, 1.0, 20, 0.7)
   call glEndList ()

   gear2 = glGenLists(1)
   call glNewList(gear2, GL_COMPILE)
     call glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, green)
     call gear(0.5, 2.0, 2.0, 10, 0.7)
   call glEndList ()

   gear3 = glGenLists(1)
   call glNewList(gear3, GL_COMPILE)
     call glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, blue)
     call gear(1.3, 2.0, 0.5, 10, 0.7)
   call glEndList()

   call glEnable (GL_NORMALIZE)

    cstr = glGetString(GL_VENDOR)
    call c_f_pointer(cptr=cstr, fptr=str)
    l=min(len(str),index(str,char(0)))
    write(*,*) 'OpenGL vendor is ',str(1:l)

   write(*,*)
   cstr = glGetString(GL_RENDERER)
   call c_f_pointer(cptr=cstr, fptr=str)
   l=min(len(str),index(str,char(0)))
   write(*,*) 'OpenGL RENDERER is ',str(1:l)

   cstr = glGetString(GL_VENDOR)
   call c_f_pointer(cptr=cstr, fptr=str)
   l=min(len(str),index(str,char(0)))
   write(*,*) 'OpenGL VENDOR is ',str(1:l)

   cstr = glGetString(GL_EXTENSIONS)
   call c_f_pointer(cptr=cstr, fptr=str)
   l=min(len(str),index(str,char(0)))
   write(*,*) 'OpenGL EXTENSIONS is ',str(1:l)

   write(*,*)

   call gdk_gl_drawable_gl_end(gldrawable)
   !
   ! *** OpenGL END ***
   !
   
   !
   ! create timer
   !
!   if( timer == NULL )then
!     timer = g_timer_new()
!   endif
!   call g_timer_start (timer)

end subroutine
integer function idle(widget)

   use iso_c_binding

   use gtk, only: gtk_widget_get_window, &
                  gtk_widget_get_allocation, &
                  GTKTRUE => TRUE, &
                  GTKFALSE => FALSE
 
   use gdk, only: gdk_window_invalidate_rect, &
                  gdk_window_process_updates

   type(c_ptr), value, intent(in) :: widget

   type(c_ptr) :: allocation
   type(c_ptr) :: window
  
   angle = angle + 2.0

   window = gtk_widget_get_window(widget)
   call gtk_widget_get_allocation(widget, allocation)

   !
   ! Invalidate the whole window
   !
   call gdk_window_invalidate_rect(window, allocation, GTKFALSE)

   !
   ! Update synchronously (fast)
   !
   if( is_sync )then
     call gdk_window_process_updates(window, GTKFALSE)
   endif
   
   idle = GTKTRUE

end function idle
subroutine idle_add(widget)

   use iso_c_binding

   use moduul, only: idle_id

   use g, only: g_idle_add_full

   type(c_ptr), value, intent(in) :: widget

   integer(c_int), parameter :: G_PRIORITY_HIGH_IDLE = 100
   integer(c_int), parameter :: GDK_PRIORITY_REDRAW  = G_PRIORITY_HIGH_IDLE + 20
 
   if( idle_id == 0 )then
    
     idle_id = g_idle_add_full(GDK_PRIORITY_REDRAW,c_funloc(idle), &
                               widget, C_NULL_FUNPTR)
   endif

end subroutine idle_add
subroutine idle_remove(widget)

   use iso_c_binding
  
   use moduul, only: idle_id

   use g, only: g_source_remove

   type(c_ptr), value, intent(in) :: widget

   if( idle_id /= 0 )then
     i = g_source_remove(idle_id)
     idle_id = 0
   endif

end subroutine idle_remove
integer function map (widget,event,gdata)

   use iso_c_binding

   type(c_ptr), value, intent(in) :: widget, event, gdata

   call idle_add(widget)

   map = 1 !TRUE

end function map
integer function unmap(widget,event,gdata)

   use iso_c_binding

   type(c_ptr), value, intent(in) :: widget, event, gdata

   call idle_remove(widget)

   unmap = 1 !TRUE

end function unmap
integer function visible(widget,event,gdata)

   use iso_c_binding

   use gtk, only: GDK_VISIBILITY_FULLY_OBSCURED 

   use gdk_events

   type(c_ptr), value, intent(in) :: widget, gdata
   type(GdkEventVisibility) :: event

   if( event%state == GDK_VISIBILITY_FULLY_OBSCURED )then
     call idle_remove(widget)
   else
     call idle_add(widget)
   endif

   visible = 1 !TRUE

end function visible
integer function key(widget,event,gdata)
!
!  change view angle, exit upon ESC
!
   use iso_c_binding

   use gtk, only: gtk_widget_get_allocation, &
                  GTKTRUE => TRUE, &
                  GTKFALSE => FALSE
   
   use gdk, only: gdk_window_invalidate_rect

   type(c_ptr), value, intent(in) :: widget, event, gdata

   type(c_ptr) :: allocation
   
   integer(c_int), parameter :: GDK_Key_z      = z'007a'
   integer(c_int), parameter :: GDK_Key_Cz     = z'005a'
   integer(c_int), parameter :: GDK_Key_Up     = z'ff52'
   integer(c_int), parameter :: GDK_Key_Down   = z'ff54'
   integer(c_int), parameter :: GDK_Key_Left   = z'ff51'
   integer(c_int), parameter :: GDK_Key_Right  = z'ff53'
   integer(c_int), parameter :: GDK_Key_Escape = z'ff1b'

   select case(keyval)
     case(GDK_Key_z)
       view_rotz = view_rotz + 5.0
     case(GDK_Key_CZ)
       view_rotz = view_rotz - 5.0
     case(GDK_Key_Up)
       view_rotx = view_rotx + 5.0
     case(GDK_Key_Down)
       view_rotx = view_rotx - 5.0
     case(GDK_Key_Left)
       view_roty = view_roty + 5.0
     case(GDK_Key_Right)
       view_roty = view_roty - 5.0
     case(GDK_Key_Escape)
       call gtk_main_quit()
     case default
       key = GTKFALSE
   end select

   call gtk_widget_get_allocation(widget, allocation)
   call gdk_window_invalidate_rect(widget, allocation, GTKFALSE)

   key = GTKTRUE

end function key
program main

   use iso_c_binding

   use gtk, only: gtk_init, &
                  gtk_window_new, &
                  GTK_WINDOW_TOPLEVEL, &
                  gtk_window_set_title, &
                  gtk_container_set_reallocate_redraws, &
                  gtk_container_add, &
                  g_signal_connect, &
                  gtk_widget_show, &
                  gtk_widget_set_size_request, &
                  gtk_widget_add_events, &
                  gtk_box_pack_start, &
                  gtk_main, &
                  gtk_main_quit, &
                  GTKTRUE => TRUE, &
                  GTKFALSE => FALSE


   use gtkglwidget, only: gdk_gl_config_new_by_mode

   use gtkgldrawable, only: gtk_gl_init, &
                  gtk_widget_set_gl_capability

   use gdk, only: gdk_gl_config_new_by_mode, &
                  GDK_GL_MODE_RGB, &
                  GDK_GL_MODE_DEPTH, &
                  GDK_GL_MODE_DOUBLE, &  
                  GDK_GL_RGBA_TYPE, &
                  GDK_VISIBILITY_NOTIFY_MASK

   use moduul
 
   type(c_ptr) :: glconfig, window, vbox, drawing_area, button
   integer     :: i

   !
   ! Init GTK
   !
   call gtk_init()

   !
   ! Init GtkGLExt
   !
   i = gtk_gl_init()

   !
   ! Command line options
   !
   !for (i = 0 i < argc i++)
   ! {
   !   if (strcmp (argv[i], "--async") == 0)
   !     is_sync = FALSE
   ! }

   !
   ! Configure OpenGL-capable visual
   !
   !
   ! Try double-buffered visual
   !
   glconfig = gdk_gl_config_new_by_mode(ior(ior(GDK_GL_MODE_RGB, &
                                                GDK_GL_MODE_DEPTH, &
                                                GDK_GL_MODE_DOUBLE)))
   if( glconfig == 0 )then
     write(*,*) "*** Cannot find the double-buffered visual"
     write(*,*) "*** Trying single-buffered visual"

     !
     ! Try single-buffered visual
     !
     glconfig = gdk_gl_config_new_by_mode(ior(GDK_GL_MODE_RGB, &
                                              GDK_GL_MODE_DEPTH))
     if( glconfig == 0 )then
       write(*,*) "*** No appropriate OpenGL-capable visual found"
       stop
     endif
   endif

   !
   ! Top-level window
   !
   window = gtk_window_new(GTK_WINDOW_TOPLEVEL)
   call gtk_window_set_title(window, "gears")

   !
   ! Get automatically redrawn if any of their children changed allocation
   !
   call gtk_container_set_reallocate_redraws(window, TRUE)

   call g_signal_connect(window, "delete_event", c_funloc(gtk_main_quit), C_NULL_CHAR)

   !
   ! VBox
   !
   vbox = gtk_vbox_new(FALSE, 0)
   call gtk_container_add(window, vbox)
   call gtk_widget_show(vbox)

   !
   ! Drawing area for drawing OpenGL scene
   !
   drawing_area = gtk_drawing_area_new()
   call gtk_widget_set_size_request(drawing_area, 300, 300)

   !
   ! Set OpenGL-capability to the widget
   !
   call gtk_widget_set_gl_capability(drawing_area, &
				                     glconfig, &
                                     c_null_ptr, &
				                     TRUE, &
				                     GDK_GL_RGBA_TYPE)

   call gtk_widget_add_events (drawing_area, GDK_VISIBILITY_NOTIFY_MASK)

   call g_signal_connect_after(drawing_area, "realize",c_funloc(init), c_null_ptr)
   call g_signal_connect(drawing_area, "configure_event",c_funloc(resize), c_null_ptr)
   call g_signal_connect(drawing_area, "expose_event",c_funloc(draw), c_null_ptr)
   call g_signal_connect(drawing_area, "map_event",c_funloc(map), c_null_ptr)
   call g_signal_connect(drawing_area, "unmap_event",c_funloc(unmap), c_null_ptr)
   call g_signal_connect(drawing_area, "visibility_notify_event",c_funloc(visible), c_null_ptr)

   call g_signal_connect_swapped(window, "key_press_event", &
			    c_funloc(key), drawing_area)

   call gtk_box_pack_start(vbox, drawing_area, TRUE, TRUE, 0)

   call gtk_widget_show(drawing_area)

   !
   ! Simple quit button
   !
   button = gtk_button_new_with_label("Quit")

   call g_signal_connect(button, "clicked",G_CALLBACK(gtk_main_quit), NULL)

   call gtk_box_pack_start(vbox, button, FALSE, FALSE, 0)

   call gtk_widget_show(button)

   !
   ! Show window
   !
   call gtk_widget_show(window)

   !
   ! Main loop
   !
   !cal idle_add(drawing_area)
   call gtk_main()

end


